package com.factory;

import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.core.type.classreading.CachingMetadataReaderFactory;
import org.springframework.core.type.classreading.MetadataReader;
import org.springframework.core.type.classreading.MetadataReaderFactory;
import org.springframework.util.ClassUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.IOException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

public class GetApiTest {
    private final static String BASE_PACKAGE = "cn.jiheng.factory.application.web.interfaces.facade.andon";
    private final static String RESOURCE_PATTERN = "/**/*.class";

    public static void main(String[] args) {
        List<String> pathList = new ArrayList<>();

        //spring工具类，可以获取指定路径下的全部类
        ResourcePatternResolver resourcePatternResolver = new PathMatchingResourcePatternResolver();
        try {
            String pattern = ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX +
                    ClassUtils.convertClassNameToResourcePath(BASE_PACKAGE) + RESOURCE_PATTERN;

            Resource[] resources = resourcePatternResolver.getResources(pattern);

            //MetadataReader 的工厂类
            MetadataReaderFactory readerfactory = new CachingMetadataReaderFactory(resourcePatternResolver);
            for (Resource resource : resources) {
                //用于读取类信息
                MetadataReader reader = readerfactory.getMetadataReader(resource);
                //扫描到的class
                String classname = reader.getClassMetadata().getClassName();
                Class<?> clazz = Class.forName(classname);

                //判断是否有指定主解
                RestController anno1 = clazz.getAnnotation(RestController.class);
                if (anno1 == null) {
                    continue;
                }
                String path = "";
                RequestMapping anno2 = clazz.getAnnotation(RequestMapping.class);
                if (anno2 != null) {
                    String[] value = anno2.value();
                    path = value[0];
                }
                if (path.length() > 0 && !path.startsWith("/")) {
                    path = "/" + path;
                }

                //将注解中的类型值作为key，对应的类作为 value
                Method[] methods = clazz.getMethods();
                for (Method method : methods) {
                    RequestMapping annotation = method.getAnnotation(RequestMapping.class);
                    GetMapping ag = method.getAnnotation(GetMapping.class);
                    PostMapping ap = method.getAnnotation(PostMapping.class);
                    if (annotation != null) {
                        String v = annotation.value()[0];
                        if (!v.startsWith("/")) {
                            v = "/" + v;
                        }
                        pathList.add(path + v);
                    }
                    if (ag != null) {
                        String v = ag.value()[0];
                        if (!v.startsWith("/")) {
                            v = "/" + v;
                        }
                        pathList.add(path + v);
                    }
                    if (ap != null) {
                        String v = ap.value()[0];
                        if (!v.startsWith("/")) {
                            v = "/" + v;
                        }
                        pathList.add(path + v);
                    }
                }

            }
        } catch (IOException | ClassNotFoundException e) {
        }
        for (String path : pathList) {
            System.out.println("/web" + path);
        }
    }

}