package com.factory.controller.web;


import com.factory.base.config.validateGroup.GroupOne;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.vo.req.BaseIdReq;
import com.factory.base.entity.vo.req.BasePageIdReq;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.entity.vo.res.FileRes;
import com.factory.web.entity.vo.req.web.*;
import com.factory.web.entity.vo.res.web.ini.*;
import com.factory.web.service.ini.BentelerIniVersionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiOperationSupport;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 计划 - 检验报告单
 *
 * @author Dl
 * @date 2021-05-12 09:37:25
 */
@Api(tags = "WEB - 首件")
@RestController
@RequestMapping("/benteler/web")
@Slf4j
public class InitialPlanWebController {
    @Autowired
    private BentelerIniVersionService bentelerIniVersionService;

    @ApiOperation(value = "新建首件计划")
    @PostMapping("/addIniPlan")
    @ApiOperationSupport(order = 1)
    public ResponseBean addIniPlan(@RequestBody AddIniPlanReq req) {
        return ResponseBeanBuilder.ok(bentelerIniVersionService.addIniPlan(req));
    }

    @ApiOperation(value = "首件计划列表")
    @ApiOperationSupport(order = 2)
    @PostMapping("/planlist")
    public ResponseBean<PageBean<PagePlanRes>> planlist(@RequestBody PlanListReq req) {
        return bentelerIniVersionService.pagePlan_web(req);
    }

    @ApiOperation(value = "禁用计划")
    @PostMapping("/disablePlan")
    @ApiOperationSupport(order = 3)
    public ResponseBean disablePlan(@RequestBody BooleanReq req) {
        return bentelerIniVersionService.disablePlan(req);
    }

    @ApiOperation(value = "待审核列表")
    @PostMapping("/unAuditList")
    @ApiOperationSupport(order = 4)
    public ResponseBean<PageBean<unAuditListRes>> unAuditList(@RequestBody PlanListReq req) {
        return bentelerIniVersionService.unAuditList(req);
    }

    @ApiOperation(value = "查看版本表头")
    @PostMapping("/getVersionBase")
    @ApiOperationSupport(order = 5)
    public ResponseBean<GetVersionBaseRes> getVersionBase(@RequestBody BaseIdReq req) {
        return bentelerIniVersionService.getVersionBase(req);
    }

    @ApiOperation(value = "计划列表 - 编辑(新增版本)")
    @PostMapping("/addIniVersion")
    @ApiOperationSupport(order = 6)
    public ResponseBean addIniVersion(@RequestBody AddIniPlanReq req) {
        return bentelerIniVersionService.addIniVersion(req);
    }

    @ApiOperation(value = "查询是否存在其他待审核")
    @PostMapping("/getIsUnAuditVersionId")
    @ApiOperationSupport(order = 7)
    public ResponseBean<Boolean> getIsUnAuditVersionId(@RequestBody BaseIdReq req) {
        return bentelerIniVersionService.getIsUnAuditVersionId(req);
    }

    @ApiOperation(value = "查看版本 - 待审核标准详情 计划id")
    @PostMapping("/getUnAuditDetails")
    public ResponseBean<AddIniPlanReq> getUnAuditDetails(@RequestBody BaseIdReq req) {
        return bentelerIniVersionService.getUnAuditDetails(req);
    }

    @ApiOperation(value = "查看版本 - 正在使用标准详情")
    @PostMapping("/getCurrentDetails")
    public ResponseBean<AddIniPlanReq> getCurrentDetails(@RequestBody BaseIdReq req) {
        return bentelerIniVersionService.getCurrentDetails(req);
    }

    @ApiOperation(value = "查看点检  - 结果详情")
    @PostMapping("/getCheckResutGread")
    public ResponseBean<List<getCheckResutGreadRes>> getCheckResutGread(@RequestBody getCheckResutGreadReq req) {
        return ResponseBeanBuilder.ok(bentelerIniVersionService.getCheckResutGread(req));
    }

    @ApiOperation(value = "历史版本简介")
    @PostMapping("/historyVersion")
    public ResponseBean<PageBean<GetVersionHistoryRes>> historyVersion(@Validated(GroupOne.class) @RequestBody BasePageIdReq req) {
        return bentelerIniVersionService.historyVersion(req);
    }

    @ApiOperation(value = "首件计划导出excel")
    @PostMapping("/exportInitialPlan")
    public ResponseBean<FileRes> exportInitialPlan(@RequestBody ExportInitialPlanReq req) {
        return bentelerIniVersionService.exportInitialPlan(req);
    }

}
