package com.factory.controller.web;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.pm.OnsiteApprovalPlan;
import com.factory.web.entity.pm.OnsiteCalendar;
import com.factory.web.entity.pm.OnsiteItem;
import com.factory.web.entity.pm.OnsiteTitle;
import com.factory.web.entity.pm.OnsiteWorkshop;
import com.factory.web.entity.pm.req.AddItemResultListReq;
import com.factory.web.entity.pm.req.AddPmPlanReq;
import com.factory.web.entity.pm.req.EditItemResultListReq;
import com.factory.web.entity.pm.req.EditPmPlanReq;
import com.factory.web.entity.pm.req.PmPlanPauseFlagReq;
import com.factory.web.entity.pm.req.SearchAbnormalListReq;
import com.factory.web.entity.pm.req.SearchAllWorkshopReq;
import com.factory.web.entity.pm.req.SearchItemListReq;
import com.factory.web.entity.pm.req.SearchPmAllCheckFlagReq;
import com.factory.web.entity.pm.req.SearchPmCheckRecordListReq;
import com.factory.web.entity.pm.req.SearchPmItemNoGoListReq;
import com.factory.web.entity.pm.req.SearchPmPlanCheckListReq;
import com.factory.web.entity.pm.req.SearchPmPlanListForPadReq;
import com.factory.web.entity.pm.req.SearchPmPlanListReq;
import com.factory.web.entity.pm.req.SearchPmPlanReq;
import com.factory.web.entity.pm.req.SearchPmWorkshopNoGoListReq;
import com.factory.web.entity.pm.req.SearchTaskReq;
import com.factory.web.entity.pm.req.SearchWorkshopReq;
import com.factory.web.entity.pm.res.AbnormalTaskInfoRes;
import com.factory.web.entity.pm.res.PmAllWorkshopListRes;
import com.factory.web.entity.pm.res.PmCheckListRes;
import com.factory.web.entity.pm.res.PmCheckRecordListRes;
import com.factory.web.entity.pm.res.PmItemNoGoListRes;
import com.factory.web.entity.pm.res.PmNoGoTaskAapprovalRes;
import com.factory.web.entity.pm.res.PmPlanCheckListRes;
import com.factory.web.entity.pm.res.PmPlanListForPadRes;
import com.factory.web.entity.pm.res.PmPlanListRes;
import com.factory.web.entity.pm.res.PmPlanRes;
import com.factory.web.entity.pm.res.PmWorkshopNoGoListRes;
import com.factory.web.service.pm.PreventMistakeService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiOperationSupport;
import lombok.extern.slf4j.Slf4j;

/**
 * 防错计划
 *
 * @author wzy
 * @date 2021-07-09 12:00:00
 */
@Api(tags = "防错")
@RestController
@RequestMapping("/benteler/pm")
@Slf4j
public class PreventMistakeController {

	@Autowired
	private PreventMistakeService preventMistakeService;

	@ApiOperation(value = "新建防错计划")
	@PostMapping("/addPmPlan")
	@ApiOperationSupport(order = 1)
	public ResponseBean<Void> addPmPlan(@RequestBody AddPmPlanReq req) {
		return preventMistakeService.addPmPlan(req);
	}

	@ApiOperation(value = "取得Web端防错计划点检列表")
	@ApiOperationSupport(order = 2)
	@PostMapping("/getPmPlanCheckList")
	public ResponseBean<PageBean<PmPlanCheckListRes>> getPmPlanCheckList(@RequestBody SearchPmPlanCheckListReq req) {
		return preventMistakeService.getPmPlanCheckList(req);
	}

	//,"getWaitApprovalPlanList"
	@ApiOperation(value = "取得Web端待审批计划列表")
	@ApiOperationSupport(order = 3)
	@PostMapping({"/getPmPlanlist"})
	public ResponseBean<PageBean<PmPlanListRes>> getPmPlanlist(@RequestBody SearchPmPlanListReq req) {
		return preventMistakeService.getPmPlanList(req);
	}

	@ApiOperation(value = "取得防错计划信息:用于PC待审核列表编辑")
	@ApiOperationSupport(order = 4)
	@PostMapping("/getPmPlan")
	public ResponseBean<PmPlanRes> getPmPlan(@RequestBody SearchPmPlanReq req) {
		PmPlanRes pmPlanRes = preventMistakeService.getPmPlan(req);
		return ResponseBeanBuilder.ok(pmPlanRes);
	}

	@ApiOperation(value = "禁用解禁防错计划")
	@PostMapping("/disablePmPlan")
	@ApiOperationSupport(order = 5)
	public ResponseBean<Void> disablePmPlan(@RequestBody PmPlanPauseFlagReq req) {
		return preventMistakeService.disablePmPlan(req);
	}

	@ApiOperation(value = "取得工作站及站下巡检项目信息")
	@ApiOperationSupport(order = 6)
	@PostMapping("/getWorkshop")
	public ResponseBean<OnsiteWorkshop> getWorkshop(@RequestBody SearchWorkshopReq req) {
		return preventMistakeService.getWorkshop(req);
	}

	@ApiOperation(value = "修改防错计划")
	@PostMapping("/editPmPlan")
	@ApiOperationSupport(order = 7)
	public ResponseBean<Void> editPmPlan(@RequestBody EditPmPlanReq req) {
		return preventMistakeService.editPmPlan(req);
	}

	@ApiOperation(value = "取得计划审批信息")
	@ApiOperationSupport(order = 8)
	@PostMapping("/getPmPlanApprovalList")
	public ResponseBean<List<OnsiteApprovalPlan>> getPmPlanApprovalList(@RequestBody SearchPmPlanReq req) {
		return preventMistakeService.getPmPlanApprovalList(req);
	}

	@ApiOperation(value = "取得每频次检查信息")
	@ApiOperationSupport(order = 9)
	@PostMapping("/getPmCheckList")
	public ResponseBean<List<PmCheckListRes>> getPmCheckList(@RequestBody SearchPmPlanReq req) {
		return preventMistakeService.getPmCheckList(req);
	}

	@ApiOperation(value = "取得巡检规则信息")
	@ApiOperationSupport(order = 10)
	@PostMapping("/getCalendarList")
	public ResponseBean<List<OnsiteCalendar>> getCalendarList(@RequestBody SearchPmPlanReq req) {
		return preventMistakeService.getCalendarList(req);
	}

	@ApiOperation(value = "检查记录列表")
	@ApiOperationSupport(order = 11)
	@PostMapping("/getPmCheckRecordList")
	public ResponseBean<List<PmCheckRecordListRes>> getPmCheckRecordList(@RequestBody SearchPmCheckRecordListReq req) {
		return preventMistakeService.getPmCheckRecordList(req);
	}

	@ApiOperation(value = "取得Pad端防错计划列表")
	@ApiOperationSupport(order = 12)
	@PostMapping("/getPmPlanListForPad")
	public ResponseBean<List<PmPlanListForPadRes>> getPmPlanListForPad(@RequestBody SearchPmPlanListForPadReq req) {
		return preventMistakeService.getPmPlanListForPad(req);
	}

	@ApiOperation(value = "通过计划ID取得表头")
	@ApiOperationSupport(order = 13)
	@PostMapping("/getTitle")
	public ResponseBean<OnsiteTitle> getTitle(@RequestBody SearchPmPlanReq req) {
		OnsiteTitle onsiteTitle = preventMistakeService.getTitle(req.getBentelerPlanId());
		return ResponseBeanBuilder.ok(onsiteTitle);
	}

	@ApiOperation(value = "通过计划ID取得工作站列表")
	@ApiOperationSupport(order = 14)
	@PostMapping("/getWorkshopList")
	public ResponseBean<List<OnsiteWorkshop>> getWorkshopList(@RequestBody SearchPmAllCheckFlagReq req) {
		return preventMistakeService.getWorkshopList(req);
	}

	@ApiOperation(value = "通过工作站ID取得巡检项目列表(包括已保存检查结果)")
	@ApiOperationSupport(order = 15)
	@PostMapping("/getItemList")
	public ResponseBean<List<OnsiteItem>> getItemList(@RequestBody SearchItemListReq req) {
		return preventMistakeService.getItemList(req);
	}

	@ApiOperation(value = "添加巡检项目检查结果")
	@PostMapping("/addItemResult")
	@ApiOperationSupport(order = 16)
	public ResponseBean<Void> addItemResult(@RequestBody AddItemResultListReq req) {
		return preventMistakeService.addItemResult(req);
	}

	@ApiOperation(value = "任务检查完后提交")
	@PostMapping("/submitTask")
	@ApiOperationSupport(order = 17)
	public ResponseBean<Void> submitTask(@RequestBody SearchTaskReq req) {
		return preventMistakeService.submitTask(req);
	}

	@ApiOperation(value = "取得不放行任务对应的工作站")
	@ApiOperationSupport(order = 18)
	@PostMapping("/getPmWorkshopNoGoList")
	public ResponseBean<List<PmWorkshopNoGoListRes>> getPmWorkshopNoGoList(@RequestBody SearchPmWorkshopNoGoListReq req) {
		return preventMistakeService.getPmWorkshopNoGoList(req);
	}

	@ApiOperation(value = "取得不放行任务对应的点检项目")
	@ApiOperationSupport(order = 19)
	@PostMapping("/getPmItemNoGoList")
	public ResponseBean<List<PmItemNoGoListRes>> getPmItemNoGoList(@RequestBody SearchPmItemNoGoListReq req) {
		return preventMistakeService.getPmItemNoGoList(req);
	}
	
	@ApiOperation(value = "取得不放行任务对应的质量经理审批信息")
	@ApiOperationSupport(order = 20)
	@PostMapping("/getPmNoGoTaskAapproval")
	public ResponseBean<PmNoGoTaskAapprovalRes> getPmNoGoTaskAapproval(@RequestBody SearchTaskReq req) {
		return preventMistakeService.getPmNoGoTaskAapproval(req);
	}
	
	@ApiOperation(value = "修改巡检项目检查结果")
	@PostMapping("/editItemResult")
	@ApiOperationSupport(order = 21)
	public ResponseBean<Void> editItemResult(@RequestBody EditItemResultListReq req) {
		return preventMistakeService.editItemResult(req);
	}
	
	@ApiOperation(value = "取得所有工作站")
	@PostMapping("/getAllWorkshopList")
	@ApiOperationSupport(order = 22)
	public ResponseBean<List<PmAllWorkshopListRes>> getAllWorkshopList(@RequestBody SearchAllWorkshopReq req) {
		return preventMistakeService.getAllWorkshopList(req);
	}
	
//	@ApiOperation(value = "判断检验结果是否填写完成")
//	@PostMapping("/getPmAllCheckFlag")
//	@ApiOperationSupport(order = 23)
//	public ResponseBean<PmAllCheckFlagRes> getPmAllCheckFlag(@RequestBody SearchPmAllCheckFlagReq req) {
//		return ResponseBeanBuilder.ok(preventMistakeService.getPmAllCheckFlag(req));
//	}

    @ApiOperation(value = "取得异常列表")
    @PostMapping("/getAbnormalList")
    @ApiOperationSupport(order = 24)
    public ResponseBean<List<AbnormalTaskInfoRes>> getAbnormalList(@RequestBody SearchAbnormalListReq req) {
        return preventMistakeService.getAbnormalList(req);
    }

    @ApiOperation(value = "重检前清空之前的点检结果")
    @PostMapping("/setPmRepeatResult")
    @ApiOperationSupport(order = 25)
    public ResponseBean<Void> setPmRepeatResult(@RequestBody SearchTaskReq req) {
        preventMistakeService.setPmRepeatResult(req.getOnsiteTaskId());
        return ResponseBeanBuilder.ok();
    }

    // 计划审批、任务审批，都在小程序端做
}
