package com.factory.controller.wx;

import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.vo.res.unfinishedNum;
import com.factory.wx.entity.req.PlanApprovalReq;
import com.factory.wx.entity.req.PlanInfoReq;
import com.factory.wx.entity.req.PlanPageReq;
import com.factory.wx.entity.req.WxUnfinishedNumReq;
import com.factory.wx.entity.res.PlanApprovalDetailRes;
import com.factory.wx.entity.res.PlanPassRes;
import com.factory.wx.entity.res.ReminderRes;
import com.factory.wx.service.PlanApprovalService;
import com.factory.wx.service.PlanPassService;
import com.factory.wx.service.PlanReminderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 微信小程序端计划
 *
 * @author sMArT
 * @version 1.0
 * @date 2021-07-05 3:06 下午
 */
@Api(tags = "微信小程序端")
@RestController
@RequestMapping("/plans/approval")
@Slf4j
public class PlanApprovalController {

    @Autowired
    private PlanApprovalService planApprovalService;
    @Autowired
    private PlanPassService planPassService;
    @Autowired
    private PlanReminderService planReminderService;

    @ApiOperation(value = "计划审批列表")
    @PostMapping("/list")
    public ResponseBean<PageBean<BentelerPlan>> approvalList(@RequestBody PlanPageReq planPageReq) {
        PageBean<BentelerPlan> approvalList = planApprovalService.getApprovalListByPage(planPageReq);
        return ResponseBeanBuilder.ok(approvalList);
    }

    @ApiOperation(value = "计划审批详情-首件")
    @PostMapping("/detail/initial")
    public ResponseBean<PlanApprovalDetailRes> approvalDetail(@RequestBody PlanInfoReq planInfoReq) {
        PlanApprovalDetailRes planApprovalDetailRes = planApprovalService.getApprovalFADetail(planInfoReq);
        return ResponseBeanBuilder.ok(planApprovalDetailRes);
    }

    @ApiOperation(value = "计划审批详情-防错")
    @PostMapping("/detail/pm")
    public ResponseBean<PlanApprovalDetailRes> approvalDetailPM(@RequestBody PlanInfoReq planInfoReq) {
        PlanApprovalDetailRes planApprovalDetailRes = planApprovalService.getApprovalPMDetail(planInfoReq);
        return ResponseBeanBuilder.ok(planApprovalDetailRes);
    }


    @ApiOperation(value = "计划审批")
    @PostMapping("/operate")
    public ResponseBean<Void> approvalOperate(@RequestBody PlanApprovalReq planApprovalReq) {
        return planApprovalService.approvalOperate(planApprovalReq);
    }

    @ApiOperation(value = "菜单的数量提醒  相当于统计各个1层级列表（计划审批，计划放行，计划提醒） 的数量")
    @PostMapping("/unfinishedNum")
    public ResponseBean<unfinishedNum> unfinishedNum(@RequestBody WxUnfinishedNumReq wxUnfinishedNumReq) {
        PlanPageReq build0 = PlanPageReq.builder().userId(wxUnfinishedNumReq.getUserId()).page(1).size(999).checkType(0L).build();
        PlanPageReq build1 = PlanPageReq.builder().userId(wxUnfinishedNumReq.getUserId()).page(1).size(999).checkType(1L).build();
        PlanPageReq build2 = PlanPageReq.builder().userId(wxUnfinishedNumReq.getUserId()).page(1).size(999).checkType(2L).build();

        //计划审批
        Long totalfir = planApprovalService.getApprovalListByPage(build0).getTotal();
        //计划放行
        int size = planPassService.getPassList(build1).size();
        int size1 = planPassService.getPassList(build2).size();
        //计划提醒
        long count = planReminderService.getReminderList(build0).stream().filter(item -> item.getStatus() == 0).count();

        return ResponseBeanBuilder.ok(unfinishedNum.builder().approval(totalfir.intValue()).pass(size + size1).remind(((int) count)).build());
    }

}
