package com.factory.controller.wx;

import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.wx.entity.req.PlanInfoReq;
import com.factory.wx.entity.req.PlanPageReq;
import com.factory.wx.entity.res.PlanReminderDetailRes;
import com.factory.wx.entity.res.ReminderRes;
import com.factory.wx.service.PlanReminderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 微信小程序端计划
 *
 * @author sMArT
 * @version 1.0
 * @date 2021-07-05 3:06 下午
 */
@Api(tags = "微信小程序端")
@RestController
@RequestMapping("/plans/reminder")
@Slf4j
public class PlanReminderController {

    @Autowired
    private PlanReminderService planReminderService;

    @ApiOperation(value = "计划提醒列表")
    @PostMapping("/list")
    public ResponseBean<Map<String, Object>> reminderList(@RequestBody PlanPageReq planPageReq) {
        List<ReminderRes> reminderList = planReminderService.getReminderList(planPageReq);
        Map<String, Object> map = new HashMap<>();
        map.put("row", reminderList);
        return ResponseBeanBuilder.ok(map);
    }

    @ApiOperation(value = "首件计划提醒详情")
    @PostMapping("/detail/initial")
    public ResponseBean<PlanReminderDetailRes> reminderDetail(@RequestBody PlanInfoReq planInfoReq) {
        PlanReminderDetailRes planReminderDetailRes = planReminderService.getReminderDetail(planInfoReq);
        return ResponseBeanBuilder.ok(planReminderDetailRes);
    }

    @ApiOperation(value = "防错计划提醒详情")
    @PostMapping("/detail/pm")
    public ResponseBean<PlanReminderDetailRes> reminderDetailPM(@RequestBody PlanInfoReq planInfoReq) {
        PlanReminderDetailRes planReminderDetailRes = planReminderService.getReminderDetailPM(planInfoReq);
        return ResponseBeanBuilder.ok(planReminderDetailRes);
    }

    @ApiOperation(value = "计划提醒确认")
    @PostMapping("/operate")
    public ResponseBean<Void> passOperate(@RequestBody PlanInfoReq planInfoReq) {
        int r = planReminderService.confirmReminder(planInfoReq);
        if(r==0) {
        	return ResponseBeanBuilder.fail("无权限或已经提醒");
        }
        return ResponseBeanBuilder.ok();
    }

}
