package com.factory.controller.wx;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.web.entity.statistics.ExceptionCountRes;
import com.factory.web.entity.statistics.StatisticsDateReq;
import com.factory.web.service.statistics.StatisticsService;
import com.factory.wx.entity.req.AbnormalDetailReq;
import com.factory.wx.entity.req.AbnormalPlanReq;
import com.factory.wx.entity.res.AbnormalDetailRes;
import com.factory.wx.entity.res.AbnormalPlanRes;
import com.factory.wx.service.AbnormalStatisticsService;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 微信小程序端异常统计
 *
 * @author sMArT
 * @version 1.0
 * @date 2021-07-05 3:55 下午
 */
@Api(tags = "微信小程序端")
@RestController
@RequestMapping("/statistics")
public class StatisticsController {
    @Autowired
    private StatisticsService statisticsService;
    @Autowired
    private AbnormalStatisticsService abnormalStatisticsService;

    @ApiOperation(value = "异常统计")
    @PostMapping("/abnormal")
    public ResponseBean<ExceptionCountRes> abnormal(@RequestBody StatisticsDateReq statisticsDateReq) {
        return statisticsService.getExceptionsByMonth(statisticsDateReq);
    }

    @ApiOperation(value = "异常计划列表")
    @PostMapping("/list")
    public ResponseBean<Map<String,Object>> statisticsList(@RequestBody AbnormalPlanReq abnormalPlanReq) {
        List<AbnormalPlanRes> abnormalPlanResList = abnormalStatisticsService.getExceptionPlansByMonth(abnormalPlanReq);
        Map<String,Object> map = new HashMap<>();
        map.put("row", abnormalPlanResList);
        return ResponseBeanBuilder.ok(map);
    }

    @ApiOperation(value = "异常计划详情(按照计划显示列表)")
    @PostMapping("/detail")
    public ResponseBean<AbnormalDetailRes> abnormalDetail(@RequestBody AbnormalDetailReq abnormalDetailReq) {
        AbnormalDetailRes abnormalDetailRes = abnormalStatisticsService.getAbnormalDetail(abnormalDetailReq);
        return ResponseBeanBuilder.ok(abnormalDetailRes);
    }

    @ApiOperation(value = "异常重检计划详情：暂无使用空")
    @PostMapping("/detail/pm")
    public ResponseBean<AbnormalDetailRes> abnormalDetailPM(@RequestBody AbnormalDetailReq abnormalDetailReq) {
        return ResponseBeanBuilder.ok();
    }
}
