package com.factory.controller.wx;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.task.HttpCilentUntil;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.wx.entity.req.UserBindReq;
import com.factory.wx.entity.res.FunctionsRes;
import com.factory.wx.service.WxService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 小程序功能显示
 *
 * @author sMArT
 * @version 1.0
 * @date 2021-07-11 2:35 下午
 */
@Api(tags = "微信小程序端")
@RestController
@RequestMapping("/functions")
@Slf4j
public class WxController {

    @Value("${wx.minprogram.appsecret}")
    private String appsecret;

    @Value("${wx.minprogram.appid}")
    private String appid;

    @Autowired
    private WxService wxService;

    @ApiOperation(value = "用户功能列表")
    @GetMapping("/list")
    public ResponseBean<List<FunctionsRes>> getUserFunctions(HttpServletRequest request) {
        Long userId = (Long) request.getAttribute("LOGIN_USER_ID");//获取用户id
        return wxService.getUserFunction(userId);
    }


    @ApiOperation(value = "小程序获取openid")
    @PostMapping("/bind")
    public ResponseBean bindWechatUser(@RequestBody UserBindReq userBindReq, HttpServletResponse response, HttpServletRequest request) throws Throwable {
        try {
            response.setContentType("text/html");
            request.setCharacterEncoding("UTF-8");
            response.setCharacterEncoding("UTF-8");
            Map params = new HashMap();
            params.put("secret", appsecret);
            params.put("appid", appid);
            params.put("grant_type", "authorization_code");
            params.put("js_code", userBindReq.getCode());
            params.put("connect_redirect", "1");
            String result = HttpCilentUntil.httpGetToString(
                    "https://api.weixin.qq.com/sns/jscode2session", params);
            JSONObject jsonObject = JSON.parseObject(result);
            String openid = jsonObject.get("openid").toString();

            Long userId = (Long) request.getAttribute("LOGIN_USER_ID");//获取用户id
            wxService.bindWeChatUser(openid, userId);
        } catch (Exception e) {
            return ResponseBeanBuilder.fail("小程序绑定用户失败");
        }
        return ResponseBeanBuilder.ok();
    }

    @ApiOperation(value = "检测用户是否绑定微信")
    @PostMapping("/bind/check")
    public ResponseBean bindWechatUserCheck(@RequestBody UserBindReq userBindReq, HttpServletResponse response, HttpServletRequest request) throws Throwable {
        Integer checked = 0;//默认为绑定
        try {
            response.setContentType("text/html");
            request.setCharacterEncoding("UTF-8");
            response.setCharacterEncoding("UTF-8");
            Map params = new HashMap();
            params.put("secret", appsecret);
            params.put("appid", appid);
            params.put("grant_type", "authorization_code");
            params.put("js_code", userBindReq.getCode());
            params.put("connect_redirect", "1");
            String result = HttpCilentUntil.httpGetToString(
                    "https://api.weixin.qq.com/sns/jscode2session", params);
            JSONObject jsonObject = JSON.parseObject(result);
            String openid = jsonObject.get("openid").toString();

            Long userId = (Long) request.getAttribute("LOGIN_USER_ID");//获取用户id
            checked = wxService.selectBindWeChatUserCount(openid, userId);
        } catch (Exception e) {
            return ResponseBeanBuilder.fail("小程序绑定用户失败");
        }
        return ResponseBeanBuilder.ok(checked);
    }

}
