package com.factory.web.entity.vo.req.web;

import com.baomidou.mybatisplus.annotation.TableField;
import com.factory.base.config.validateGroup.GroupOne;
import com.fasterxml.jackson.annotation.JsonInclude;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.List;

/**
 * @program: intelligentfactory
 * @description:
 * @author: dl
 * @create: 06月29日 11时
 **/
@Data
@Builder
@NoArgsConstructor
@AllArgsConstructor
@JsonInclude()
@ApiModel("")
public class AddIniPlanReq {

    @Size(max = 50, message = "名字长度过长", groups = GroupOne.class)
    @NotEmpty(message = "计划名称不能为空", groups = GroupOne.class)
    @ApiModelProperty(value = "计划名称", example = "计划名称")
    private String name;

    @NotEmpty(message = "责任人不能为空", groups = GroupOne.class)
    @ApiModelProperty(value = "责任人id列表", example = "[2,3]")
    private List<Long> principalIds;


    @ApiModelProperty(value = "检验开始时间", example = "20:20:20")
    private LocalTime checkBegintime;

    @ApiModelProperty(value = "检验结束时间", example = "20:20:20")
    private LocalTime checkEndtime;

    @ApiModelProperty(value = "提醒首件pad时长", example = "10")
    private Integer remindIniPad;

    @ApiModelProperty(value = "提醒两百pad提交时刻", example = "20:20:20")
    private LocalTime remindTwoPad;

    @ApiModelProperty(value = "升级提醒时间", example = "20:20:20")
    private LocalTime upgradeTime;


    @NotNull(message = "启动日期不能为空", groups = GroupOne.class)
    @ApiModelProperty(value = "生效日期", example = "2015-05-05")
    private LocalDate beginDate;

    @ApiModelProperty(value = "截止日期", example = "2015-05-05")
    private LocalDate endDate;


    @Size(max = 20, message = "项目名称长度过长", groups = GroupOne.class)
    @NotEmpty(message = "项目名称不能为空", groups = GroupOne.class)
    @ApiModelProperty(value = "项目名称", example = "project")
    private String project;

    @Size(max = 20, message = "零件长度过长", groups = GroupOne.class)
    @NotEmpty(message = "零件不能为空", groups = GroupOne.class)
    @ApiModelProperty(value = "零件名称", example = "part")
    private String part;

    @Size(max = 20, message = "本特勒零件号长度过长", groups = GroupOne.class)
    @NotEmpty(message = "本特勒零件号不能为空", groups = GroupOne.class)
    @ApiModelProperty(value = "本特勒零件号", example = "btlPartNo")
    private String btlPartNo;

    @Size(max = 20, message = "客户零件号长度过长", groups = GroupOne.class)
    @NotEmpty(message = "客户零件号不能为空", groups = GroupOne.class)
    @ApiModelProperty(value = "客户零件号", example = "clientPartNo")
    private String clientPartNo;

    @Size(max = 20, message = "客户零件号长度过长", groups = GroupOne.class)
    @NotEmpty(message = "客户零件号不能为空", groups = GroupOne.class)
    @ApiModelProperty(value = "客户零件号", example = "verson")
    private String verson;

    @Size(max = 20, message = "Dmba长度过长", groups = GroupOne.class)
    @NotEmpty(message = "Dmba不能为空", groups = GroupOne.class)
    @ApiModelProperty(value = "Dmba", example = "Dmba")
    private String Dmba;

    @ApiModelProperty(value = "指导文件id", example = "15")
    private Long guideDocumentId;

    @TableField(exist = false)
    @ApiModelProperty(value = "计划id", example = "1")
    private Long planId;

    @TableField(exist = false)
    @ApiModelProperty(value = "版本id", example = "1")
    private Long versionId;

    @ApiModelProperty(value = "受控文件id")
    private Long controlledDocumentId;
    @TableField(exist = false)
    @ApiModelProperty(value = "工作站")
    private List<AddIniPlanWorkReq> versionWorks;

}
