package com.factory.web.mapper.fa;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.web.entity.fa.FirstArticleApprovalTask;
import com.factory.web.entity.fa.FirstArticleItemResult;
import com.factory.web.entity.fa.FirstArticleTask;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.fa.req.*;
import com.factory.web.entity.fa.res.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 首件计划
 *
 * @author wzy
 * @date 2021-07-09 12:00:00
 */
@Mapper
public interface FirstArticleMapper extends BaseMapper<BentelerPlan> {

    /**
     * 取得防错计划点检列表
     *
     * @param page
     * @param req
     * @return 防错计划点检列表
     */
    public Page<FaPlanListRes> getFaPlanCheckList(@Param("page") Page page, @Param("req") SearchFaPlanCheckListReq req);

    /**
     * 取得Pad端防错计划列表
     *
     * @param req
     * @return Pad端防错计划列表
     */
    public List<FaPlanListForPadRes> getFaPlanListForPad(@Param("req") SearchFaPlanListForPadReq req);
    /**
     * 取得Pad端中断任务
     *
     * @param req
     * @return Pad端防错计划列表
     */
    public List<FaTaskResForPad> getFaBreakTaskForPad(@Param("req") SearchFaWorkshopListForPadReq req);

    /**
     * 得Pad端工作站列表
     *
     * @param req
     * @return 工作站列表
     */
    public List<FirstArticleWorkshop> getFaWorkshopListForPad(@Param("req") SearchFaWorkshopListForPadReq req);

    /**
     * 得Pad端巡检项目列表
     *
     * @param req
     * @return 巡检项目列表
     */
    public List<FaItemListForPadRes> getFaItemListForPad(@Param("req") SearchFaItemListForPadReq req);

    /**
     * 取得不放行任务对应的工作站
     *
     * @param req
     * @return 不放行任务对应的工作站
     */
    public List<FaWorkshopNoGoListRes> getFaWorkshopNoGoList(@Param("req") SearchFaWorkshopNoGoListReq req);

    /**
     * 取得不放行任务对应的点检项目
     *
     * @param req
     * @return 不放行任务对应的点检项目
     */
    public List<FaItemNoGoListRes> getFaItemNoGoList(@Param("req") SearchFaItemNoGoListReq req);

    /**
     * 取得任务（首件、200件都是后台生成的任务，每个用户、计划、当天、类型（首件或200件））对应的应该是一个task。中断的task可以随便提交，所以要取状态是0初始化的，用户是-1这个userId不用传
     *
     * @param req
     * @return 任务
     */
    public FirstArticleTask getFaTaskInfo(@Param("req") SearchFaTaskInfoReq req);

    /**
     * 查询中断最新的task
     *
     * @param req
     * @return 任务
     */
    public FirstArticleTask getFaTaskInfoCatchLatest(@Param("req") SearchFaTaskInfoReq req);

    /**
     * 取得不放行任务对应的审批人、审批意见、审批时间
     *
     * @param req
     * @return 不放行任务对应的审批人、审批意见、审批时间
     */
    public FaNoGoTaskAapprovalRes getFaNoGoTaskAapproval(@Param("req") SearchFaTaskReq req);

    /**
     * 取得首件、200件、中断每天的点检记录
     *
     * @param req
     * @return 首件、200件、中断每天的点检记录
     */
    public List<FirstArticleItemResult> getFaItemResultList(@Param("req") SearchFaItemResultListReq req);

    /**
     * 取得计划下的点检项目数
     *
     * @param req
     * @return 计划下的点检项目数
     */
    public int getFaItemCount(@Param("req") SearchFaPlanReq req);

    /**
     * 取得计划下当前task的点检结果数
     *
     * @param req
     * @return 计划下当前task的点检结果数
     */
    public int getFaItemResultCount(@Param("req") SearchFaItemResultCountReq req);

    /**
     * 取得工作站下的点检项目数
     *
     * @param req
     * @return 工作站下的点检项目数
     */
    public int getFaWsItemCount(@Param("req") SearchFaWsItemResultCountReq req);

    /**
     * 取得工作站下当前task的点检结果数
     *
     * @param req
     * @return 工作站下当前task的点检结果数
     */
    public int getFaWsItemResultCount(@Param("req") SearchFaWsItemResultCountReq req);

    /**
     * 取得用户列表
     *
     * @param req
     * @return 用户列表
     */
    public List<UserListRes> getUserList(@Param("req") SearchUserListReq req);

    /**
     * 判断当前检查项目是否是放行
     *
     * @param req
     * @return 检查项目是否是放行
     */
    public int getFaItemReleaseCount(@Param("req") SearchFaItemReleaseCountReq req);

    public List<FirstArticleApprovalTask> getFaItemRelease(@Param("req") SearchFaItemReleaseCountReq req);

    /**
     * 取得点检人
     *
     * @param req
     * @return 点检人
     */
    public List<FaCalendarRes> getFaCalendarList(@Param("req") SearchFaCalendarReq req);

    /**
     * 取得重检项目下，未完成重检的数量
     *
     * @param req
     * @return 未完成重检的数量
     */
    public int getFaRepeatItemResultCount(@Param("req") SearchFaWsItemResultCountReq req);

}
