package com.factory.web.mapper.pm;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.web.entity.pm.OnsiteApprovalPlan;
import com.factory.web.entity.pm.OnsiteWorkshop;
import com.factory.web.entity.pm.req.*;
import com.factory.web.entity.pm.res.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 防错计划
 *
 * @author wzy
 * @date 2021-07-09 12:00:00
 */
@Mapper
public interface PreventMistakeMapper extends BaseMapper<BentelerPlan> {

	/**
	 * 待审批计划列表
	 * @param page
	 * @param req
	 * @return
	 */
	public Page<PmPlanListRes> getWaitApprovalPlanList(@Param("page") Page page, @Param("req") SearchPmPlanListReq req);
	
    /**
     * 取得防错计划点检列表
     *
     * @param page
     * @param req
     * @return 防错计划点检列表
     */
    public Page<PmPlanCheckListRes> getPmPlanCheckList(@Param("page") Page page, @Param("req") SearchPmPlanCheckListReq req);

    /**
     * 取得防错计划列表
     *
     * @param page
     * @param req
     * @return 防错计划列表
     * @see getWaitApprovalPlanList
     */
    @Deprecated
    public Page<PmPlanListRes> getPmPlanList(@Param("page") Page page, @Param("req") SearchPmPlanListReq req);

    /**
     * 取得防错计划当天任务信息
     *
     * @param req
     * @return 防错计划当天任务信息
     */
    public List<PmTaskListRes> getTaskList(@Param("req") SearchPmPlanReq req);

    /**
     * 取得工作站和巡检项目
     *
     * @param req
     * @return 检查记录列表
     */
    public List<PmCheckRecordListRes> getPmWorkshopItemList(@Param("req") SearchPmPlanReq req);

    /**
     * 取得一个巡检项目对应的一个月的巡检结果
     *
     * @param req
     * @return 检查记录列表
     */
    public List<PmCheckRecordRes> getPmItemResultList(@Param("req") SearchCheckRecordReq req);

    /**
     * 取得Pad端防错计划列表
     *
     * @param req
     * @return Pad端防错计划列表
     */
    public List<PmPlanListForPadRes> getPmPlanListForPad(@Param("req") SearchPmPlanListForPadReq req);

    /**
     * 取得审批人
     *
     * @param req
     * @return 审批人
     */
    public ApprovalUserRes getApprovalUser(@Param("req") SearchApprovalUserReq req);

    /**
     * 取得每频次点检信息
     *
     * @param req
     * @return 每频次点检信息
     */
    public List<PmCheckListRes> getPmCheckList(@Param("req") SearchPmPlanReq req);

    /**
     * 取得不放行任务对应的工作站
     *
     * @param req
     * @return 不放行任务对应的工作站
     */
    public List<PmWorkshopNoGoListRes> getPmWorkshopNoGoList(@Param("req") SearchPmWorkshopNoGoListReq req);

    /**
     * 取得不放行任务对应的点检项目
     *
     * @param req
     * @return 不放行任务对应的点检项目
     */
    public List<PmItemNoGoListRes> getPmItemNoGoList(@Param("req") SearchPmItemNoGoListReq req);

    /**
     * 取得不放行任务对应的审批人、审批意见、审批时间
     *
     * @param req
     * @return 不放行任务对应的审批人、审批意见、审批时间
     */
    public PmNoGoTaskAapprovalRes getPmNoGoTaskAapproval(@Param("req") SearchTaskReq req);

    /**
     * 取得计划的最新审批信息
     *
     * @param req
     * @return 计划的最新审批信息
     */
    public OnsiteApprovalPlan getLatestApprovalInfo(@Param("req") SearchPmPlanReq req);

    /**
     * 取得计划下的点检项目数
     *
     * @param req
     * @return 计划下的点检项目数
     */
    public int getPmItemCount(@Param("req") SearchPmPlanReq req);

    /**
     * 取得计划下当前task的点检结果数
     *
     * @param req
     * @return 计划下当前task的点检结果数
     */
    public int getPmItemResultCount(@Param("req") SearchPmAllCheckFlagReq req);

    /**
     * 取得工作站下的点检项目数
     *
     * @param req
     * @return 工作站下的点检项目数
     */
    public int getPmWsItemCount(@Param("req") SearchPmWsAllCheckFlagReq req);

    /**
     * 取得工作站下当前task的点检结果数
     *
     * @param req
     * @return 工作站下当前task的点检结果数
     */
    public int getPmWsItemResultCount(@Param("req") SearchPmWsAllCheckFlagReq req);


    /**
     * 取得防错放行
     *
     * @param req
     * @return 防错放行
     */
    public List<AbnormalTaskInfoRes> getPmReleaseList(@Param("req") SearchAbnormalReq req);

    /**
     * 取得防错重检
     *
     * @param req
     * @return 防错重检
     */
    public List<AbnormalTaskInfoRes> getPmRepeatList(@Param("req") SearchAbnormalReq req);

    /**
     * 取得防错工作站下的巡检项目和检查结果
     *
     * @param req
     * @return 防错工作站下的巡检项目和检查结果
     */
    public List<AbnormalItemRes> getPmAbnormalItemList(@Param("req") SearchAbnormalReq req);

    /**
     * 取得首件放行
     *
     * @param req
     * @return 首件放行
     */
    public List<AbnormalTaskInfoRes> getFaReleaseList(@Param("req") SearchAbnormalReq req);

    /**
     * 取得首件重检
     *
     * @param req
     * @return 首件重检
     */
    public List<AbnormalTaskInfoRes> getFaRepeatList(@Param("req") SearchAbnormalReq req);

    /**
     * 取得首件工作站下的巡检项目
     *
     * @param req
     * @return 首件工作站下的巡检项目
     */
    public List<AbnormalItemRes> getFaAbnormalItemList(@Param("req") SearchAbnormalReq req);

    /**
     * 取得首件检查结果
     *
     * @param req
     * @return 首件检查结果
     */
    public List<AbnormalItemResultRes> getFaAbnormalItemResultList(@Param("req") SearchAbnormalReq req);

    /**
     * 取得PAD端工作站
     *
     * @param req
     * @return PAD端工作站
     */
    public List<OnsiteWorkshop> getPmWorkshopListForPad(@Param("req") SearchPmAllCheckFlagReq req);

    /**
     * 取得重检项目下，未完成重检的数量
     *
     * @param req
     * @return 未完成重检的数量
     */
    public int getPmRepeatItemResultCount(@Param("req") SearchPmWsAllCheckFlagReq req);

}
