package com.factory.web.service.fa;

import com.baomidou.mybatisplus.extension.service.IService;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.user.entity.vo.res.FileRes;
import com.factory.web.entity.fa.FirstArticleItemResult;
import com.factory.web.entity.fa.FirstArticleTask;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.fa.req.*;
import com.factory.web.entity.fa.res.*;
import com.factory.web.entity.pm.ApprovalRule;
import com.factory.web.entity.vo.req.ExportInitialPlanDetailsReq;

import java.util.List;

/**
 * 首件计划
 *
 * @author wzy
 * @date 2021-07-09 12:00:00
 */
public interface FirstArticleService extends IService<BentelerPlan> {


	/**
	 * 获取历史信息计划
	 * @param req
	 * @return
	 */
	public ResponseBean<List<BentelerPlan>> getHisPlanList(SearchHisPlanListReq req) ;

	/**
	 * 新建首件计划
	 *
	 * @param req
	 * @return
	 */
	public ResponseBean<Void> addFaPlan(AddFaPlanReq req);

	/**
	 * 校验计划名是否存在
	 *
	 * @param name
	 * @return true存在，false不存在
	 */
	public boolean checkPlanNameExist(String name,Long originalId);

	/**
	 * 校验工作站名是否存在
	 *
	 * @param wsName
	 * @return true存在，false不存在
	 */
	public boolean checkWsNameExist(String wsName);

	/**
	 * 取得首件计划信息
	 *
	 * @param req
	 * @return 首件计划信息
	 */
	public FaPlanRes getFaPlan(SearchFaPlanReq req);

	/**
	 * 校验当前计划是否审批通过
     *
     * @param bentelerPlanId
     * @return true通过，false不通过
     */
    public boolean checkApprovalPlan(long bentelerPlanId);

    /**
     * 修改首件计划
     *
     * @param req
     * @return
     */
    public ResponseBean<Void> editFaPlan(EditFaPlanReq req);

    /**
     * 取得首件计划点检列表
     *
     * @param req
     * @return 首件计划点检列表
     */
    public ResponseBean<PageBean<FaPlanListRes>> getFaPlanCheckList(SearchFaPlanCheckListReq req);

    /**
     * 取得Pad端首件计划列表
     *
     * @param req
     * @return Pad端首件计划列表
     */
    public ResponseBean<List<FaPlanListForPadRes>> getFaPlanListForPad(SearchFaPlanListForPadReq req);

    /**
     * 得Pad端工作站
     *
     * @param req
     * @return 工作站
     */
    public ResponseBean<List<FirstArticleWorkshop>> getFaWorkshopListForPad(SearchFaWorkshopListForPadReq req)  throws Exception;

    /**
     * 取得Pad端巡检项目
     *
     * @param req
     * @return 巡检项目
     */
    public ResponseBean<List<FaItemListForPadRes>> getFaItemListForPad(SearchFaItemListForPadReq req) throws Exception;

    /**
     * 添加巡检项目检查结果
     *
     * @param req
     * @return
     */
    public ResponseBean<Void> addFaItemResult(AddFaItemResultListReq req);

    /**
     * 任务提交
     *
     * @param req
     * @return
     */
    public ResponseBean<Void> submitFaTask(SubmitFaTaskReq req);

    /**
     * 取得不放行任务对应的工作站
     *
     * @param req
     * @return 不放行任务对应的工作站
     */
    public ResponseBean<List<FaWorkshopNoGoListRes>> getFaWorkshopNoGoList(SearchFaWorkshopNoGoListReq req);

    /**
     * 取得不放行任务对应的点检项目
     *
     * @param req
     * @return 不放行任务对应的点检项目
     */
    public ResponseBean<List<FaItemNoGoListRes>> getFaItemNoGoList(SearchFaItemNoGoListReq req);

    /**
     * 修改巡检项目检查结果(这里只有重检走这里)
     *
     * @param req
     * @return
     */
    public ResponseBean<Void> editFaItemResult(EditFaItemResultListReq req);

    /**
     * 任务和巡检结果记录历史
     *
     * @param firstArticleTaskId
     * @return
     */
    public void setFirstArticleTaskHis(Long firstArticleTaskId);

    /**
     * 取得表头信息
     *
     * @param bentelerPlanId
     * @return 表头信息
     */
    public FirstArticleTitle getFaTitle(Long bentelerPlanId);

    /**
     * 取得任务（首件、200件都是后台生成的任务，每个用户、计划、当天、类型（首件或200件））对应的应该是一个task。中断的task可以随便提交，所以要取状态是0初始化的，用户是-1这个userId不用传
     *
     * @param req
     * @return 任务
     */
    public FirstArticleTask getFaTaskInfo(SearchFaTaskInfoReq req);

    /**
     * 查询中断最新的task
     *
     * @param req
     * @return 任务
     */
    public FirstArticleTask getFaTaskInfoCatchLatest(SearchFaTaskInfoReq req)  throws Exception;

    /**
     * 取得不放行任务对应的质量经理审批信息
     *
     * @param req
     * @return 不放行任务对应的质量经理审批信息
     */
    public ResponseBean<FaNoGoTaskAapprovalRes> getFaNoGoTaskAapproval(SearchFaTaskReq req);

    /**
     * 取得所有计划
     *
     * @param req
     * @return 所有计划
     */
    public ResponseBean<List<BentelerPlan>> getALLPlanList(SearchALLPlanListReq req);

    /**
     * 检查记录列表
     *
     * @param req
     * @return 检查记录列表
     */
    public ResponseBean<FaCheckRecordListRes> getFaCheckRecordList(SearchFaCheckRecordListReq req);

    /**
     * 取得首件、200件、中断每天的点检记录
     *
     * @param req
     * @return 首件、200件、中断每天的点检记录
     */
    public List<FirstArticleItemResult> getFaItemResultList(SearchFaItemResultListReq req);

//    /**
//     * 判断检验结果是否填写完成
//     * @param req
//     * @return 判断检验结果是否填写完成
//     */
//    public FaAllCheckFlagRes getFaAllCheckFlag(SearchFaAllCheckFlagReq req);

    /**
     * 取得用户列表
     *
     * @param req
     * @return 取得用户列表
     */
    public List<UserListRes> getUserList(List<Long> roleId);

    /**
     * 取得点检人
     *
     * @param req
     * @return 点检人
     */
    public List<FaCalendarRes> getFaCalendarList(SearchFaCalendarReq req);

    ResponseBean<FileRes> exportFaCheckRecordList(SearchFaCheckRecordListReq req);

    ResponseBean<FileRes> exportInitialPlanDetails(ExportInitialPlanDetailsReq req);

    /**
     * task二级审批不通过的，即为重检，此时将点检结果item_result为不通过的1都改成成-1，这样重检时相当于重新开始，没有之前首次点检的结果
     *
     * @param firstArticleTaskId
     * @return
     */
    public void setFaRepeatResult(long firstArticleTaskId);
    
    /**
     * 审批信息和通知
     * @param approvalRuleNextLevel
     * @param taskId
     * @param userId
     */
    public void setApproval(ApprovalRule approvalRuleNextLevel,Long taskId,Long userId);
    
    /**
     * 获取中断任务列表以及完成状态
     * @param req
     * @return
     */
    public List<FaTaskResForPad> getFaBreakTaskForPad(SearchFaWorkshopListForPadReq req);
}
