package com.factory.web.service.impl.ini;

import cn.hutool.core.convert.Convert;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.enumPo.CheckStatus;
import com.factory.base.entity.enumPo.CheckTypeSubmit;
import com.factory.base.entity.enumPo.PlanType;
import com.factory.base.entity.enumPo.ReviewStatus;
import com.factory.base.entity.po.benteler.*;
import com.factory.base.entity.vo.req.BaseIdReq;
import com.factory.base.entity.vo.req.BasePageIdReq;
import com.factory.base.entity.vo.req.BaseTypeReq;
import com.factory.base.util.file.FileUtils;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.entity.vo.res.FileRes;
import com.factory.user.service.CommonService;
import com.factory.user.service.UserInfoService;
import com.factory.web.entity.excel.FillData;
import com.factory.web.entity.excel.TestFileUtil;
import com.factory.web.entity.vo.req.pad.GetVersionCheckItemReq;
import com.factory.web.entity.vo.req.pad.submitCheckReq;
import com.factory.web.entity.vo.req.pad.workDetailsReq;
import com.factory.web.entity.vo.req.web.*;
import com.factory.web.entity.vo.res.pad.ini.*;
import com.factory.web.entity.vo.res.web.ini.*;
import com.factory.web.mapper.ini.BentelerIniVersionMapper;
import com.factory.web.service.BentelerPlanService;
import com.factory.web.service.ini.*;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import java.io.File;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


@Service("bentelerIniVersionService")
@Transactional(rollbackFor = Exception.class)
public class BentelerIniVersionServiceImpl extends ServiceImpl<BentelerIniVersionMapper, BentelerIniVersion> implements BentelerIniVersionService {

    @Autowired
    private UserInfoService userInfoService;
    @Autowired
    private CommonService commonService;
    @Autowired
    private BentelerIniVersionMapper versionMapper;
    @Autowired
    private BentelerPlanService planService;
    @Autowired
    private BentelerIniVersionService versionService;
    @Autowired
    private BentelerIniVersionItemService versionItemService;
    @Autowired
    private BentelerIniCheckResultService checkResultService;
    @Autowired
    private BentelerIniWorkstationService workstationService;
    @Autowired
    private BentelerIniCheckItemService checkItemService;
    @Autowired
    private BentelerIniAbnormalService iniAbnormalService;
    @Autowired
    private BentelerIniCheckTypeService checkTypeService;

    @Override
    public ResponseBean<PageBean<PagePlanRes>> pagePlan_web(PlanListReq req) {
        String name = req.getName();
        Integer checkStatus = req.getCheckStatus();
        Page<PagePlanRes> page = new Page<>(req.getPage(), req.getSize());
        return ResponseBeanBuilder.ok(versionMapper.pagePlan(page, name, checkStatus));
    }

    @Override
    public ResponseBean addIniPlan(AddIniPlanReq req) {
        //校验
        if (planNameRepetition(req.getName())) {
            return ResponseBeanBuilder.fail("计划名称重复");
        }
        //新增计划
        BentelerPlan plan = BentelerPlan.builder()
                .planType(PlanType.首件.value)
                .name(req.getName())
                .pauseFlag(false)
                .build();
        planService.save(plan);
        //新增版本
        /*点检类型*/
        ArrayList<Integer> allCheckType = req.getVersionWorks().stream().collect(
                ArrayList::new,
                (theList, item) -> {
                    Collection<Integer> union1 = CollectionUtils.union(theList,
                            item.getBentelerIniVersionItems().stream().collect(
                                    ArrayList::new,
                                    (theList2, item2) -> {
                                        Collection<Integer> union = CollectionUtils.union(theList2, item2.getCheckType());
                                        theList2.addAll(union);
                                    },
                                    ArrayList::addAll
                            )
                    );
                    theList.addAll(union1);
                },
                ArrayList::addAll

        );
        /*基础信息*/
        Map<String, String> baseMap = new HashMap<String, String>() {{
            put("project", req.getProject());
            put("part", req.getPart());
            put("btlPartNo", req.getBtlPartNo());
            put("clientPartNo", req.getClientPartNo());
            put("verson", req.getVerson());
            put("Dmba", req.getDmba());
        }};

        List<Long> principalIds = req.getPrincipalIds();
        Long principleOne = principalIds.get(0);
        Long principleTwo = null;
        if (principalIds.size() > 1) {
            principleTwo = principalIds.get(1);
        }
        //新增版本
        BentelerIniVersion version = BentelerIniVersion.builder()
                .planId(plan.getId())
                .allChecktype(Convert.toStr(allCheckType))
                .baseMap(JSONUtil.toJsonStr(baseMap))
                .checkStatus(CheckStatus.未完成.value)
                .beginDate(req.getBeginDate())
                .endDate(req.getEndDate())
                .guideDocumentId(req.getGuideDocumentId())
                .principleOne(principleOne)
                .principleTwo(principleTwo)
                .reviewStatus(ReviewStatus.待质量经理审核.value)

                .checkBegintime(req.getCheckBegintime())
                .checkEndtime(req.getCheckEndtime())
                .remindIniPad(req.getRemindIniPad())
                .remindTwoPad(req.getRemindTwoPad())
                .upgradeTime(req.getUpgradeTime())
                .build();
        versionService.save(version);


        //新增检查项
        ArrayList<BentelerIniVersionItem> VersionItems = new ArrayList<>();

        req.getVersionWorks().forEach(versionWork -> {
            //新增工作站
            BentelerIniWorkstation workstation = BentelerIniWorkstation.builder()
                    .versionId(version.getId())
                    .workstation(versionWork.getWorkstation())
                    .productionLine(versionWork.getProductionLine())
                    .versionId(version.getId())
                    .build();
            workstationService.save(workstation);
            //新增点检项
            versionWork.getBentelerIniVersionItems().forEach(item -> {
                BentelerIniVersionItem versionItem = BentelerIniVersionItem.builder()
                        .planId(plan.getId())
                        .versionId(version.getId())
                        .workstationId(workstation.getId())
                        .checkStandardTop(item.getCheckStandardTop())
                        .checkStandardFloor(item.getCheckStandardFloor())
                        .checkType(Convert.toStr(item.getCheckType()))
                        .dataType(item.getDataType())
                        .frequency(item.getFrequency())
                        .frequencyNo(item.getFrequencyNo())
                        .inspectionItem(item.getInspectionItem())
                        .measuRemark(item.getMeasuRemark())
                        .level(item.getLevel())
                        .method(item.getMethod())
                        .unit(item.getUnit())
                        .tool(item.getTool())
                        .toolRemark(item.getToolRemark())
                        .workPrefix(item.getWorkPrefix())
                        .workStationBegin(item.getWorkStationBegin())
                        .workStationEnd(item.getWorkStationEnd())

                        .checkBegintime(req.getCheckBegintime())
                        .checkEndtime(req.getCheckEndtime())
                        .remindIniPad(req.getRemindIniPad())
                        .remindTwoPad(req.getRemindTwoPad())
                        .upgradeTime(req.getUpgradeTime())
                        .build();
                VersionItems.add(versionItem);
            });

        });
        versionItemService.saveBatch(VersionItems);


        return ResponseBeanBuilder.ok();
    }

    @Override
    public ResponseBean disablePlan(BooleanReq req) {
        Long versionId = req.getId();

        Long planId = versionService.getById(versionId).getPlanId();

        planService.updateById(BentelerPlan.builder()
                .id(planId)
                .pauseFlag(req.getBoolea())
                .build());
        return ResponseBeanBuilder.ok();
    }

    @Override
    public ResponseBean<PageBean<unAuditListRes>> unAuditList(PlanListReq req) {
        Page<unAuditListRes> page = new Page<>(req.getPage(), req.getSize());
        Page<unAuditListRes> unAuditListResPage = versionMapper.unAuditList(page, req);
        return ResponseBeanBuilder.ok(unAuditListResPage, Function.identity());
    }

    @Override
    public ResponseBean<GetVersionBaseRes> getVersionBase(BaseIdReq req) {
        Long versionId = req.getId();
        BentelerIniVersion version = versionService.getById(versionId);
        String baseMap = version.getBaseMap();
        //Map<String, String> stringStringMap = Convert.toMap(String.class, String.class, JSONUtil.parseObj(baseMap));
        Map<String, String> stringStringMap = JSON.parseObject(version.getBaseMap(), new TypeReference<HashMap<String, String>>() {
        });
        GetVersionBaseRes basemap = GetVersionBaseRes.builder()
                .project(stringStringMap.get("project"))
                .part(stringStringMap.get("part"))
                .btlPartNo(stringStringMap.get("btlPartNo"))
                .clientPartNo(stringStringMap.get("clientPartNo"))
                .verson(stringStringMap.get("verson"))
                .Dmba(stringStringMap.get("Dmba"))
                .build();
        return ResponseBeanBuilder.ok(basemap);
    }


    @Override
    public ResponseBean<AddIniPlanReq> getUnAuditDetails(BaseIdReq req) {
        Long planId = req.getId();
        BentelerPlan plan_DB = planService.getById(planId);
        if (plan_DB == null) {
            return ResponseBeanBuilder.fail("计划不存在,请检查计划id");
        }
        BentelerIniVersion unAuditVersion = versionMapper.getPlan_UnAuditVersion(planId);
        if (unAuditVersion == null) {
            ResponseBeanBuilder.fail("计划无审核中信息");
        }
        BentelerPlan plan1 = assemblerVersionId_plan(unAuditVersion.getId());


        return ResponseBeanBuilder.ok(planPo2planReq(plan1));
    }

    @Override
    public ResponseBean<AddIniPlanReq> getCurrentDetails(BaseIdReq req) {
        Long planId = req.getId();
        BentelerIniVersion currentVersion = versionMapper.getPlan_CurrentVersion(planId);
        if (currentVersion == null) {
            return ResponseBeanBuilder.fail("版本暂未审核通过");
        }
        BentelerPlan plan = assemblerVersionId_plan(currentVersion.getId());

        return ResponseBeanBuilder.ok(planPo2planReq(plan));
    }

    private BentelerPlan assemblerVersionId_plan(Long versionId) {
        BentelerIniVersion version = versionService.getById(versionId);
        BentelerPlan plan = planService.getById(version.getPlanId());

        List<BentelerIniWorkstation> works = workstationService.list(Wrappers.<BentelerIniWorkstation>lambdaQuery()
                .eq(BentelerIniWorkstation::getVersionId, version.getId())
        );
        List<BentelerIniWorkstation> ass_works = works.stream().map(workstation -> {
            List<BentelerIniVersionItem> versionItems = versionItemService.list(Wrappers.<BentelerIniVersionItem>lambdaQuery()
                    .eq(BentelerIniVersionItem::getWorkstationId, workstation.getId()));
            workstation.setVersionItems(versionItems);
            return workstation;
        }).collect(Collectors.toList());
        version.setWorkstations(ass_works);
        plan.setVersion(version);
        return plan;
    }

    @Override
    public ResponseBean addIniVersion(AddIniPlanReq req) {

        Long versionId = req.getVersionId();
        Long planId = versionService.getById(versionId).getPlanId();

        //获取待审核版本  并删除
        BentelerIniVersion unAuditVersion = versionMapper.getPlan_UnAuditVersion(planId);
        if (unAuditVersion != null) {
            Long versionId_DB = unAuditVersion.getId();
            versionService.removeById(versionId_DB);
        }

        String name = planService.getById(planId).getName();
        if (!req.getName().equals(name)) {
            if (planNameRepetition(req.getName())) {
                return ResponseBeanBuilder.fail("计划名称重复");
            }
        }
        //转换数据库
        BentelerPlan plan_req = planReq2planPo(req, ReviewStatus.待质量经理审核);
        BentelerIniVersion version_req = plan_req.getVersion();
        version_req.setPlanId(planId);
        versionService.save(version_req);
        List<BentelerIniWorkstation> workstations = version_req.getWorkstations();
        workstationService.saveBatch(workstations);
        workstations.forEach(item -> {
            List<BentelerIniVersionItem> versionItems = item.getVersionItems();
            versionItems.forEach(versionItem -> {
                versionItem.setPlanId(planId);
                versionItem.setVersionId(version_req.getId());
            });
            versionItemService.saveBatch(versionItems);
        });
        return ResponseBeanBuilder.ok();
    }


    @Override
    public ResponseBean<Boolean> getIsUnAuditVersionId(BaseIdReq req) {
        //查询是否存在其他待审核
        Long versionId = req.getId();
        BentelerIniVersion version_DB = versionService.getById(versionId);
        BentelerIniVersion unAuditVersion = versionMapper.getPlan_WaitAuditVersion(version_DB.getPlanId());
        if (unAuditVersion != null) {
            return ResponseBeanBuilder.ok(true);
        }
        return ResponseBeanBuilder.ok(false);
    }

    @Override
    public ResponseBean<PageBean<GetVersionHistoryRes>> historyVersion(BasePageIdReq req) {
        Long vertsionId = req.getId();
        BentelerIniVersion version = versionService.getById(vertsionId);
        Long planId = version.getPlanId();
        Page<BentelerIniVersion> page = new Page<>(req.getPage(), req.getSize());
        Page<BentelerIniVersion> versionPage = versionService.page(page, Wrappers.<BentelerIniVersion>lambdaQuery()
                .eq(BentelerIniVersion::getPlanId, planId)
                .eq(BentelerIniVersion::getReviewStatus, ReviewStatus.历史版本.value)
        );

        ResponseBean<PageBean<GetVersionHistoryRes>> ok = ResponseBeanBuilder.ok(versionPage,
                item -> GetVersionHistoryRes.builder()
                        .id(item.getId())
                        .createdTime(item.getCreatedTime())
                        .planId(item.getPlanId())
                        .build()
        );
        return ok;
    }

    @Override
    public ResponseBean<planlistRes> planlist_pad() {
        LocalDate now = LocalDate.now();
        List<getNowCheckStatusRes> inifinish = versionMapper.getVersion_DateCheckStatus(CheckStatus.首件已完成.value, now);
        List<getNowCheckStatusRes> twoFinish = versionMapper.getVersion_DateCheckStatus(CheckStatus.两百已完成.value, now);
        List<getNowCheckStatusRes> unFinish = versionMapper.getVersion_DateCheckStatus(CheckStatus.未完成.value, now);

        planlistRes res = planlistRes.builder()
                .inifinish(inifinish)
                .twoFinish(twoFinish)
                .unFinish(unFinish)
                .build();
        return ResponseBeanBuilder.ok(res);
    }

    @Override
    public ResponseBean<List<workDetailsRes>> workDetails(BaseTypeReq req) {
        Long versionId = req.getId();
        Integer type = req.getType();
        //req
        GetVersionCheckItemReq getVersionCheckItemReq = GetVersionCheckItemReq.builder()
                .versionId(versionId)
                .checkType(type)
                .localDate(LocalDate.now())
                .build();
        //点检项 结果
        List<BentelerIniCheckItem> checkItems = versionMapper.getVersion_CheckItem(getVersionCheckItemReq);
        Map<Long, List<BentelerIniCheckItem>> workId_checkItem_map =
                checkItems.stream().collect(Collectors.groupingBy(BentelerIniCheckItem::getWorkId));

        List<workDetailsRes> resList = new ArrayList<>();

        workId_checkItem_map.entrySet().forEach(item -> {
            BentelerIniWorkstation workstation = workstationService.getById(item.getKey());

            boolean saveOrSubmit = item.getValue().stream().allMatch(checkitem -> {
                return !checkitem.getSubmitType().equals(CheckTypeSubmit.未保存.value);
            });

            workDetailsRes res = workDetailsRes.builder()
                    .workName(workstation.getWorkstation() + "-" + workstation.getProductionLine())
                    .submitType(saveOrSubmit == true ? 1 : 2)
                    .build();
            resList.add(res);
        });
        return ResponseBeanBuilder.ok(resList);
    }

    @Override
    public ResponseBean<workTypeDetailsRes> workTypeDetails(workDetailsReq req) {
        Long versionId = workstationService.getById(req.getWorkStationId()).getVersionId();
        BentelerIniCheckResult checkResult = checkResultService.getOne(Wrappers.<BentelerIniCheckResult>lambdaQuery()
                .eq(BentelerIniCheckResult::getVersionId, versionId)
                .like(BentelerIniCheckResult::getCreatedTime, LocalDate.now())
        );
        List<BentelerIniVersionItem> versionItems = versionMapper.getWork_VersionItem(
                req.getWorkStationId(),
                req.getCheckType(),
                LocalDate.now()
        );
        assemblerVersionItem_checkItem(versionItems, req.getCheckType(), LocalDate.now());
        workTypeDetailsRes res = workTypeDetailsRes.builder()
                .workId(req.getWorkStationId())
                .versionItems(versionItems)
                .useTool(checkResult.getUseSpanner())
                .toolRemark(checkResult.getSpannerRemark())
                .build();
        return ResponseBeanBuilder.ok(res);
    }

    @Override
    public ResponseBean saveCheckItem(@Validated workTypeDetailsRes req) {
        //保存记录项
        List<BentelerIniVersionItem> versionItems = req.getVersionItems();
        BentelerIniVersionItem fistVersionItem = versionItems.stream().findFirst().orElse(null);
        List<BentelerIniCheckItem> checkItems = versionItems.stream().map(item -> {
            item.getCheckItems().forEach(checkItem -> {
                List<IniGread> iniGreads = checkItem.getIniGreads();
                String greadStr = JSONUtil.toJsonStr(iniGreads);
                checkItem.setGread(greadStr);
                checkItem.setSubmitType(CheckTypeSubmit.保存.value);
            });
            return item;
        }).collect(
                ArrayList::new,
                (theList, item) -> {
                    theList.addAll(item.getCheckItems());
                },
                ArrayList::addAll
        );
        checkItemService.updateBatchById(checkItems);
        //扳手备注
        BentelerIniCheckItem checkItem = fistVersionItem.getCheckItems().stream().findFirst().orElse(null);
        BentelerIniCheckResult result = checkResultService.getById(checkItem.getCheckResultId());
        result.setUseSpanner(req.getUseTool());
        result.setSpannerRemark(req.getToolRemark());
        checkResultService.updateById(result);
        //修改状态
        BentelerIniCheckType checkType = checkTypeService.getOne(Wrappers.<BentelerIniCheckType>lambdaQuery()
                .eq(BentelerIniCheckType::getCheckResultId, result.getId())
                .like(BentelerIniCheckType::getCreatedTime, LocalDate.now())
                .eq(BentelerIniCheckType::getCheckType, Integer.valueOf(fistVersionItem.getCheckType()))
        );
        checkType.setStatus(CheckTypeSubmit.保存.value);

        checkTypeService.updateById(checkType);

        return ResponseBeanBuilder.ok();
    }

    @Override
    public ResponseBean submitCheck(submitCheckReq req) {
        Long resultId = req.getResultId();
        Integer checkType = req.getCheckType();
        //修改点检总表状态
        BentelerIniCheckResult result = checkResultService.getById(resultId);
        result.setCheckStatus(checkType);
        checkResultService.updateById(result);
        //点检分类记录表
        BentelerIniCheckType iniCheckType = getResult_checkType(resultId, checkType, LocalDate.now());

        //记录提交信息
        iniCheckType.setStatus(CheckTypeSubmit.提交.value);
        iniCheckType.setSubmitUser(commonService.getLoginUserInfo().getId());
        iniCheckType.setSubmitTime(LocalDateTime.now());
        checkTypeService.updateById(iniCheckType);
        //统计异常
        List<BentelerIniCheckItem> checkItems = checkItemService.list(Wrappers.<BentelerIniCheckItem>lambdaQuery()
                .eq(BentelerIniCheckItem::getCheckResultId, resultId)
                .eq(BentelerIniCheckItem::getCheckType, checkType)
                .like(BentelerIniCheckItem::getCreatedTime, LocalDate.now())
        );
        checkItems.forEach(item -> {
            item.setSubmitType(CheckTypeSubmit.提交.value);

            BentelerIniVersionItem versionItem = versionItemService.getById(item.getVersionItemId());

            List<IniGread> iniGreads = item.getIniGreads();
            boolean allPass = iniGreads.stream().allMatch(gread -> {
                if (gread.isGreadFlag()) {
                    return versionItem.getCheckStandardTop().compareTo(gread.getGread()) > -1
                            &&
                            versionItem.getCheckStandardFloor().compareTo(gread.getGread()) < 1;
                } else {
                    return gread.isPassResult();
                }
            });
            if (!allPass) {
                //todo
            }
        });
        //todo
        //todo
        checkItemService.saveBatch(checkItems);
        return ResponseBeanBuilder.ok();
    }

    @Override
    public BentelerIniCheckType getResult_checkType(Long resultId, Integer checkType, LocalDate now) {
        return checkTypeService.getOne(Wrappers.<BentelerIniCheckType>lambdaQuery()
                .eq(BentelerIniCheckType::getCheckResultId, resultId)
                .eq(BentelerIniCheckType::getCheckType, checkType)
                .like(BentelerIniCheckType::getCreatedTime, now)
        );
    }


    @Override
    public void assemblerVersionItem_checkItem(List<BentelerIniVersionItem> versionItems, Integer checkType, LocalDate now) {
        versionItems.forEach(item -> {
            Long versionItemId = item.getId();
            List<BentelerIniCheckItem> checkItems = checkItemService.list(Wrappers.<BentelerIniCheckItem>lambdaQuery()
                    .eq(BentelerIniCheckItem::getVersionItemId, versionItemId)
                    .eq(BentelerIniCheckItem::getCheckType, checkType)
                    .like(BentelerIniCheckItem::getCreatedTime, now)
            );
            assemblerCheckItem_iniGread(checkItems);
            item.setCheckItems(checkItems);

        });
    }

    @Override
    public void assemblerCheckItem_iniGread(List<BentelerIniCheckItem> checkItems) {
        checkItems.forEach(checkItem -> {
            String gread = checkItem.getGread();
            JSONArray greadsJson = JSONUtil.parseArray(gread);
            List<IniGread> iniGreads = greadsJson.toList(IniGread.class);
            checkItem.setIniGreads(iniGreads);
        });
    }

    //PlanReq转化planPo
    @Override
    public BentelerPlan planReq2planPo(AddIniPlanReq req, ReviewStatus reviewStatus) {
        //计划
        BentelerPlan plan = BentelerPlan.builder()
                .planType(PlanType.首件.value)
                .name(req.getName())
                .pauseFlag(false)
                .build();
        //version
        ArrayList<Integer> allCheckType = req.getVersionWorks().stream().collect(
                ArrayList::new,
                (theList, item) -> CollectionUtils.union(theList, item.getBentelerIniVersionItems().stream().collect(
                        ArrayList::new,
                        (theList2, item2) -> CollectionUtils.union(theList2, item2.getCheckType()),
                        ArrayList::addAll
                )),
                ArrayList::addAll

        );
        Map<String, String> baseMap = new HashMap<String, String>() {{
            put("project", req.getName());
            put("part", req.getName());
            put("btlPartNo", req.getName());
            put("clientPartNo", req.getName());
            put("verson", req.getName());
            put("Dmba", req.getName());
        }};

        List<Long> principalIds = req.getPrincipalIds();
        Long principleOne = principalIds.get(0);
        Long principleTwo = null;
        if (principalIds.size() > 1) {
            principleTwo = principalIds.get(1);
        }
        BentelerIniVersion version = BentelerIniVersion.builder()
                .id(req.getVersionId())
                .planId(plan.getId())
                .checkStatus(CheckStatus.未完成.value)
                .principleOne(principleOne)
                .principleTwo(principleTwo)
                .checkBegintime(req.getCheckBegintime())
                .checkEndtime(req.getCheckEndtime())
                .remindIniPad(req.getRemindIniPad())
                .remindTwoPad(req.getRemindTwoPad())
                .upgradeTime(req.getUpgradeTime())
                .beginDate(req.getBeginDate())
                .endDate(req.getEndDate())
                .allChecktype(Convert.toStr(allCheckType))
                .reviewStatus(reviewStatus.value)
                .baseMap(JSONUtil.toJsonStr(baseMap))
                .guideDocumentId(req.getGuideDocumentId())
                .controlledDocumentId(req.getControlledDocumentId())
                .build();
        //item
        ArrayList<BentelerIniWorkstation> workstations = new ArrayList<>();
        req.getVersionWorks().forEach(versionWork -> {
            BentelerIniWorkstation workstation = BentelerIniWorkstation.builder()
                    .versionId(version.getId())
                    .workstation(versionWork.getWorkstation())
                    .productionLine(versionWork.getProductionLine())
                    .build();
            //item
            ArrayList<BentelerIniVersionItem> VersionItems = new ArrayList<>();
            versionWork.getBentelerIniVersionItems().forEach(item -> {
                BentelerIniVersionItem versionItem = BentelerIniVersionItem.builder()
                        .planId(plan.getId())
                        .versionId(version.getId())
                        .checkStandardTop(item.getCheckStandardTop())
                        .checkStandardFloor(item.getCheckStandardFloor())
                        .checkType(Convert.toStr(item.getCheckType()))
                        .dataType(item.getDataType())
                        .frequency(item.getFrequency())
                        .frequencyNo(item.getFrequencyNo())
                        .inspectionItem(item.getInspectionItem())
                        .measuRemark(item.getMeasuRemark())
                        .level(item.getLevel())
                        .method(item.getMethod())
                        .unit(item.getUnit())
                        .tool(item.getTool())
                        .toolRemark(item.getToolRemark())
                        .workPrefix(item.getWorkPrefix())
                        .workStationBegin(item.getWorkStationBegin())
                        .workStationEnd(item.getWorkStationEnd())

                        .checkBegintime(req.getCheckBegintime())
                        .checkEndtime(req.getCheckEndtime())
                        .remindIniPad(req.getRemindIniPad())
                        .remindTwoPad(req.getRemindTwoPad())
                        .upgradeTime(req.getUpgradeTime())
                        .build();
                VersionItems.add(versionItem);
            });
            workstation.setVersionItems(VersionItems);
            workstations.add(workstation);
        });
        version.setWorkstations(workstations);
        plan.setVersion(version);
        return plan;
    }

    //planPo转化AddIniPlanReq
    @Override
    public AddIniPlanReq planPo2planReq(BentelerPlan plan) {
        BentelerIniVersion version = plan.getVersion();
        List<BentelerIniWorkstation> workstations = version.getWorkstations();
        List<AddIniPlanWorkReq> workReq = workstations.stream().map(workstation -> {
            List<BentelerIniVersionItem> versionItems = workstation.getVersionItems();
            List<AddIniPlanItemReq> itemReqs = versionItems.stream().map(versionItemf -> {
                List<Integer> checkTypes = Convert.toList(Integer.class, versionItemf.getCheckType());
                return AddIniPlanItemReq.builder()
                        .checkType(checkTypes)
                        .measuRemark(versionItemf.getMeasuRemark())
                        .method(versionItemf.getMethod())
                        .checkStandardTop(versionItemf.getCheckStandardTop())
                        .checkStandardFloor(versionItemf.getCheckStandardFloor())
                        .workPrefix(versionItemf.getWorkPrefix())
                        .workStationBegin(versionItemf.getWorkStationBegin())
                        .workStationEnd(versionItemf.getWorkStationEnd())
                        .frequency(versionItemf.getFrequency())
                        .frequencyNo(versionItemf.getFrequencyNo())
                        .dataType(versionItemf.getDataType())
                        .level(versionItemf.getLevel())
                        .unit(versionItemf.getUnit())
                        .tool(versionItemf.getTool())
                        .toolRemark(versionItemf.getToolRemark())
                        .measuRemark(versionItemf.getMeasuRemark())
                        .inspectionItem(versionItemf.getInspectionItem())
                        .build();
            }).collect(Collectors.toList());


            return AddIniPlanWorkReq.builder()
                    .productionLine(workstation.getProductionLine())
                    .workstation(workstation.getWorkstation())
                    .BentelerIniVersionItems(itemReqs)
                    .build();

        }).collect(Collectors.toList());


        String baseMapStr = version.getBaseMap();

        Map<String, String> baseMap = JSON.parseObject(baseMapStr, new TypeReference<HashMap<String, String>>() {
        });
        // Map<String, String> baseMap = Convert.toMap(String.class, String.class, JSONUtil.parseObj());

        //责任人
        Long principleOne = version.getPrincipleOne();
        Long getPrincipleTwo = version.getPrincipleTwo();

        return AddIniPlanReq.builder()
                .name(plan.getName())
                .principalIds(Arrays.asList(principleOne, getPrincipleTwo))
                .checkEndtime(version.getCheckEndtime())
                .remindIniPad(version.getRemindIniPad())
                .remindTwoPad(version.getRemindTwoPad())
                .upgradeTime(version.getUpgradeTime())
                .beginDate(version.getBeginDate())
                .endDate(version.getEndDate())
                .guideDocumentId(version.getGuideDocumentId())
                .project(baseMap.get("project"))
                .part(baseMap.get("part"))
                .btlPartNo(baseMap.get("btlPartNo"))
                .clientPartNo(baseMap.get("clientPartNo"))
                .verson(baseMap.get("verson"))
                .Dmba(baseMap.get("Dmba"))

                .checkBegintime(version.getCheckBegintime())
                .checkEndtime(version.getCheckEndtime())
                .remindIniPad(version.getRemindIniPad())
                .remindTwoPad(version.getRemindTwoPad())
                .upgradeTime(version.getUpgradeTime())

                .versionWorks(workReq)
                .build();
    }

    //计划名称是否重复
    @Override
    public boolean planNameRepetition(String name) {
        List<BentelerPlan> plans = planService.list(Wrappers.<BentelerPlan>lambdaQuery()
                .eq(BentelerPlan::getName, name));
        if (CollectionUtils.isNotEmpty(plans)) {
            return true;
        }
        return false;
    }

    @Override
    public ResponseBean<FileRes> exportInitialPlan(ExportInitialPlanReq req) {
        Long versionId = req.getId();
        getCheckResutGreadReq resutGreadReq = getCheckResutGreadReq.builder()
                .localDate(req.getChooseDate())
                .versionId(versionId)
                .build();

        List<getCheckResutGreadRes> checkResutGread = getCheckResutGread(resutGreadReq);
        BentelerIniVersion version = versionService.getById(versionId);


        //Map<String, String> baseMap = Convert.toMap(String.class, String.class,
        //        JSONUtil.parseObj(version.getBaseMap()));

        Map<String, String> baseMap = JSON.parseObject(version.getBaseMap(), new TypeReference<HashMap<String, String>>() {
        });
        // 模板注意 用{} 来表示你要用的变量 如果本来就有"{","}" 特殊字符 用"\{","\}"代替
        String templateFileName = "systemLib/template/首件计划表.xlsx";

        // 方案1 根据对象填充
        File tempFile = FileUtils.createTempPdf(".xlsx");

        String fileName = TestFileUtil.getPath() + "simpleFill" + System.currentTimeMillis() + ".xlsx";
        // 这里 会填充到第一个sheet， 然后文件流会自动关闭
        int[] a = {6};
        ExcelWriter excelWriter = EasyExcel.write(tempFile, FillData.class)
//                .registerWriteHandler(new ExcelFillCellMergeStrategy(5, a))
                .withTemplate(templateFileName).build();
        WriteSheet writeSheet = EasyExcel.writerSheet().build();
        //新一行
        FillConfig fillConfig = FillConfig.builder().forceNewRow(Boolean.TRUE).build();


        return commonService.exportExcel(FileUtils.createTempPdf(".xlsx"));
    }

    @Override
    public List<getCheckResutGreadRes> getCheckResutGread(getCheckResutGreadReq req) {
        Long versionId = req.getVersionId();
        LocalDate localDate = req.getLocalDate();
        return versionMapper.getVersion_CheckResutGread(versionId, localDate);
    }

    @Override
    public ResponseBean<List<abnormalListRes>> abnormalList() {
        //异常记录
        List<BentelerIniAbnormal> abnormals = iniAbnormalService.list(Wrappers.<BentelerIniAbnormal>lambdaQuery()
                .eq(BentelerIniAbnormal::getSolveFlag, false)
                .like(BentelerIniAbnormal::getOpenTime, LocalDate.now())
        );

        Map<Long, List<BentelerIniAbnormal>> versionid_ab_map = abnormals.stream().collect(Collectors.groupingBy(BentelerIniAbnormal::getVersionId));
        List<abnormalListRes> resList = versionid_ab_map.entrySet().stream().map(item -> {
            BentelerIniVersion version = versionService.getById(item.getKey());
            BentelerPlan plan = planService.getById(version.getPlanId());
            return abnormalListRes.builder().name(plan.getName()).versionId(version.getId()).build();
        }).collect(Collectors.toList());
        return ResponseBeanBuilder.ok(resList);
    }

    @Override
    public ResponseBean<List<BentelerIniWorkstation>> abnormalWorkGroup(BaseTypeReq req) {
        Long versionId = req.getId();
        List<BentelerIniAbnormal> abnormals = iniAbnormalService.list(Wrappers.<BentelerIniAbnormal>lambdaQuery()
                .eq(BentelerIniAbnormal::getSolveFlag, false)
                .like(BentelerIniAbnormal::getOpenTime, LocalDate.now())
                .eq(BentelerIniAbnormal::getVersionId, versionId)
                .eq(BentelerIniAbnormal::getCheckType, req.getType())
        );
        Map<Long, List<BentelerIniAbnormal>> work_map = abnormals.stream().collect(Collectors.groupingBy(BentelerIniAbnormal::getWorkstationId));
        List<BentelerIniWorkstation> works = work_map.keySet().stream()
                .map(bentelerIniAbnormals -> workstationService.getById(bentelerIniAbnormals))
                .collect(Collectors.toList());
        return ResponseBeanBuilder.ok(works);
    }

}
