package com.factory.web.service.impl.statistics;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.base.util.time.TimeUtils;
import com.factory.web.entity.fa.FirstArticleTask;
import com.factory.web.entity.fa.FirstArticleTaskHisBlob;
import com.factory.web.entity.pm.OnsiteTask;
import com.factory.web.entity.pm.OnsiteTaskHisBlob;
import com.factory.web.entity.statistics.ExceptionCountRes;
import com.factory.web.entity.statistics.StatisticsDateReq;
import com.factory.web.mapper.fa.FirstArticleTaskHisBlobMapper;
import com.factory.web.mapper.fa.FirstArticleTaskMapper;
import com.factory.web.mapper.pm.OnsiteTaskHisBlobMapper;
import com.factory.web.mapper.pm.OnsiteTaskMapper;
import com.factory.web.service.statistics.StatisticsService;

import lombok.extern.slf4j.Slf4j;

/**
 * 异常统计
 *
 * @author sMArT
 * @version 1.0
 * @date 2021-07-22 4:03 下午
 */
@Slf4j
@Service("statisticsService")
public class StatisticsServiceImpl implements StatisticsService {
    @Autowired
    private OnsiteTaskMapper onsiteTaskMapper;
    @Autowired
    private OnsiteTaskHisBlobMapper onsiteTaskHisBlobMapper;
    @Autowired
    private FirstArticleTaskMapper firstArticleTaskMapper;
    @Autowired
    private FirstArticleTaskHisBlobMapper firstArticleTaskHisBlobMapper;

    @Override
    public ResponseBean<ExceptionCountRes> getExceptionsByMonth(StatisticsDateReq statisticsDateReq) {

        
        List<String> releasedDates = new ArrayList<>();
        List<String> recheckedDates = new ArrayList<>();
        LocalDateTime[] se = TimeUtils.getMonthStartAndEnd(statisticsDateReq.getMonth());
        //取首件放行状态数量
        List<FirstArticleTask> firstArticleTaskList = firstArticleTaskMapper.selectList(Wrappers.<FirstArticleTask>lambdaQuery()
                .eq(FirstArticleTask::getStatus, 2)
                .between(FirstArticleTask::getTaskTime, se[0],se[1]));
        firstArticleTaskList.forEach(firstArticleTask -> {
            releasedDates.add(TimeUtils.formatLocalDate(firstArticleTask.getTaskTime()));
        });
        //去首件重检数量
        List<FirstArticleTaskHisBlob> firstArticleTaskHisBlobList = firstArticleTaskHisBlobMapper.getFinishRecheckTask(se[0],se[1]);
        firstArticleTaskHisBlobList.forEach(firstArticleTaskHisBlob -> {
            recheckedDates.add(TimeUtils.getDayString(firstArticleTaskHisBlob.getCreatedTime()));
        });

        //取防错放行状态
        List<OnsiteTask> onsiteTaskList = onsiteTaskMapper.selectList(Wrappers.<OnsiteTask>lambdaQuery()
                .eq(OnsiteTask::getStatus, 2)
                .between(OnsiteTask::getTaskTime, se[0],se[1]));
        onsiteTaskList.forEach(onsiteTask -> {
            releasedDates.add(TimeUtils.formatLocalDate(onsiteTask.getTaskTime()));
        });
        //取防错重检数量
        List<OnsiteTaskHisBlob> onsiteTaskHisBlobList = onsiteTaskHisBlobMapper.getFinishRecheckTask(se[0],se[1]);
        onsiteTaskHisBlobList.forEach(onsiteTaskHisBlob -> {
            recheckedDates.add(TimeUtils.getDayString(onsiteTaskHisBlob.getCreatedTime()));
        });
        //合并去重
        List<String> exceptionDays = Stream.of(recheckedDates, releasedDates)
                .flatMap(Collection::stream).distinct().collect(Collectors.toList());

        return ResponseBeanBuilder.ok(ExceptionCountRes.builder()
                .exceptionCount(releasedDates.size() + recheckedDates.size())
                .recheckedCount(recheckedDates.size())
                .releasedCount(releasedDates.size())
                .exceptionDates(exceptionDays)
                .build());
    }

}
