package com.factory.web.service.pm;

import com.baomidou.mybatisplus.extension.service.IService;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.web.entity.pm.*;
import com.factory.web.entity.pm.req.*;
import com.factory.web.entity.pm.res.*;

import java.util.List;

/**
 * 防错计划
 *
 * @author wzy
 * @date 2021-07-09 12:00:00
 */
public interface PreventMistakeService extends IService<BentelerPlan> {


	/**
	 * 新建防错计划
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean<Void> addPmPlan(AddPmPlanReq req);

	/**
	 * 校验计划名是否存在
	 * 
	 * @param name
	 * @return true存在，false不存在
	 */
	public boolean checkPlanNameExist(String name,Long originalId);

	/**
	 * 校验工作站名是否存在
	 * 
	 * @param wsName
	 * @return true存在，false不存在
	 */
	public boolean checkWsNameExist(String wsName);

	/**
	 * 取得防错计划点检列表
	 * 
	 * @param req
	 * @return 防错计划点检列表
	 */
	public ResponseBean<PageBean<PmPlanCheckListRes>> getPmPlanCheckList(SearchPmPlanCheckListReq req);

	/**
	 * 取得防错计划列表
	 * 
	 * @param req
	 * @return 防错计划列表
	 */
	public ResponseBean<PageBean<PmPlanListRes>> getPmPlanList(SearchPmPlanListReq req);

	/**
	 * 取得防错计划当天任务信息
	 * 
	 * @param req
	 * @return 防错计划当天任务信息
	 */
	public List<PmTaskListRes> getTaskList(SearchPmPlanReq req);

	/**
	 * 取得防错计划信息
	 * 
	 * @param req
	 * @return 防错计划信息
	 */
	public PmPlanRes getPmPlan(SearchPmPlanReq req);

	/**
	 * 禁用解禁防错计划
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean<Void> disablePmPlan(PmPlanPauseFlagReq req);

	/**
	 * 取得工作站信息
	 * 
	 * @param req
	 * @return 工作站信息
	 */
	public ResponseBean<OnsiteWorkshop> getWorkshop(SearchWorkshopReq req);

	/**
	 * 修改防错计划
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean<Void> editPmPlan(EditPmPlanReq req);

	/**
	 * 取得计划审批信息
	 * 
	 * @param req
	 * @return 计划审批信息
	 */
	public ResponseBean<List<OnsiteApprovalPlan>> getPmPlanApprovalList(SearchPmPlanReq req);

	/**
	 * 取得巡检规则信息
	 * 
	 * @param req
	 * @return 巡检规则信息
	 */
	public ResponseBean<List<OnsiteCalendar>> getCalendarList(SearchPmPlanReq req);

	/**
	 * 取得检查记录列表
	 * 
	 * @param req
	 * @return 检查记录列表
	 */
	public ResponseBean<List<PmCheckRecordListRes>> getPmCheckRecordList(SearchPmCheckRecordListReq req);

	/**
	 * 取得Pad端防错计划列表
	 * 
	 * @param req
	 * @return Pad端防错计划列表
	 */
	public ResponseBean<List<PmPlanListForPadRes>> getPmPlanListForPad(SearchPmPlanListForPadReq req);

	/**
	 * 取得表头信息
	 * 
	 * @param bentelerPlanId
	 * @return 表头信息
	 */
	public OnsiteTitle getTitle(Long bentelerPlanId);

	/**
	 * 取得工作站列表
	 * 
	 * @param req
	 * @return 工作站列表
	 */
	public ResponseBean<List<OnsiteWorkshop>> getWorkshopList(SearchPmAllCheckFlagReq req);

	/**
	 * 取得巡检项目列表
	 * 
	 * @param req
	 * @return 巡检项目列表
	 */
	public ResponseBean<List<OnsiteItem>> getItemList(SearchItemListReq req);

	/**
	 * 添加巡检项目检查结果
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean<Void> addItemResult(AddItemResultListReq req);

	/**
	 * 任务提交
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean<Void> submitTask(SearchTaskReq req);

	/**
	 * 取得审批人
	 * 
	 * @param req
	 * @return 审批人
	 */
	public ResponseBean<ApprovalUserRes> getApprovalUser(SearchApprovalUserReq req);
	

    /**
     * 取得审批人
     *
     * @param approvalLevel 1是1级审批 2是2级审批
     * @param type          1计划审批  2首件任务审批  3防错任务审批
     * @return 审批人
     */
    public ApprovalUserRes getApprovalUserInfo(int approvalLevel, int type);


	/**
	 * 取得每频次点检信息
	 * 
	 * @param req
	 * @return 每频次点检信息
	 */
	public ResponseBean<List<PmCheckListRes>> getPmCheckList(SearchPmPlanReq req);
	
	/**
	 * 取得不放行任务对应的工作站
	 * 
	 * @param req
	 * @return 不放行任务对应的工作站
	 */
	public ResponseBean<List<PmWorkshopNoGoListRes>> getPmWorkshopNoGoList(SearchPmWorkshopNoGoListReq req);
	
	/**
	 * 取得不放行任务对应的点检项目
	 * 
	 * @param req
	 * @return 不放行任务对应的点检项目
	 */
	public ResponseBean<List<PmItemNoGoListRes>> getPmItemNoGoList(SearchPmItemNoGoListReq req);
	
	/**
	 * 取得不放行任务对应的质量经理审批信息
	 * 
	 * @param req
	 * @return 不放行任务对应的质量经理审批信息
	 */
	public ResponseBean<PmNoGoTaskAapprovalRes> getPmNoGoTaskAapproval(SearchTaskReq req);
	
	/**
	 * 修改巡检项目检查结果(这里只有重检走这里)
	 * 
	 * @param req
	 * @return
	 */
	public ResponseBean<Void> editItemResult(EditItemResultListReq req);
	


    /**
     * 修改防错计划
     *
     * @param bentelerPlanId
     * @return
     */
    public void setOnsitePlanHis(Long bentelerPlanId);

    /**
     * 任务和巡检结果记录历史
     *
     * @param onsiteTaskId
     * @return
     */
    public void setOnsiteTaskHis(Long onsiteTaskId);

    /**
     * 取得所有工作站
     *
     * @param req
     * @return 所有工作站
     */
    public ResponseBean<List<PmAllWorkshopListRes>> getAllWorkshopList(SearchAllWorkshopReq req);

    /**
     * 取得计划的最新审批信息
     *
     * @param req
     * @return 计划的最新审批信息
     */
    public OnsiteApprovalPlan getLatestApprovalInfo(SearchPmPlanReq req);

    /**
     * 判断检验结果是否填写完成
     *
     * @param req
     * @return 判断检验结果是否填写完成
     */
    public PmAllCheckFlagRes getPmAllCheckFlag(SearchPmAllCheckFlagReq req);

    /**
     * 取得异常列表
     *
     * @param req
     * @return 异常列表
     */
    public ResponseBean<List<AbnormalTaskInfoRes>> getAbnormalList(SearchAbnormalListReq req);

    /**
     * task二级审批不通过的，即为重检，此时将点检结果item_result为不通过的1都改成成-1，这样重检时相当于重新开始，没有之前首次点检的结果
     *
     * @param onsiteTaskId
     * @return
     */
    public void setPmRepeatResult(long onsiteTaskId);
    
    /**
     * 审批信息和通知
     * @param approvalRuleNextLevel
     * @param taskId
     * @param userId
     */
    public void setApproval(ApprovalRule approvalRuleNextLevel,Long taskId,Long userId) ;

}
