package com.factory.wx.service.impl;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.base.util.time.TimeUtils;
import com.factory.web.entity.fa.FirstArticleApprovalTask;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.pm.OnsiteApprovalTask;
import com.factory.web.entity.pm.OnsiteTitle;
import com.factory.web.entity.pm.OnsiteWorkshop;
import com.factory.web.entity.pm.req.SearchAbnormalReq;
import com.factory.web.entity.pm.res.AbnormalApprovalTaskRes;
import com.factory.web.entity.pm.res.AbnormalItemRes;
import com.factory.web.entity.pm.res.AbnormalItemResultRes;
import com.factory.web.entity.pm.res.AbnormalTaskInfoRes;
import com.factory.web.entity.pm.res.AbnormalWorkshopRes;
import com.factory.web.entity.statistics.StatisticsDateReq;
import com.factory.web.mapper.BentelerPlanMapper;
import com.factory.web.mapper.FileMapper;
import com.factory.web.mapper.pm.PreventMistakeMapper;
import com.factory.web.service.fa.FirstArticleApprovalTaskService;
import com.factory.web.service.fa.FirstArticleService;
import com.factory.web.service.fa.FirstArticleWorkshopService;
import com.factory.web.service.impl.FileEntry;
import com.factory.web.service.pm.OnsiteApprovalTaskService;
import com.factory.web.service.pm.OnsiteWorkshopService;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.AbnormalDetailReq;
import com.factory.wx.entity.req.AbnormalPlanReq;
import com.factory.wx.entity.res.AbnormalDetailRes;
import com.factory.wx.entity.res.AbnormalPlanRes;
import com.factory.wx.mapper.AbnormalStatisticsMapper;
import com.factory.wx.service.AbnormalStatisticsService;
import com.factory.wx.service.WxPlanService;

import lombok.extern.slf4j.Slf4j;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-23 1:33 上午
 */
@Slf4j
@Service("abnormalStatisticsService")
public class AbnormalStatisticsServiceImpl implements AbnormalStatisticsService {
    @Autowired
    private AbnormalStatisticsMapper abnormalStatisticsMapper;
    @Autowired
    private FileMapper fileMapper;
    @Autowired
    private BentelerPlanMapper bentelerPlanMapper;
    @Autowired
    private PreventMistakeMapper preventMistakeMapper;
    @Autowired
    private FirstArticleWorkshopService firstArticleWorkshopService;
    @Autowired
    private FirstArticleService firstArticleService;
    @Autowired
    private OnsiteWorkshopService onsiteWorkshopService;
    @Autowired
    private PreventMistakeService preventMistakeService;
    @Autowired
    private WxPlanService wxPlanService;
    @Autowired
    private OnsiteApprovalTaskService onsiteApprovalTaskService;
    @Autowired
    private FirstArticleApprovalTaskService firstArticleApprovalTaskService;

    /**
     * 根据年月获取异常计划
     *
     * @param abnormalPlanReq
     * @return
     */
    @Override
    public List<AbnormalPlanRes> getExceptionPlansByMonth(AbnormalPlanReq abnormalPlanReq) {
    	LocalDateTime month = getMonth(abnormalPlanReq);
        
        LocalDateTime[] se = TimeUtils.getStartAndEndDayTime(month, 3, 0);
        abnormalPlanReq.setSdate(se[0]);
        abnormalPlanReq.setEdate(se[1]);
        List<AbnormalPlanRes> releasedAbnormalPlans = new ArrayList<>();
        //已重检
        if (abnormalPlanReq.getAbnormalType() == 1) {
            releasedAbnormalPlans.addAll(abnormalStatisticsMapper.getRecheckedAbnormalPlans(abnormalPlanReq));
        }
        //已放行
        if (abnormalPlanReq.getAbnormalType() == 2) {
            releasedAbnormalPlans.addAll(abnormalStatisticsMapper.getReleasedAbnormalPlans(abnormalPlanReq));
        }
        return releasedAbnormalPlans;
    }

    /**
     * 异常放行详情
     *
     * @param abnormalDetailReq
     * @return
     */
    @Override
    public AbnormalDetailRes getAbnormalDetail(AbnormalDetailReq abnormalDetailReq) {
        AbnormalDetailRes abnormalDetailRes = new AbnormalDetailRes();
        BentelerPlan bentelerPlan = bentelerPlanMapper.selectById(abnormalDetailReq.getPlanId());
        abnormalDetailRes.setPlanId(bentelerPlan.getId());
        abnormalDetailRes.setPlanName(bentelerPlan.getName());
        abnormalDetailRes.setPlanType(bentelerPlan.getPlanType());
        if (bentelerPlan.getPlanType() == 1) {
            //获取title
            FirstArticleTitle title = firstArticleService.getFaTitle(abnormalDetailReq.getPlanId());
            //判断是否有附件
            int attrCount = fileMapper.selectCount(Wrappers.<FileEntry>lambdaQuery()
                    .eq(FileEntry::getId, title.getUploadAttachmentId()));
            abnormalDetailRes.setFirstArticleTitle(title);
            abnormalDetailRes.setAttachment(attrCount == 0 ? 0 : 1);
            //取放行信息
            List<AbnormalTaskInfoRes> abnormalTaskInfoResList = getFaAbnormalTaskInfoRes(abnormalDetailReq);
            abnormalDetailRes.setAbnormalTaskInfos(abnormalTaskInfoResList);
        }else if (bentelerPlan.getPlanType() == 2) {
            //获取title
            OnsiteTitle title = preventMistakeService.getTitle(abnormalDetailReq.getPlanId());
            //判断是否有附件
            int attrCount = fileMapper.selectCount(Wrappers.<FileEntry>lambdaQuery()
                    .eq(FileEntry::getId, title.getUploadAttachmentId()));
            abnormalDetailRes.setTitleInfo(title);
            abnormalDetailRes.setAttachment(attrCount == 0 ? 0 : 1);
            //取放行信息
            List<AbnormalTaskInfoRes> abnormalTaskInfoResList = getPMAbnormalTaskInfoRes(abnormalDetailReq);
            abnormalDetailRes.setAbnormalTaskInfos(abnormalTaskInfoResList);
        }

        return abnormalDetailRes;
    }

    /**
     * 获取首件信息
     *
     * @param abnormalDetailReq
     * @return
     */
    private List<AbnormalTaskInfoRes> getFaAbnormalTaskInfoRes(AbnormalDetailReq abnormalDetailReq) {
        List<AbnormalTaskInfoRes> faList = new ArrayList<>();
        if (abnormalDetailReq.getAbnormalType() == 1) {
            //重检内容
            faList = abnormalStatisticsMapper.getFaRecheckedListByPlanId(abnormalDetailReq.getPlanId(),abnormalDetailReq.getTaskId());
        }else  {//if (abnormalDetailReq.getAbnormalType() == 2)
            //放行内容
            faList = abnormalStatisticsMapper.getFaReleaseListByPlanId(abnormalDetailReq.getPlanId(),abnormalDetailReq.getTaskId());
        }

        for (AbnormalTaskInfoRes faInfo : faList) {
            List<FirstArticleWorkshop> firstArticleWorkshopList = firstArticleWorkshopService.list(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                    .eq(FirstArticleWorkshop::getBentelerPlanId, faInfo.getBentelerPlanId()));

            List<AbnormalWorkshopRes> abnormalWorkshopList = new ArrayList<>();
            AbnormalWorkshopRes abnormalWorkshop = null;
            for (FirstArticleWorkshop workshop : firstArticleWorkshopList) {
                abnormalWorkshop = new AbnormalWorkshopRes();
                abnormalWorkshop.setWorkshopId(workshop.getId());
                abnormalWorkshop.setWsName(workshop.getWsName());
                abnormalWorkshopList.add(abnormalWorkshop);
            }

            for (int i = 0; i < abnormalWorkshopList.size(); i++) {

                SearchAbnormalReq searchReq = new SearchAbnormalReq();
                searchReq.setWorkshopId(abnormalWorkshopList.get(i).getWorkshopId());
                searchReq.setTaskId(faInfo.getTaskId());
                List<AbnormalItemRes> abnormalItemList = abnormalStatisticsMapper.getFaAbnormalItemResultList(searchReq);

                for (AbnormalItemRes abnormalItemInfo : abnormalItemList) {

                    Long userId = abnormalItemInfo.getItemResultUserId();
                    if (null != userId) {
                        String userName = wxPlanService.getUserNameById(abnormalItemInfo.getItemResultUserId());
                        abnormalItemInfo.setItemResultUser(userName);
                    }
                    if (faInfo.getTypeFlag().equals(0)) {
                        List<AbnormalItemResultRes> firstItemResultList = this.getAbnormalItemResultList(faInfo.getTaskId(), abnormalItemInfo.getItemId(), 0);
                        abnormalItemInfo.setFirstItemResultList(firstItemResultList);
                    }
                    if (faInfo.getTypeFlag().equals(1)) {
                        List<AbnormalItemResultRes> twoHundredItemResultList = this.getAbnormalItemResultList(faInfo.getTaskId(), abnormalItemInfo.getItemId(), 1);
                        abnormalItemInfo.setTwoHundredItemResultList(twoHundredItemResultList);
                    }
                    if (faInfo.getTypeFlag().equals(2)) {
                        List<AbnormalItemResultRes> catchItemResultList = this.getAbnormalItemResultList(faInfo.getTaskId(), abnormalItemInfo.getItemId(), 2);
                        abnormalItemInfo.setCatchItemResultList(catchItemResultList);
                    }
                }

                abnormalWorkshopList.get(i).setAbnormalItemList(abnormalItemList);
            }
            faInfo.setAbnormalWorkshopList(abnormalWorkshopList.stream().filter(workshop -> null != workshop.getAbnormalItemList() && workshop.getAbnormalItemList().size() > 0).collect(Collectors.toList()));

            if (abnormalDetailReq.getAbnormalType() == 1) {
                for (AbnormalTaskInfoRes task : faList) {
                    List<FirstArticleApprovalTask> approvalTasks = firstArticleApprovalTaskService.list(Wrappers.<FirstArticleApprovalTask>lambdaQuery()
                            .eq(FirstArticleApprovalTask::getFirstArticleTaskId, task.getTaskId())
                            .orderByAsc(FirstArticleApprovalTask::getApprovalLevel)
                            );
                    List<AbnormalApprovalTaskRes> resultInfos = new ArrayList<>();
                    for (FirstArticleApprovalTask t : approvalTasks) {
                        AbnormalApprovalTaskRes o = new AbnormalApprovalTaskRes();
                        o.setStatus(t.getStatus());
                        o.setReason(t.getReason());
                        o.setUpdatedTime(t.getUpdatedTime());
                        o.setUsername(task.getPerson());
                        resultInfos.add(o);
                        if(t.getStatus()==2) {
                        	task.setReason(t.getReason());
                            task.setUpdatedTime(t.getUpdatedTime());
                        }
                    }
                    task.setAbnormalApprovalTaskResList(resultInfos);
                }
            }

        }
        return faList;
    }

    /**
     * 获取防错信息
     *
     * @param abnormalDetailReq
     * @return
     */
    private List<AbnormalTaskInfoRes> getPMAbnormalTaskInfoRes(AbnormalDetailReq abnormalDetailReq) {
        List<AbnormalTaskInfoRes> pmList = new ArrayList<>();
        if (abnormalDetailReq.getAbnormalType() == 1) {
            //重检内容
            pmList = abnormalStatisticsMapper.getPmRecheckedListByPlanId(abnormalDetailReq.getPlanId(),abnormalDetailReq.getTaskId());
        }else   {//if (abnormalDetailReq.getAbnormalType() == 2)
            //放行内容
            pmList = abnormalStatisticsMapper.getPmReleaseListByPlanId(abnormalDetailReq.getPlanId(),abnormalDetailReq.getTaskId());
        }
        for (AbnormalTaskInfoRes pmInfo : pmList) {
            List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopService.list(Wrappers.<OnsiteWorkshop>lambdaQuery()
                    .eq(OnsiteWorkshop::getBentelerPlanId, pmInfo.getBentelerPlanId()));

            List<AbnormalWorkshopRes> abnormalWorkshopList = new ArrayList<AbnormalWorkshopRes>();
            for (OnsiteWorkshop workshop : onsiteWorkshopList) {
                AbnormalWorkshopRes abnormalWorkshop = new AbnormalWorkshopRes();
                abnormalWorkshop.setWorkshopId(workshop.getId());
                abnormalWorkshop.setWsName(workshop.getWsName());
                abnormalWorkshopList.add(abnormalWorkshop);
            }

            for (int i = 0; i < abnormalWorkshopList.size(); i++) {

                SearchAbnormalReq searchReq = new SearchAbnormalReq();
                searchReq.setWorkshopId(abnormalWorkshopList.get(i).getWorkshopId());
                searchReq.setTaskId(pmInfo.getTaskId());
                //放行或者重检通过
				List<AbnormalItemRes> abnormalItemList = abnormalStatisticsMapper.getPmAbnormalItemResultList(searchReq)
						.stream().filter(abnormalItemRes -> abnormalItemRes.getResult() > 0
								|| abnormalItemRes.getResultBak() > 0)
						.collect(Collectors.toList());
				  for (AbnormalItemRes abnormalItemRes : abnormalItemList) {
					  if(abnormalItemRes.getResult()==0) {
						  abnormalItemRes.setResult(abnormalItemRes.getResultBak());
					  }
                    Long userId = abnormalItemRes.getItemResultUserId();
                    if (null != userId) {
                        String userName = wxPlanService.getUserNameById(abnormalItemRes.getItemResultUserId());
                        abnormalItemRes.setItemResultUser(userName);
                    }
                }
                abnormalWorkshopList.get(i).setAbnormalItemList(abnormalItemList);
            }

            if (abnormalDetailReq.getAbnormalType() == 1) {
                for (AbnormalTaskInfoRes task : pmList) {
                    List<OnsiteApprovalTask> taskList = onsiteApprovalTaskService.list(Wrappers.<OnsiteApprovalTask>lambdaQuery()
                            .eq(OnsiteApprovalTask::getOnsiteTaskId, task.getTaskId()));
                    List<AbnormalApprovalTaskRes> resultInfos = new ArrayList<>();
                    for (OnsiteApprovalTask t : taskList) {
                        AbnormalApprovalTaskRes o = new AbnormalApprovalTaskRes();
                        o.setStatus(t.getStatus());
                        o.setReason(t.getReason());
                        o.setUpdatedTime(t.getUpdatedTime());
                        o.setUsername(task.getPerson());
                        resultInfos.add(o);
                    }
                    task.setAbnormalApprovalTaskResList(resultInfos);
                }
            }

            pmInfo.setAbnormalWorkshopList(abnormalWorkshopList.stream().filter(workshop -> workshop.getAbnormalItemList().size() > 0).collect(Collectors.toList()));
        }
        return pmList;
    }

    /**
     * 取得检查结果
     *
     * @param taskId
     * @param itemId
     * @param typeFlag 0表示首件 1表示200件 2表示中断
     * @return 检查结果
     */
    private List<AbnormalItemResultRes> getAbnormalItemResultList(long taskId, long itemId, int typeFlag) {
        SearchAbnormalReq searchReq = new SearchAbnormalReq();
        searchReq.setTaskId(taskId);
        searchReq.setItemId(itemId);
        searchReq.setTypeFlag(typeFlag);
        List<AbnormalItemResultRes> abnormalItemResultList = preventMistakeMapper.getFaAbnormalItemResultList(searchReq);
        return abnormalItemResultList;
    }

    /**
     * 验证日期格式
     *
     * @param abnormalPlanReq 默认2021-06
     * @return
     */
    private LocalDateTime getMonth(AbnormalPlanReq statisticsDateReq) {
    	LocalDateTime month = LocalDateTime.now();
        try {
            month = TimeUtils.paserMonthString(statisticsDateReq.getMonth());
        } catch (Exception e) {
            log.error("年月格式错误",e);
            ResponseBeanBuilder.fail("年月格式错误");
        }
        return month;
    }

}
