package com.factory.wx.service.impl;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.factory.base.entity.aggregates.PageBean;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.enumPo.PlanStatus;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.service.UserRoleService;
import com.factory.web.entity.fa.FirstArticleItem;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.pm.ApprovalRule;
import com.factory.web.entity.pm.OnsiteApprovalPlan;
import com.factory.web.entity.pm.OnsiteItem;
import com.factory.web.entity.pm.OnsiteTitle;
import com.factory.web.entity.pm.OnsiteWorkshop;
import com.factory.web.mapper.BentelerPlanMapper;
import com.factory.web.mapper.fa.FirstArticleItemMapper;
import com.factory.web.mapper.fa.FirstArticleWorkshopMapper;
import com.factory.web.mapper.pm.ApprovalRuleMapper;
import com.factory.web.mapper.pm.OnsiteApprovalPlanMapper;
import com.factory.web.mapper.pm.OnsiteItemMapper;
import com.factory.web.mapper.pm.OnsiteWorkshopMapper;
import com.factory.web.service.BentelerPlanService;
import com.factory.web.service.pm.OnsiteApprovalPlanService;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.PlanApprovalReq;
import com.factory.wx.entity.req.PlanInfoReq;
import com.factory.wx.entity.req.PlanPageReq;
import com.factory.wx.entity.res.PlanApprovalDetailRes;
import com.factory.wx.entity.res.WxPlanBaseInfoRes;
import com.factory.wx.service.PlanApprovalService;
import com.factory.wx.service.WxPlanService;

import cn.hutool.core.collection.CollUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-13 12:42 下午
 */
@Slf4j
@Service("planApprovalService")
public class PlanApprovalServiceImpl implements PlanApprovalService {

    
    @Autowired
    private BentelerPlanMapper bentelerPlanMapper;
    @Autowired
    private OnsiteWorkshopMapper onsiteWorkshopMapper;
    @Autowired
    private OnsiteItemMapper onsiteItemMapper;
    @Autowired
    private OnsiteApprovalPlanMapper onsiteApprovalPlanMapper;
    @Autowired
	private OnsiteApprovalPlanService onsiteApprovalPlanService;
    @Autowired
    private ApprovalRuleMapper approvalRuleMapper;
    @Autowired
    private PreventMistakeService preventMistakeService;
    @Autowired
    private FirstArticleWorkshopMapper firstArticleWorkshopMapper;
    @Autowired
    private FirstArticleItemMapper firstArticleItemMapper;
    @Autowired
    private WxPlanService wxPlanService;
    
    @Autowired
    private BentelerPlanService planService;
    
    @Autowired
    private UserRoleService userRoleService;

    /**
     * 分页显示计划列表
     *
     * @param planPageReq
     * @return
     */
    @Override
    public PageBean<BentelerPlan> getApprovalListByPage(PlanPageReq planPageReq) {
    	
    	IPage<BentelerPlan> page = new Page<>(planPageReq.getPage(), planPageReq.getSize());
    	List<Long> roleIds =userRoleService.getRoleIdsByUserId(planPageReq.getUserId());
    	if(!roleIds.isEmpty()) {
	        LambdaQueryWrapper<OnsiteApprovalPlan> approvalPlanWrapper = new LambdaQueryWrapper<>();
	        approvalPlanWrapper.in(OnsiteApprovalPlan::getApprovalRole, roleIds);
	        approvalPlanWrapper.eq(OnsiteApprovalPlan::getStatus, 0);//获取未处理过的计划
	        List<OnsiteApprovalPlan> onsiteApprovalPlanList = onsiteApprovalPlanMapper.selectList(approvalPlanWrapper);
	
	        if (CollUtil.isNotEmpty(onsiteApprovalPlanList)) {
	            LambdaQueryWrapper<BentelerPlan> wrapper = new LambdaQueryWrapper<>();
	            List<Long> planIds = onsiteApprovalPlanList.stream().map(OnsiteApprovalPlan::getBentelerPlanId).collect(Collectors.toList());
	            wrapper.in(BentelerPlan::getId, planIds);
	
	            if (StringUtils.isNotEmpty(planPageReq.getName())) {
	                wrapper.like(BentelerPlan::getName, planPageReq.getName());
	            }
	            wrapper.eq(BentelerPlan::getDeleted, 0);//未删除
	            wrapper.eq(BentelerPlan::getPauseFlag, 0);//未禁用
	            wrapper.in(BentelerPlan::getStatus, PlanStatus.APPROVAL_REFUSE.value,PlanStatus.WAIT_APPROVAL.value);
	            
	            page = bentelerPlanMapper.selectPage(page, wrapper);
	        }
    	}

        return PageBean.<BentelerPlan>builder().row(page.getRecords()).total(page.getTotal()).page(page.getPages()).build();
    }

    /**
     * 获取审批计划详情-首件
     *
     * @param planInfoReq
     * @return
     */
    @Override
    public PlanApprovalDetailRes getApprovalFADetail(PlanInfoReq planInfoReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes =
                wxPlanService.getFirstArticlePlanBaseInfo(planInfoReq.getPlanId());
        List<FirstArticleWorkshop> firstArticleWorkshopList =
                firstArticleWorkshopMapper.selectList(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                        .eq(FirstArticleWorkshop::getBentelerPlanId, planInfoReq.getPlanId()));
        firstArticleWorkshopList.forEach(firstArticleWorkshop -> {
            firstArticleWorkshop.setFirstArticleItemList(firstArticleItemMapper.selectList(Wrappers.<FirstArticleItem>lambdaQuery()
                    .eq(FirstArticleItem::getWorkshopId, firstArticleWorkshop.getId())));
        });

        List<Integer> itemType = getFAItemFrequency(firstArticleWorkshopList);

        return PlanApprovalDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(itemType)
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .firstArticleTitle((FirstArticleTitle) wxPlanBaseInfoRes.getTitle())
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .firstArticleWorkshops(firstArticleWorkshopList)
                .build();
    }

    /**
     * 首件频次归类
     *
     * @param firstArticleWorkshopList
     * @return
     */
    private List<Integer> getFAItemFrequency(List<FirstArticleWorkshop> firstArticleWorkshopList) {
        List<Integer> itemType = new ArrayList<>();
        firstArticleWorkshopList.forEach(workshop -> {
            workshop.getFirstArticleItemList().forEach(item -> {
                if (item.getFirstFlag() == 1) itemType.add(0);
                if (item.getTwoHundredFlag() == 1) itemType.add(1);
                if (item.getCatchFlag() == 1) itemType.add(2);
            });
        });
        return itemType.stream().distinct().collect(Collectors.toList());
    }


    /**
     * 获取审批计划详情-防错
     *
     * @param planInfoReq
     * @return
     */
    @Override
    public PlanApprovalDetailRes getApprovalPMDetail(PlanInfoReq planInfoReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes =
                wxPlanService.getOnsitePlanBaseInfo(planInfoReq.getPlanId());

        List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopMapper.selectList(Wrappers.<OnsiteWorkshop>lambdaQuery()
                .eq(OnsiteWorkshop::getBentelerPlanId, planInfoReq.getPlanId()));
        onsiteWorkshopList.forEach(onsiteWorkshop -> {
            onsiteWorkshop.setOnsiteItemList(onsiteItemMapper.selectList(Wrappers.<OnsiteItem>lambdaQuery()
                    .eq(OnsiteItem::getWorkshopId, onsiteWorkshop.getId())));
        });
        List<Integer> itemType = getPMItemFrequency(onsiteWorkshopList);

        return PlanApprovalDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(itemType)
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .titleInfo((OnsiteTitle) wxPlanBaseInfoRes.getTitle())
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .onsiteWorkshops(onsiteWorkshopList)
                .build();
    }

    /**
     * 获取防错频次
     *
     * @param onsiteWorkshopList
     */
    private List<Integer> getPMItemFrequency(List<OnsiteWorkshop> onsiteWorkshopList) {
        List<Integer> frequency = new ArrayList<>();
        onsiteWorkshopList.forEach(workshop -> {
            workshop.getOnsiteItemList().forEach(item -> {
                frequency.add(item.getTypeFlag());
            });
        });
        return frequency.stream().distinct().collect(Collectors.toList());
    }

    /**
     * 计划审批
     *
     * @param planApprovalReq
     * @return
     */
    @Override
    public ResponseBean<Void> approvalOperate(PlanApprovalReq planApprovalReq) {
    	List<Long> roleIds =userRoleService.getRoleIdsByUserId(planApprovalReq.getUserId());
    	if(roleIds.isEmpty()) {
    		 ResponseBeanBuilder.fail("无权限审批");
    	}
        OnsiteApprovalPlan onsiteApprovalPlan = new OnsiteApprovalPlan();
        onsiteApprovalPlan.setStatus(planApprovalReq.getStatus());
        onsiteApprovalPlan.setApprovalUserId(planApprovalReq.getUserId());
        
        OnsiteApprovalPlan old = onsiteApprovalPlanMapper.selectOne( Wrappers.<OnsiteApprovalPlan>lambdaQuery()
                .eq(OnsiteApprovalPlan::getBentelerPlanId, planApprovalReq.getPlanId())
                .eq(OnsiteApprovalPlan::getStatus, 0));
        if(old==null) {
        	ResponseBeanBuilder.fail("请勿重复审批");
        }
        if(!roleIds.contains(old.getApprovalRole())) {
        	 ResponseBeanBuilder.fail("无权限审批");
        }
        onsiteApprovalPlan.setId(old.getId());
        int updateStatus = onsiteApprovalPlanMapper.updateById(onsiteApprovalPlan);
        if (updateStatus > 0) {
            //审批通过触发下一级别
            if (planApprovalReq.getStatus() == 1) {
            	//获取当前审批级别
            	int userLevel = old.getApprovalLevel();
            	ApprovalRule approvalRuleNextLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
        				.eq(ApprovalRule::getType, ApprovalRule.APPROVAL_TYPE_NEW)
        				.gt(ApprovalRule::getApprovalLevel, userLevel)
        				.orderByAsc(ApprovalRule::getApprovalLevel)
        				.last("limit 1"));
                if (approvalRuleNextLevel!=null) {
                	onsiteApprovalPlanService.save(OnsiteApprovalPlan.builder()
        					.bentelerPlanId(planApprovalReq.getPlanId())
        					.approvalLevel(approvalRuleNextLevel.getApprovalLevel()) // 1级审批
        					.approvalRole(approvalRuleNextLevel.getRoleId())
        					.status(0) // 0未审批
        					.build());
                	
                }else {//TODO 预计到生效时间暂停旧计划开启新计划
                	BentelerPlan plan = planService.getById(planApprovalReq.getPlanId());
                	BentelerPlan planUpdate = BentelerPlan.builder()
                			.id(planApprovalReq.getPlanId())
                			.status(PlanStatus.APPROVAL_PASS.value)
                			.build();
                	if(!LocalDate.now().isBefore(plan.getEffectiveDate())) {//有效期已经过了当前日期了,那么立刻生效
                		if(plan.getPrevId()!=null) {
                			planService.lambdaUpdate().eq(BentelerPlan::getOriginalId,plan.getOriginalId())
            				.in(BentelerPlan::getStatus, PlanStatus.APPROVAL_FORMAL.value,PlanStatus.APPROVAL_PASS.value)
            				.lt(BentelerPlan::getId,plan.getId())
            				.set(BentelerPlan::getStatus, PlanStatus.HISTORY.value).update();
                		}
                		planUpdate.setStatus(PlanStatus.APPROVAL_FORMAL.value);
                	}
        			planService.updateById(planUpdate);
                }
            }
            //审批不通过
            if (planApprovalReq.getStatus() == 2) {
                //审批不通过，保存审批历史
                preventMistakeService.setOnsitePlanHis(planApprovalReq.getPlanId());
            }
        }
        return ResponseBeanBuilder.ok();
    }

}
