package com.factory.wx.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.service.UserRoleService;
import com.factory.web.entity.fa.FirstArticleApprovalTask;
import com.factory.web.entity.fa.FirstArticleCalendar;
import com.factory.web.entity.fa.FirstArticleTask;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.pm.ApprovalRule;
import com.factory.web.mapper.fa.FirstArticleApprovalTaskMapper;
import com.factory.web.mapper.pm.ApprovalRuleMapper;
import com.factory.web.service.fa.FirstArticleCalendarService;
import com.factory.web.service.fa.FirstArticleService;
import com.factory.web.service.fa.FirstArticleTaskService;
import com.factory.web.service.fa.FirstArticleWorkshopService;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.PlanApprovalTaskReq;
import com.factory.wx.entity.req.PlanPassReq;
import com.factory.wx.entity.req.SearchPlanPassReq;
import com.factory.wx.entity.req.SendMsgReq;
import com.factory.wx.entity.res.PlanPassDetailRes;
import com.factory.wx.entity.res.PlanPassItemRes;
import com.factory.wx.entity.res.PlanPassItemResultRes;
import com.factory.wx.entity.res.PlanPassTaskRes;
import com.factory.wx.entity.res.PlanPassWorkshopRes;
import com.factory.wx.entity.res.WxPlanBaseInfoRes;
import com.factory.wx.mapper.PlanPassMapper;
import com.factory.wx.service.PlanPassFAService;
import com.factory.wx.service.WxPlanService;
import com.factory.wx.service.WxService;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-08-03 10:29 上午
 */

@Service("planPassFAService")
public class PlanPassFAServiceImpl implements PlanPassFAService {
    /**
     * 审批规则类型为：2首件
     */
    @Autowired
    private PlanPassMapper planPassMapper;
    @Autowired
    private FirstArticleCalendarService firstArticleCalendarService;
    @Autowired
    private FirstArticleTaskService firstArticleTaskService;
    @Autowired
    private FirstArticleApprovalTaskMapper firstArticleApprovalTaskMapper;
    @Autowired
    private FirstArticleService firstArticleService;
    @Autowired
    private WxPlanService wxPlanService;
    @Autowired
    private FirstArticleWorkshopService firstArticleWorkshopService;
    @Autowired
    private ApprovalRuleMapper approvalRuleMapper;
    @Autowired
    private PreventMistakeService preventMistakeService;
    @Autowired
    private WxService wxService;
    @Autowired
    private UserRoleService userRoleService;
    
    
    /**
     * 计划放行详情-首件
     *
     * @param planPassReq
     * @return
     */
    @Override
    public PlanPassDetailRes getPassFADetail(PlanPassReq planPassReq) {
    	List<Long> roleIds = userRoleService.getRoleIdsByUserId(planPassReq.getUserId());
    	planPassReq.setRoleIds(roleIds);
        WxPlanBaseInfoRes wxPlanBaseInfoRes = wxPlanService.getFirstArticlePlanBaseInfo(planPassReq.getPlanId());
        List<PlanPassTaskRes> planPassTaskResList = null;
        List<Integer> itemType = wxPlanBaseInfoRes.getFrequency();
        if(!roleIds.isEmpty()) {
        	planPassTaskResList = getFATaskList(planPassReq);
//            itemType = getItemFAFrequency(planPassTaskResList);
        }
       

        return PlanPassDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(itemType)
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .firstArticleTitle((FirstArticleTitle) wxPlanBaseInfoRes.getTitle())
                .taskList(planPassTaskResList)
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .build();
    }

    /**
     * 获取首件task
     *
     * @param planPassReq
     * @return
     */
    private List<PlanPassTaskRes> getFATaskList(PlanPassReq planPassReq) {
    	
    	//modify 按照taskId和planId查询
        List<PlanPassTaskRes> faList = planPassMapper.getFaPassTaskListByPlanId(planPassReq);
        for (PlanPassTaskRes faInfo : faList) {
            List<FirstArticleWorkshop> firstArticleWorkshopList = firstArticleWorkshopService.list(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                    .eq(FirstArticleWorkshop::getBentelerPlanId, planPassReq.getPlanId()));

            List<PlanPassWorkshopRes> planPassWorkshopList = new ArrayList<PlanPassWorkshopRes>();
            PlanPassWorkshopRes planPassWorkshop = null;
            for (FirstArticleWorkshop workshop : firstArticleWorkshopList) {
                planPassWorkshop = new PlanPassWorkshopRes();
                planPassWorkshop.setWorkshopId(workshop.getId());
                planPassWorkshop.setWsName(workshop.getWsName());
                planPassWorkshop.setPlName(workshop.getPlName());
                planPassWorkshopList.add(planPassWorkshop);
            }

            for (int i = 0; i < planPassWorkshopList.size(); i++) {

                SearchPlanPassReq searchReq = new SearchPlanPassReq();
                searchReq.setWorkshopId(planPassWorkshopList.get(i).getWorkshopId());
                searchReq.setResultFlag(1);
                searchReq.setTypeFlag(faInfo.getTypeFlag());
                //只选择有问题的
                List<PlanPassItemRes> planPassItemList = planPassMapper.getFaPassItemList(searchReq);

                for (PlanPassItemRes planPassItemRes : planPassItemList) {
                	List<PlanPassItemResultRes> firstItemResultList = this.getFaItemResultList(faInfo.getTaskId(), planPassItemRes.getItemId(), faInfo.getTypeFlag(),1);
                	
                	if( faInfo.getTypeFlag()==0) {
                		planPassItemRes.setFirstItemResultList(firstItemResultList);
                	}else if( faInfo.getTypeFlag()==1) {
                		planPassItemRes.setTwoHundredItemResultList(firstItemResultList);
                	}else if( faInfo.getTypeFlag()==3) {
                		planPassItemRes.setCatchItemResultList(firstItemResultList);
                	}

                }
                planPassWorkshopList.get(i).setItemList(planPassItemList);

                faInfo.setWorkshopList(planPassWorkshopList.stream().filter(workshop -> null != workshop.getItemList() && workshop.getItemList().size() > 0).collect(Collectors.toList()));
            }
        }
        return faList;
    }

    /**
     * 首件频次拼装
     *
     * @param planPassTaskResList
     * @return
     */
    private List<Integer> getItemFAFrequency(List<PlanPassTaskRes> planPassTaskResList) {
        List<Integer> itemType = new ArrayList<>();
        planPassTaskResList.forEach(task -> {
            task.getWorkshopList().forEach(workshop -> {
                workshop.getItemList().forEach(item -> {
                    if (item.getFirstFlag() == 1) itemType.add(0);
                    if (item.getTwoHundredFlag() == 1) itemType.add(1);
                    if (item.getCatchFlag() == 1) itemType.add(2);
                });
            });
        });
        return itemType.stream().distinct().collect(Collectors.toList());
    }

    /**
     * 计划放行
     *
     * @param planApprovalTaskReq
     * @return
     */
    @Override
    public ResponseBean<Void> approvalFATaskOperate(PlanApprovalTaskReq planApprovalTaskReq) {
        int updateStatus = 0; // 更新状态
        //更新首件审批信息
        updateStatus = updateFirstArticleApprovalTask(planApprovalTaskReq);
        if (updateStatus > 0) {

            settingApprovalTask(planApprovalTaskReq);//审批通过，保存审批历史  审批不通过触发下一级别
            return ResponseBeanBuilder.ok();
        }else {
        	return ResponseBeanBuilder.fail("无权放行");
        }
    }

    /**
     * 更新首件任务审批状态
     *
     * @param planApprovalTaskReq
     * @return
     */
    private int updateFirstArticleApprovalTask(PlanApprovalTaskReq planApprovalTaskReq) {
        int updateStatus = 0;
        FirstArticleApprovalTask otUpdate = new FirstArticleApprovalTask();
        otUpdate.setReason(planApprovalTaskReq.getReason());
        otUpdate.setStatus(planApprovalTaskReq.getStatus());
        otUpdate.setApprovalUserId(planApprovalTaskReq.getUserId());
        FirstArticleApprovalTask otOld = firstArticleApprovalTaskMapper.selectOne(Wrappers.<FirstArticleApprovalTask>lambdaQuery()
                .eq(FirstArticleApprovalTask::getFirstArticleTaskId, planApprovalTaskReq.getTaskId())
                .eq(FirstArticleApprovalTask::getStatus, 0).last("limit 1"));
        if(otOld==null||!planApprovalTaskReq.getRoleIds().contains(otOld.getApprovalRole())) {
        	return updateStatus;
        }
        planApprovalTaskReq.setApprovalLevel(Integer.parseInt(otOld.getApprovalLevel()));
        otUpdate.setId(otOld.getId()); 
        updateStatus = firstArticleApprovalTaskMapper.updateById(otUpdate);
        return updateStatus;
    }

    /**
     * 更新task状态
     *
     * @return
     */
    private boolean updateFirstArticleTask(PlanApprovalTaskReq planApprovalTaskReq) {
        int taskStatus = 0; //task表状态
        if (planApprovalTaskReq.getStatus() == 1) {
            taskStatus = 2;
        } // 放行
        if (planApprovalTaskReq.getStatus() == 2) {
            taskStatus = 3;
        } // 不通过

        return firstArticleTaskService.updateById(FirstArticleTask.builder()
                .id(planApprovalTaskReq.getTaskId())
                .status(taskStatus).build());
    }

    /**
     * 设置审批task任务
     *
     * @param planApprovalTaskReq
     */
    private void settingApprovalTask(PlanApprovalTaskReq planApprovalTaskReq) {
        //审批通过，保存审批历史
        if (planApprovalTaskReq.getStatus() == 1) {
            //审批通过，更新task状态
            updateFirstArticleTask(planApprovalTaskReq);
        }else {
//        //审批不通过触发下一级别
//        if (planApprovalTaskReq.getStatus() == 2) {
        	//设置防错下一级别审批
        	setNextApprovalLevel(planApprovalTaskReq);
        }
    }

    /**
     * 设置首件下一级别人员
     *
     * @param planApprovalTaskReq
     */
    private void setNextApprovalLevel(PlanApprovalTaskReq planApprovalTaskReq) {
    	  ApprovalRule approvalRuleNextLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
                  .eq(ApprovalRule::getType, ApprovalRule.APPROVAL_TYPE_FA)
                  .gt(ApprovalRule::getApprovalLevel, planApprovalTaskReq.getApprovalLevel())
                  .orderByAsc(ApprovalRule::getApprovalLevel)
                  .last("limit 1"));
        if (approvalRuleNextLevel!=null) {
        	firstArticleService.setApproval(approvalRuleNextLevel, planApprovalTaskReq.getTaskId(), planApprovalTaskReq.getUserId());
        } else {
            //如果是最后一个审批级别则设置task状态为不通过
            updateFirstArticleTask(planApprovalTaskReq);
            //最有一级记录task日志，为不通过
            firstArticleService.setFirstArticleTaskHis(planApprovalTaskReq.getTaskId());
            firstArticleService.setFaRepeatResult(planApprovalTaskReq.getTaskId());
        }
    }

    /**
     * 获取首件item
     *
     * @param taskId
     * @param itemId
     * @param typeFlag 0首件,1二百建,2中断
     * @param resultFlag 0没问题结果1有问题结果 其他全部
     * @return
     */
    private List<PlanPassItemResultRes> getFaItemResultList(long taskId, long itemId, int typeFlag,Integer resultFlag) {
        SearchPlanPassReq searchReq = new SearchPlanPassReq();
        searchReq.setTaskId(taskId);
        searchReq.setItemId(itemId);
        searchReq.setTypeFlag(typeFlag);
        searchReq.setResultFlag(resultFlag);
        List<PlanPassItemResultRes> planPassItemResultResList = planPassMapper.getFaPassItemResultList(searchReq);
        return planPassItemResultResList;
    }
}
