package com.factory.wx.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.service.UserRoleService;
import com.factory.web.entity.pm.ApprovalRule;
import com.factory.web.entity.pm.OnsiteApprovalTask;
import com.factory.web.entity.pm.OnsiteCalendar;
import com.factory.web.entity.pm.OnsiteTask;
import com.factory.web.entity.pm.OnsiteTitle;
import com.factory.web.entity.pm.OnsiteWorkshop;
import com.factory.web.mapper.pm.ApprovalRuleMapper;
import com.factory.web.mapper.pm.OnsiteApprovalTaskMapper;
import com.factory.web.mapper.pm.OnsiteTaskMapper;
import com.factory.web.service.pm.OnsiteCalendarService;
import com.factory.web.service.pm.OnsiteWorkshopService;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.req.PlanApprovalTaskReq;
import com.factory.wx.entity.req.PlanPassReq;
import com.factory.wx.entity.req.SearchPlanPassReq;
import com.factory.wx.entity.req.SendMsgReq;
import com.factory.wx.entity.res.PlanPassDetailRes;
import com.factory.wx.entity.res.PlanPassItemRes;
import com.factory.wx.entity.res.PlanPassTaskRes;
import com.factory.wx.entity.res.PlanPassWorkshopRes;
import com.factory.wx.entity.res.WxPlanBaseInfoRes;
import com.factory.wx.mapper.PlanPassMapper;
import com.factory.wx.service.PlanPassPMService;
import com.factory.wx.service.WxPlanService;
import com.factory.wx.service.WxService;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-08-03 10:29 上午
 */

@Service("planPassPMService")
public class PlanPassPMServiceImpl implements PlanPassPMService {
    @Autowired
    private PlanPassMapper planPassMapper;
    @Autowired
    private OnsiteTaskMapper onsiteTaskMapper;
    @Autowired
    private OnsiteApprovalTaskMapper onsiteApprovalTaskMapper;
    @Autowired
    private WxPlanService wxPlanService;
    @Autowired
    private OnsiteWorkshopService onsiteWorkshopService;
    @Autowired
    private ApprovalRuleMapper approvalRuleMapper;
    @Autowired
    private PreventMistakeService preventMistakeService;
    @Autowired
    private WxService wxService;
    @Autowired
    private UserRoleService userRoleService;
    
    @Autowired
    private OnsiteCalendarService onsiteCalendarService;
    /**
     * 计划放行详情-防错
     *
     * @param planPassReq
     * @return
     */

    @Override
    public PlanPassDetailRes getPassDetailPM(PlanPassReq planPassReq) {
    	List<Long> roleIds = userRoleService.getRoleIdsByUserId(planPassReq.getUserId());
    	planPassReq.setRoleIds(roleIds);
        WxPlanBaseInfoRes wxPlanBaseInfoRes = wxPlanService.getOnsitePlanBaseInfo(planPassReq.getPlanId());
        List<PlanPassTaskRes> planPassTaskResList = null;
        List<Integer> itemType = wxPlanBaseInfoRes.getFrequency();
        if(!roleIds.isEmpty()) {
        	planPassTaskResList = getPMTaskList(planPassReq);
//        	itemType = getItemPMFrequency(planPassTaskResList);
        }

        return PlanPassDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(itemType)
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .titleInfo((OnsiteTitle) wxPlanBaseInfoRes.getTitle())
                .taskList(planPassTaskResList)
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .build();
    }

    /**
     * 获取防错点检错误列表
     *
     * @param planPassReq
     * @return
     */
    private List<PlanPassTaskRes> getPMTaskList(PlanPassReq planPassReq) {
    	//只去当前任务
        List<PlanPassTaskRes> pmList = planPassMapper.getPmPassTaskListByPlanId(planPassReq);
        for (PlanPassTaskRes pmInfo : pmList) {
            List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopService.list(Wrappers.<OnsiteWorkshop>lambdaQuery()
                    .eq(OnsiteWorkshop::getBentelerPlanId, pmInfo.getPlanId()));

            List<PlanPassWorkshopRes> workshopList = new ArrayList<PlanPassWorkshopRes>();
            for (OnsiteWorkshop workshop : onsiteWorkshopList) {
                PlanPassWorkshopRes planPassWorkshopRes = new PlanPassWorkshopRes();
                planPassWorkshopRes.setWorkshopId(workshop.getId());
                planPassWorkshopRes.setWsName(workshop.getWsName());
                workshopList.add(planPassWorkshopRes);
            }

            for (int i = 0; i < workshopList.size(); i++) {
                SearchPlanPassReq searchReq = new SearchPlanPassReq();
                searchReq.setWorkshopId(workshopList.get(i).getWorkshopId());
                searchReq.setTaskId(pmInfo.getTaskId());
                searchReq.setResultFlag(1);
                List<PlanPassItemRes> planPassItemList = planPassMapper.getPmPassItemList(searchReq).stream().filter(item -> item.getResult() > 0).collect(Collectors.toList());
                for (PlanPassItemRes planPassItemRes : planPassItemList) {
                    Long userId = planPassItemRes.getItemResultUserId();
                    if (null != userId) {
                        String userName = wxPlanService.getUserNameById(planPassItemRes.getItemResultUserId());
                        planPassItemRes.setItemResultUser(userName);
                    }
                }
                workshopList.get(i).setItemList(planPassItemList);
            }
            pmInfo.setWorkshopList(workshopList.stream().filter(workshop -> workshop.getItemList().size() > 0).collect(Collectors.toList()));
        }
        return pmList;
    }

    /**
     * 获取防错频次
     *
     * @param planPassTaskResList
     */
    private List<Integer> getItemPMFrequency(List<PlanPassTaskRes> planPassTaskResList) {
        List<Integer> frequency = new ArrayList<>();
        planPassTaskResList.forEach(task -> {
            task.getWorkshopList().forEach(workshop -> {
                workshop.getItemList().forEach(item -> {
                    frequency.add(item.getItemType());
                });
            });
        });
        return frequency.stream().distinct().collect(Collectors.toList());
    }

    /**
     * 防错计划放行
     *
     * @param planApprovalTaskReq
     * @return
     */
    @Override
    public ResponseBean<Void> approvalPMTaskOperate(PlanApprovalTaskReq planApprovalTaskReq) {
        int updateStatus = 0; // 更新状态
        //更新防错审批信息
        updateStatus = updateOnsiteApprovalTask(planApprovalTaskReq);
        if (updateStatus > 0) {
            settingApprovalTask(planApprovalTaskReq);
            return ResponseBeanBuilder.ok();
        }else {
        	return ResponseBeanBuilder.fail("无权放行");
        }
    }


    /**
     * 更新防错任务审批状态
     *
     * @param planApprovalTaskReq
     * @return
     */
    private int updateOnsiteApprovalTask(PlanApprovalTaskReq planApprovalTaskReq) {
        int updateStatus = 0;
        OnsiteApprovalTask otUpdate = new OnsiteApprovalTask();
        otUpdate.setReason(planApprovalTaskReq.getReason());
        otUpdate.setStatus(planApprovalTaskReq.getStatus());
        otUpdate.setApprovalUserId(planApprovalTaskReq.getUserId());
        
        OnsiteApprovalTask otOld = onsiteApprovalTaskMapper.selectOne(Wrappers.<OnsiteApprovalTask>lambdaQuery()
                .eq(OnsiteApprovalTask::getOnsiteTaskId, planApprovalTaskReq.getTaskId())
                .eq(OnsiteApprovalTask::getStatus, 0).last("limit 1"));
        if(otOld==null||!planApprovalTaskReq.getRoleIds().contains(otOld.getApprovalRole())) {
        	return updateStatus;
        }
        planApprovalTaskReq.setApprovalLevel(otOld.getApprovalLevel());
        otUpdate.setId(otOld.getId()); 
        updateStatus = onsiteApprovalTaskMapper.updateById(otUpdate);
        return updateStatus;
    }

    /**
     * 更新task状态
     *
     * @return
     */
    private int updateOnsiteTask(PlanApprovalTaskReq planApprovalTaskReq) {
        int taskStatus = 0; //task表状态
        if (planApprovalTaskReq.getStatus() == 1) taskStatus = 2; // 放行
        if (planApprovalTaskReq.getStatus() == 2) taskStatus = 3; // 不通过
        return onsiteTaskMapper.updateById(OnsiteTask.builder()
                .id(planApprovalTaskReq.getTaskId())
                .status(taskStatus).build());
    }

    /**
     * 设置审批task任务
     *
     * @param planApprovalTaskReq
     * @return boolean 是否有下一级别
     */
    private void settingApprovalTask(PlanApprovalTaskReq planApprovalTaskReq) {
        //审批通过
        if (planApprovalTaskReq.getStatus() == 1) {
            //审批通过，更新task状态
            updateOnsiteTask(planApprovalTaskReq);
        }else {
//        //审批不通过触发下一级别
//        if (planApprovalTaskReq.getStatus() == 2) {
            //设置防错下一级别审批
            setNextPMApprovalLevel(planApprovalTaskReq);
        }
    }

    /**
     * 设置防错下一级别人员
     *
     * @param planApprovalTaskReq
     * @return boolean 是否有下一级别
     */
    private void setNextPMApprovalLevel(PlanApprovalTaskReq planApprovalTaskReq) {
        ApprovalRule approvalRuleNextLevel = approvalRuleMapper.selectOne(Wrappers.<ApprovalRule>lambdaQuery()
                .eq(ApprovalRule::getType, ApprovalRule.APPROVAL_TYPE_PM)
                .gt(ApprovalRule::getApprovalLevel, planApprovalTaskReq.getApprovalLevel())
                .orderByAsc(ApprovalRule::getApprovalLevel)
                .last("limit 1"));
        if (approvalRuleNextLevel!=null) {
        	preventMistakeService.setApproval(approvalRuleNextLevel, planApprovalTaskReq.getTaskId(), planApprovalTaskReq.getUserId());
        } else {
            //如果是最后一个审批级别则设置task状态为不通过
            updateOnsiteTask(planApprovalTaskReq);
            //最有一级记录task日志，为不通过
            preventMistakeService.setOnsiteTaskHis(planApprovalTaskReq.getTaskId());
            preventMistakeService.setPmRepeatResult(planApprovalTaskReq.getTaskId());
        }
    }

}
