package com.factory.wx.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.service.UserRoleService;
import com.factory.web.entity.fa.res.FaPlanListRes;
import com.factory.wx.entity.req.PlanApprovalTaskReq;
import com.factory.wx.entity.req.PlanPageReq;
import com.factory.wx.entity.res.PlanPassRes;
import com.factory.wx.mapper.PlanPassMapper;
import com.factory.wx.service.PlanPassFAService;
import com.factory.wx.service.PlanPassPMService;
import com.factory.wx.service.PlanPassService;

import lombok.extern.slf4j.Slf4j;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-13 12:42 下午
 */
@Slf4j
@Service("planPassService")
public class PlanPassServiceImpl implements PlanPassService {

    @Autowired
    private PlanPassMapper planPassMapper;
    @Autowired
    private PlanPassFAService planPassFAService;
    @Autowired
    private PlanPassPMService planPassPMService;
    @Autowired
    private UserRoleService userRoleService;
    /**
     * 计划放行列表
     *
     * @param planPageReq
     * @return
     */
    @Override
    public List<PlanPassRes> getPassList(PlanPageReq planPageReq) {
    	List<Long> roleIds =userRoleService.getRoleIdsByUserId(planPageReq.getUserId());
    	List<PlanPassRes> planPassResList = new ArrayList<>();
    	Page<PlanPassRes> page = new Page<>(planPageReq.getPage(), planPageReq.getSize());
    	if(!roleIds.isEmpty()) {
    		planPageReq.setRoleIds(roleIds);
	        //分页
	        //首件列表
	        if(planPageReq.getCheckType() == 1) {
	            planPassResList = planPassMapper.getPlanPassList(page,planPageReq);
	        }else if (planPageReq.getCheckType() == 2) {//防错列表
	            planPassResList = planPassMapper.getPlanPassPMList(page,planPageReq);
	        }
    	}
        return planPassResList;
    }

    /**
     * 计划放行
     *
     * @param planApprovalTaskReq
     * @return
     */
    @Override
    public ResponseBean<Void> approvalTaskOperate(PlanApprovalTaskReq planApprovalTaskReq) {
        ResponseBean<Void> responseBean = new ResponseBean<>();
        List<Long> roleIds =userRoleService.getRoleIdsByUserId(planApprovalTaskReq.getUserId());
        planApprovalTaskReq.setRoleIds(roleIds);
        if(roleIds.isEmpty()) {
        	return ResponseBeanBuilder.fail("无权放行");
        }
        if (planApprovalTaskReq.getPlanType() == 1) {
            responseBean = planPassFAService.approvalFATaskOperate(planApprovalTaskReq);
        }
        if (planApprovalTaskReq.getPlanType() == 2) {
            responseBean = planPassPMService.approvalPMTaskOperate(planApprovalTaskReq);
        }
        return responseBean;
    }

}
