package com.factory.wx.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.factory.user.service.UserRoleService;
import com.factory.web.entity.fa.FirstArticleItem;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.fa.FirstArticleWorkshop;
import com.factory.web.entity.pm.OnsiteItem;
import com.factory.web.entity.pm.OnsiteTitle;
import com.factory.web.entity.pm.OnsiteWorkshop;
import com.factory.web.mapper.BentelerPlanMapper;
import com.factory.web.mapper.fa.FirstArticleItemMapper;
import com.factory.web.mapper.fa.FirstArticleWorkshopMapper;
import com.factory.web.mapper.pm.OnsiteCalendarMapper;
import com.factory.web.mapper.pm.OnsiteItemMapper;
import com.factory.web.mapper.pm.OnsiteTaskMapper;
import com.factory.web.mapper.pm.OnsiteWorkshopMapper;
import com.factory.web.mapper.pm.RemindApprovalTaskMapper;
import com.factory.wx.entity.req.PlanInfoReq;
import com.factory.wx.entity.req.PlanPageReq;
import com.factory.wx.entity.res.PlanReminderDetailRes;
import com.factory.wx.entity.res.ReminderRes;
import com.factory.wx.entity.res.WxPlanBaseInfoRes;
import com.factory.wx.mapper.ReminderMapper;
import com.factory.wx.service.PlanReminderService;
import com.factory.wx.service.WxPlanService;

import lombok.extern.slf4j.Slf4j;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-13 12:42 下午
 */
@Slf4j
@Service("planReminderService")
public class PlanReminderServiceImpl implements PlanReminderService {

    @Autowired
    private BentelerPlanMapper bentelerPlanMapper;
    @Autowired
    private OnsiteCalendarMapper onsiteCalendarMapper;
    @Autowired
    private OnsiteWorkshopMapper onsiteWorkshopMapper;
    @Autowired
    private OnsiteItemMapper onsiteItemMapper;
    @Autowired
    private OnsiteTaskMapper onsiteTaskMapper;
    @Autowired
    private RemindApprovalTaskMapper remindApprovalTaskMapper;
    @Autowired
    private ReminderMapper reminderMapper;
    @Autowired
    private FirstArticleWorkshopMapper firstArticleWorkshopMapper;
    @Autowired
    private FirstArticleItemMapper firstArticleItemMapper;
    @Autowired
    private WxPlanService wxPlanService;
    @Autowired
    private UserRoleService userRoleService;
    /**
     * 获取计划提醒列表
     *
     * @param planPageReq
     * @return
     */
    @Override
    public List<ReminderRes> getReminderList(PlanPageReq planPageReq) {
        List<Long> roleIds =userRoleService.getRoleIdsByUserId(planPageReq.getUserId());
        if(roleIds.isEmpty()) {
    		log.error("没有权限获取超时任务{}",planPageReq);
    		return new ArrayList<>();
    	}
        planPageReq.setRoleIds(roleIds);
        Page<ReminderRes> page = new Page<>(planPageReq.getPage(),planPageReq.getSize());
        return reminderMapper.getReminderList(page,planPageReq);
    }

    /**
     * 防错提醒详情
     *
     * @param planInfoReq
     * @return
     */
    @Override
    public PlanReminderDetailRes getReminderDetailPM(PlanInfoReq planInfoReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes = wxPlanService.getOnsitePlanBaseInfo(planInfoReq.getPlanId());
        List<OnsiteWorkshop> onsiteWorkshopList = onsiteWorkshopMapper.selectList(Wrappers.<OnsiteWorkshop>lambdaQuery()
                .eq(OnsiteWorkshop::getBentelerPlanId, planInfoReq.getPlanId()));
        onsiteWorkshopList.forEach(onsiteWorkshop -> {
            onsiteWorkshop.setOnsiteItemList(onsiteItemMapper.selectList(Wrappers.<OnsiteItem>lambdaQuery()
                    .eq(OnsiteItem::getWorkshopId, onsiteWorkshop.getId())));
        });
        return PlanReminderDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(wxPlanBaseInfoRes.getFrequency())
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .titleInfo((OnsiteTitle) wxPlanBaseInfoRes.getTitle())
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .onsiteWorkshops(onsiteWorkshopList)
                .build();

    }

    @Override
    public int confirmReminder(PlanInfoReq planInfoReq) {
    	List<Long> roleIds =userRoleService.getRoleIdsByUserId(planInfoReq.getUserId());
    	if(roleIds.isEmpty()) {
    		log.error("没有权限确认超时任务{}",planInfoReq);
    		return 0;
    	}
    	planInfoReq.setRoleIds(roleIds);
        return reminderMapper.confirmReminder(planInfoReq);
    }

    /**
     * 首件提醒
     *
     * @param planInfoReq
     * @return
     */
    @Override
    public PlanReminderDetailRes getReminderDetail(PlanInfoReq planInfoReq) {
        WxPlanBaseInfoRes wxPlanBaseInfoRes = wxPlanService.getFirstArticlePlanBaseInfo(planInfoReq.getPlanId());

        List<FirstArticleWorkshop> firstArticleWorkshopList = firstArticleWorkshopMapper.selectList(Wrappers.<FirstArticleWorkshop>lambdaQuery()
                .eq(FirstArticleWorkshop::getBentelerPlanId, planInfoReq.getPlanId()));
        firstArticleWorkshopList.forEach(firstArticleWorkshop -> {
            firstArticleWorkshop.setFirstArticleItemList(firstArticleItemMapper.selectList(Wrappers.<FirstArticleItem>lambdaQuery()
                    .eq(FirstArticleItem::getWorkshopId, firstArticleWorkshop.getId())));
        });
        return PlanReminderDetailRes.builder()
                .planId(wxPlanBaseInfoRes.getPlanId())
                .planName(wxPlanBaseInfoRes.getPlanName())
                .usersName(wxPlanBaseInfoRes.getUsersName())
                .frequency(wxPlanBaseInfoRes.getFrequency())
                .effectiveDate(wxPlanBaseInfoRes.getEffectiveDate())
                .firstArticleTitle((FirstArticleTitle) wxPlanBaseInfoRes.getTitle())
                .attachment(wxPlanBaseInfoRes.getAttachment())
                .firstArticleWorkshops(firstArticleWorkshopList)
                .build();
    }

}
