package com.factory.wx.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.po.benteler.BentelerPlan;
import com.factory.base.entity.po.remote.UserInfo;
import com.factory.user.mapper.UserInfoMapper;
import com.factory.web.entity.fa.FirstArticleCalendar;
import com.factory.web.entity.fa.FirstArticleTitle;
import com.factory.web.entity.pm.OnsiteCalendar;
import com.factory.web.entity.pm.OnsiteTitle;
import com.factory.web.mapper.BentelerPlanMapper;
import com.factory.web.mapper.FileMapper;
import com.factory.web.mapper.fa.FirstArticleCalendarMapper;
import com.factory.web.mapper.pm.OnsiteCalendarMapper;
import com.factory.web.service.fa.FirstArticleService;
import com.factory.web.service.impl.FileEntry;
import com.factory.web.service.pm.PreventMistakeService;
import com.factory.wx.entity.res.WxPlanBaseInfoRes;
import com.factory.wx.service.WxPlanService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-29 9:33 上午
 */
@Service("wxPlanService")
public class WxPlanServiceImpl implements WxPlanService {

    @Autowired
    private UserInfoMapper userInfoMapper;
    @Autowired
    private BentelerPlanMapper bentelerPlanMapper;
    @Autowired
    private FirstArticleCalendarMapper firstArticleCalendarMapper;
    @Autowired
    private FirstArticleService firstArticleService;
    @Autowired
    private OnsiteCalendarMapper onsiteCalendarMapper;
    @Autowired
    private PreventMistakeService preventMistakeService;
    @Autowired
    private FileMapper fileMapper;

    @Override
    public WxPlanBaseInfoRes getFirstArticlePlanBaseInfo(Long planId) {
        BentelerPlan bentelerPlan = bentelerPlanMapper.selectById(planId);
        FirstArticleCalendar first = firstArticleCalendarMapper.selectOne(Wrappers.<FirstArticleCalendar>lambdaQuery()
                .eq(FirstArticleCalendar::getBentelerPlanId, planId));
        //获取点检负责人
        List<String> usersName = new ArrayList<>();
        String nameA = assembleUserName(first.getPrincipalOneId(), first.getPrincipalOneShift());
        String nameB = assembleUserName(first.getPrincipalTwoId(), first.getPrincipalTwoShift());
        usersName.add(nameA);
        usersName.add(nameB);
        //获取title
        FirstArticleTitle title = firstArticleService.getFaTitle(planId);
        //判断是否有附件
        int attrCount = fileMapper.selectCount(Wrappers.<FileEntry>lambdaQuery().eq(FileEntry::getId, title.getUploadAttachmentId()));
        // 获取检验频次列表
        List<Integer> frequency = new ArrayList<>();
        if (StringUtils.isNotBlank(first.getTypeFlag())) {
            frequency = Arrays.stream(first.getTypeFlag().split(",")).map(x -> Integer.parseInt(x)).collect(Collectors.toList());
        }
        return WxPlanBaseInfoRes.builder()
                .planId(bentelerPlan.getId())
                .planName(bentelerPlan.getName())
                .usersName(usersName.stream().distinct().collect(Collectors.toList()))
                .frequency(frequency)//频次去重
                .attachment(attrCount == 0 ? 0 : 1)
                .effectiveDate(bentelerPlan.getEffectiveDate())
                .title(title)
                .build();
    }

    @Override
    public WxPlanBaseInfoRes getOnsitePlanBaseInfo(Long planId) {
        BentelerPlan bentelerPlan = bentelerPlanMapper.selectById(planId);
        List<OnsiteCalendar> onsiteCalendarList = onsiteCalendarMapper.selectList(Wrappers.<OnsiteCalendar>lambdaQuery()
                .eq(OnsiteCalendar::getBentelerPlanId, planId));

        //获取点检负责人
        List<String> usersName = new ArrayList<>();
        List<Integer> frequency = new ArrayList<>();
        onsiteCalendarList.forEach(onsiteCalendar -> {
            String name = assembleUserName(onsiteCalendar.getUserId(), onsiteCalendar.getShift());
            if (!name.isEmpty()) {
                usersName.add(name);
            }
            frequency.add(onsiteCalendar.getTypeFlag());
        });

        //获取title
        OnsiteTitle title = preventMistakeService.getTitle(planId);

        //判断是否有附件
        int attrCount = fileMapper.selectCount(Wrappers.<FileEntry>lambdaQuery().eq(FileEntry::getId, title.getUploadAttachmentId()));

        return WxPlanBaseInfoRes.builder()
                .planId(bentelerPlan.getId())
                .planName(bentelerPlan.getName())
                .usersName(usersName.stream().distinct().collect(Collectors.toList()))
                .frequency(frequency.stream().distinct().collect(Collectors.toList()))//频次去重
                .effectiveDate(bentelerPlan.getEffectiveDate())
                .title(title)
                .attachment(attrCount == 0 ? 0 : 1)
                .build();
    }

    /**
     * 根据用户id及班次拼装用户姓名
     *
     * @param userId
     * @param shift
     * @return 取不到用户信息返回空字符串，取到用户信息拼接班次
     */
    private String assembleUserName(Long userId, String shift) {
        String userName = getUserNameById(userId);
        if (!userName.isEmpty()) {
            userName = shift == null ? userName : StringUtils.join(userName, " ", shift, "班");
        }
        return userName;
    }

    /**
     * 根据用户id获取用户名
     *
     * @param userId
     * @return 返回用户名，未找到时返回空字符串
     */
    @Override
    public String getUserNameById(Long userId) {
        if (userId!=null &&userId > 0) {
            UserInfo userInfo = userInfoMapper.selectById(userId);
            if (null != userInfo && null != userInfo.getUsername()) {
                return userInfo.getUsername();
            }
        }
        return "";
    }
}
