package com.factory.wx.service.impl;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.task.AppEntry;
import com.factory.base.entity.task.PushWxMessage;
import com.factory.base.entity.task.TemplateEntry;
import com.factory.base.entity.task.ValEntry;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.service.UserRoleService;
import com.factory.wx.entity.UsUserWechat;
import com.factory.wx.entity.req.SendMsgReq;
import com.factory.wx.entity.res.FunctionsRes;
import com.factory.wx.mapper.WxMapper;
import com.factory.wx.service.WxPlanService;
import com.factory.wx.service.WxService;

import lombok.extern.slf4j.Slf4j;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-07-12 4:08 下午
 */
@Slf4j
@Service("wxService")
@Transactional(rollbackFor = Exception.class)
public class WxServiceImpl implements WxService {


    @Value("${wx.minprogram.appsecret}")
    private String appsecret;

    @Value("${wx.minprogram.appid}")
    private String appid;

    @Value("${wx.minprogram.templateId}")
    private String templateId;

    @Value("${wx.minprogram.page}")
    private String page;

    @Value("${wx.minprogram.miniprogramState}")
    private String miniprogramState;

    @Autowired
    private WxMapper wxMapper;
    @Autowired
    private WxPlanService wxPlanService;
    
    @Autowired
    private UserRoleService userRoleService;

    @Override
    public ResponseBean<List<FunctionsRes>> getUserFunction(Long userId) {
        List<FunctionsRes> functionList = wxMapper.getUserFunctions(userId);

//        List<FunctionsRes> functionsResList = new ArrayList<>();
//        functionList.forEach((i) -> {
//            FunctionsRes functionsRes = new FunctionsRes();
//            functionsRes.setFunction(i.getFunction());
//            functionsRes.setName(i.getName());
//            functionsResList.add(functionsRes);
//        });

        return ResponseBeanBuilder.ok(functionList);
    }

    @Override
    public ResponseBean bindWeChatUser(String openId, Long userId) {
        UsUserWechat usUserWechat = wxMapper.selectOne(Wrappers.<UsUserWechat>lambdaQuery()
                .eq(UsUserWechat::getUserId, userId));
        if (null != usUserWechat) {
            UsUserWechat user = new UsUserWechat();
            user.setId(usUserWechat.getId());
            user.setOpenid(openId);
            Integer rows = wxMapper.updateById(user);
//            if (rows == 0) return ResponseBeanBuilder.fail("更新绑定信息失败");
        } else {
            UsUserWechat user = new UsUserWechat();
            user.setUserId(userId);
            user.setOpenid(openId);
            wxMapper.insert(user);
        }
        return ResponseBeanBuilder.ok();
    }

    @Override
    public Integer selectBindWeChatUserCount(String openid, Long userId) {
        return wxMapper.selectCount(Wrappers.<UsUserWechat>lambdaQuery().eq(UsUserWechat::getUserId, userId).eq(UsUserWechat::getOpenid, openid));
    }

    @Override
    public void sendWechatMessage(SendMsgReq sendMsgReq) {
        try {
            String openId = wxMapper.selectOne(Wrappers.<UsUserWechat>lambdaQuery().eq(UsUserWechat::getUserId, sendMsgReq.getToUserId())).getOpenid();
            if (null != openId && !openId.isEmpty()) {
                AppEntry appEntry = AppEntry.builder().appId(appid).secret(appsecret).build();
                HashMap<String, ValEntry> dataMap = new HashMap();
                //提醒内容
                dataMap.put("thing1", ValEntry.builder().value(sendMsgReq.getContent()).build());
                //作业名称
                dataMap.put("thing2", ValEntry.builder().value(sendMsgReq.getTitle()).build());
                //截止日期
                if (null != sendMsgReq.getEndTime()) {
                    dataMap.put("time3", ValEntry.builder().value(sendMsgReq.getEndTime()).build());
                } else {
                    Date date = new Date();
                    SimpleDateFormat ft = new SimpleDateFormat("yyyy-MM-dd");
                    dataMap.put("time3", ValEntry.builder().value(ft.format(date)).build());
                }
                //发布人员
                if (null != sendMsgReq.getFromUserId()) {
                    String fromUserName = wxPlanService.getUserNameById(sendMsgReq.getFromUserId());
                    dataMap.put("thing4", ValEntry.builder().value(fromUserName).build());
                } else {
                    dataMap.put("thing4", ValEntry.builder().value("智云工厂系统").build());
                }
                TemplateEntry tpl = TemplateEntry.builder()
                        .template_id(templateId)
                        .touser(openId)
                        .page(page)
                        .miniprogram_state(miniprogramState)
                        .data(dataMap)
                        .build();
                PushWxMessage.sendMessage(appEntry, tpl);
            } else {
                log.error("下发微信消息，接收人userId:{}的openId为空", sendMsgReq.getToUserId());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("下发微信消息异常");
        }
    }

	@Override
	public void sendWechatMessageByRole(SendMsgReq sendMsgReq, Long roleId) {
		List<Long> list = userRoleService.getUserIdsByRoleId(roleId);
		if(list.isEmpty()) {
	        log.error("下发微信消息，角色:{}接收人为空", roleId);
			return;
		}
		HashMap<String, ValEntry> dataMap = new HashMap<>();
		 //提醒内容
        dataMap.put("thing1", ValEntry.builder().value(sendMsgReq.getContent()).build());
        //作业名称
        dataMap.put("thing2", ValEntry.builder().value(sendMsgReq.getTitle()).build());
        //截止日期
        if (null != sendMsgReq.getEndTime()) {
            dataMap.put("time3", ValEntry.builder().value(sendMsgReq.getEndTime()).build());
        } else {
            Date date = new Date();
            SimpleDateFormat ft = new SimpleDateFormat("yyyy-MM-dd");
            dataMap.put("time3", ValEntry.builder().value(ft.format(date)).build());
        }
        //发布人员
        if (null != sendMsgReq.getFromUserId()) {
            String fromUserName = wxPlanService.getUserNameById(sendMsgReq.getFromUserId());
            dataMap.put("thing4", ValEntry.builder().value(fromUserName).build());
        } else {
            dataMap.put("thing4", ValEntry.builder().value("智云工厂系统").build());
        }
        Set<String> openIdSet = new HashSet<>();
		for(Long userId:list) {
			try {
				UsUserWechat uUsUserWechat = wxMapper
						.selectOne(Wrappers.<UsUserWechat>lambdaQuery().eq(UsUserWechat::getUserId, userId));
				if(uUsUserWechat==null||uUsUserWechat.getOpenid()==null) {
					log.warn("下发微信消息，接收人userId:{}的openId为空", sendMsgReq.getToUserId());
					continue;
				}
				if(!openIdSet.add(uUsUserWechat.getOpenid())) {
					log.warn("下发微信消息，接收人userId:{}的openId重复",sendMsgReq.getToUserId());
					continue;
				}
				AppEntry appEntry = AppEntry.builder().appId(appid).secret(appsecret).build();
				TemplateEntry tpl = TemplateEntry.builder().template_id(templateId).touser(uUsUserWechat.getOpenid()).page(page)
						.miniprogram_state(miniprogramState).data(dataMap).build();
				PushWxMessage.sendMessage(appEntry, tpl);
			} catch (Exception e) {
				 log.error("下发微信消息异常",e);
			}
		}
		
	}
    
}
