package com.factory.file.config;


import com.factory.base.entity.constant.SystemConstant;
import com.factory.base.util.file.FileUtils;
import com.factory.base.config.properties.FileSystemProperties;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.ApplicationListener;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.event.ContextRefreshedEvent;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;


@Configuration
@Slf4j
public class UpdateFileStoreDirListener implements ApplicationListener<ContextRefreshedEvent> {

    @Resource
    private FileSystemProperties fileSystemProperties;

    @Override
    public void onApplicationEvent(ContextRefreshedEvent contextRefreshedEvent) {
        File file = new File("");
        String canonicalPath;
        try {
            canonicalPath = file.getCanonicalPath();
        } catch (IOException e) {
            log.info(SystemConstant.EXCEPTION_LOG_PREFIX, e.getLocalizedMessage());
            canonicalPath = "";
        }
        //重置下载url
        String downLoadPrefix = StringUtils.join(
                fileSystemProperties.getDownloadPrefix(),
                SystemConstant.URI_CONCAT_SYMBOL,
                fileSystemProperties.getFileStoreDir(),
                SystemConstant.URI_CONCAT_SYMBOL
        );
        fileSystemProperties.setDownloadPrefix(
                downLoadPrefix
        );
        //重置文件下载路径
        String fileStoreDirPath = StringUtils.join(
                canonicalPath,
                File.separator,
                fileSystemProperties.getFileStoreDir(),
                File.separator
        );
        fileSystemProperties.setFileStoreDirPath(
                fileStoreDirPath
        );
        //zip文件解压路径
        String zipOutDir = StringUtils.join(
                fileSystemProperties.getFileStoreDir(),
                File.separator,
                fileSystemProperties.getZipOutDir(),
                File.separator
        );
        fileSystemProperties.setZipOutDirPath(
                zipOutDir
        );

        //重置文件下载路径
        fileSystemProperties.setFileStoreDirPath(
                StringUtils.join(
                        canonicalPath,
                        File.separator,
                        fileSystemProperties.getFileStoreDir(),
                        File.separator
                )
        );


        //重置文件下载路径
        String fileTempDir = StringUtils.join(
                canonicalPath,
                File.separator,
                fileSystemProperties.getFileStoreDir(),
                File.separator,
                fileSystemProperties.getFileTempDir(),
                File.separator
        );
        fileSystemProperties.setFileTempDirPath(
                fileTempDir
        );
        //清空历史文件
        FileUtils.mkdirs(fileStoreDirPath);
        FileUtils.mkdirs(zipOutDir);
        FileUtils.mkdirs(fileTempDir);
        log.debug("重启完成后重置文件下载、存储路径");
    }
}
