package com.factory.file.service;


import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.auth.FileType;
import com.factory.base.entity.constant.SystemConstant;
import com.factory.base.entity.po.file.FileInfo;
import com.factory.base.util.file.FileUtils;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.base.config.properties.FileSystemProperties;
import com.factory.file.entity.vo.FileRes;
import com.factory.file.util.SnowFlake;
import com.factory.user.config.AuthenticationProperties;
import com.factory.user.config.handler.LoginContext;
import com.factory.user.mapper.FileInfoMapper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

@Service("FileInfo")
public class FileInfoServiceImpl extends ServiceImpl<FileInfoMapper, FileInfo> implements FileInfoService {
    @Resource
    private FileSystemProperties fileSystemProperties;
    @Resource
    private FileInfoMapper fileInfoMapper;
    @Resource
    private AuthenticationProperties authenticationProperties;
    @Resource
    private LoginContext loginContext;

    @Override
    public ResponseBean<FileRes> upload(MultipartFile file, FileType fileType) {
        String originalFilename = file.getOriginalFilename();
        String prefix = FileUtils.getPrefix(originalFilename, SystemConstant.FILE_NAME_SPLIT_SYMBOL);
        String suffix = FileUtils.getSuffix(originalFilename, SystemConstant.FILE_NAME_SPLIT_SYMBOL);
        String fileNo = String.valueOf(SnowFlake.nextId());
        String timeStr = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss"));
        String finalSuffix = "";
        if (StringUtils.isNotBlank(suffix)) {
            finalSuffix = StringUtils.join(
                    SystemConstant.FILE_NAME_CONCAT_SYMBOL,
                    suffix
            );
        }

        String uri = StringUtils.join(
                fileSystemProperties.getFileTempDir(),
                SystemConstant.URI_CONCAT_SYMBOL,
                timeStr,
                SystemConstant.URI_CONCAT_SYMBOL,
                fileNo,
                finalSuffix
        );

        String finalName = StringUtils.join(
                fileNo,
                finalSuffix
        );
        String diskPath = StringUtils.join(
                fileSystemProperties.getFileTempDirPath(),
                timeStr,
                File.separator,
                finalName
        );
        String fileTimeStr = StringUtils.join(
                fileSystemProperties.getFileStoreDirPath(),
                fileSystemProperties.getFileTempDir(),
                File.separator,
                timeStr);
        File file1 = new File(fileTimeStr);
        if (!file1.exists()) {
            file1.mkdirs();
        }
        String url = StringUtils.join(
                fileSystemProperties.getDownloadPrefix(),
                uri
        );
        try {
            //写入文件
            FileUtils.storeFileByInputStream(
                    file.getInputStream(),
                    diskPath
            );
        } catch (IOException e) {
            e.printStackTrace();
            return ResponseBeanBuilder.fail("文件上传失败");
        }
        FileInfo build = FileInfo
                .builder()
                .name(finalName)
                .originalName(prefix)
                .diskPath(diskPath)
                .size(file.getSize())
                .suffix(suffix)
                .uri(uri)
                .type(fileType)
                .build();
        fileInfoMapper.insert(build);
        FileRes fileRes = new FileRes();
        BeanUtils.copyProperties(build, fileRes);
        fileRes.setUrl(
                url
        );
        /**
         * StringUtils.join(
         *                         url, "?", authenticationProperties.getHeaderName(), loginContext.getLoginToken()
         *                 )
         */
        return ResponseBeanBuilder.ok(fileRes);
    }

    @Override
    public ResponseBean<FileRes> uploadTwo(MultipartFile file) {
        String originalFilename = file.getOriginalFilename();
        String suffix = FileUtils.getSuffix(originalFilename, SystemConstant.FILE_NAME_SPLIT_SYMBOL);
        String prefix = FileUtils.getPrefix(originalFilename, SystemConstant.FILE_NAME_SPLIT_SYMBOL);

        String timeStr = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmss"));

        //查询是否重名
        for (Integer one = 1; one < 10; one++) {
            Integer i = one;
            LambdaQueryWrapper<FileInfo> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(FileInfo::getName, originalFilename);
            List<FileInfo> all = fileInfoMapper.selectList(queryWrapper);
            if (all.size() > 0) {
                originalFilename = prefix + "(" + i + "）" + SystemConstant.FILE_NAME_CONCAT_SYMBOL + suffix;
                i++;
                LambdaQueryWrapper<FileInfo> twoWrapper = new LambdaQueryWrapper<>();
                twoWrapper.eq(FileInfo::getName, originalFilename);
                List<FileInfo> fileInfos = fileInfoMapper.selectList(twoWrapper);
                if (fileInfos.size() > 0) {
                    i++;
                    originalFilename = prefix + "(" + i + "）" + SystemConstant.FILE_NAME_CONCAT_SYMBOL + suffix;
                }
            }
        }

        String uri = StringUtils.join(
                fileSystemProperties.getFileTempDir(),
                SystemConstant.URI_CONCAT_SYMBOL,
                timeStr,
                SystemConstant.URI_CONCAT_SYMBOL,
                originalFilename
        );

        String finalName = StringUtils.join(
                originalFilename
        );
        System.out.println(finalName);
        String diskPath = StringUtils.join(
                fileSystemProperties.getFileStoreDirPath(),
                fileSystemProperties.getFileTempDir(),
                File.separator,
                timeStr,
                File.separator,
                finalName
        );
        String fileTimeStr = StringUtils.join(
                fileSystemProperties.getFileStoreDirPath(),
                fileSystemProperties.getFileTempDir(),
                File.separator,
                timeStr);
        File file1 = new File(fileTimeStr);
        if (!file1.exists()) {
            file1.mkdirs();
        }

        String url = StringUtils.join(
                fileSystemProperties.getDownloadPrefix(),
                uri
        );
        try {
            //写入文件
            FileUtils.storeFileByInputStream(
                    file.getInputStream(),
                    diskPath
            );
        } catch (IOException e) {
            e.printStackTrace();
            return ResponseBeanBuilder.fail("文件上传失败");
        }
        FileInfo build = FileInfo
                .builder()
                .name(finalName)
                .originalName(originalFilename)
                .diskPath(diskPath)
                .size(file.getSize())
                .suffix(suffix)
                .uri(finalName)
                .build();
        fileInfoMapper.insert(build);
        FileRes fileRes = new FileRes();
        BeanUtils.copyProperties(build, fileRes);
        fileRes.setUrl(url);
        return ResponseBeanBuilder.ok(fileRes);
    }

    @Override
    public ResponseBean<FileRes> getByid(Long id) {
        FileInfo fileInfo = fileInfoMapper.selectById(id);

        String uri = fileInfo.getUri();


        String url = StringUtils.join(
                fileSystemProperties.getDownloadPrefix(),
                uri
        );
        FileRes fileRes = new FileRes();
        BeanUtils.copyProperties(fileInfo, fileRes);
        fileRes.setUrl(url);
        return ResponseBeanBuilder.ok(fileRes);
    }

}
