package com.factory.init;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import com.factory.base.entity.task.ApproveEntry;
import com.factory.service.FirstTaskService;
import com.factory.service.OnsiteTaskService;

import lombok.extern.slf4j.Slf4j;

/**
 * 首件任务定时器
 */
@Slf4j
@DisallowConcurrentExecution
public class AutoApproveErrorTaskJob implements Job {

	private static int type2 = 2;
	
	private static int type3 = 3;

    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {
        ConcurrentMap<String, Object> paramMap = (ConcurrentMap<String, Object>) context.getJobDetail().getJobDataMap().get(QuartzManager.JOB_PARAM_NAME);
        try {
            createTask(paramMap);// 执行任务
        } catch (Throwable e) {
        	 log.error("execute出错",e);
        }
    }

    /**
     * 创建Task主方法
     *
     * @param paramMap
     */
    public void createTask(ConcurrentMap<String, Object> paramMap) {
    	ExecutorService executorService = (ExecutorService) paramMap.get("executorService");
        Integer result = 0;//没什么意义只是为了不出现整体并行
        List<Callable<Integer>> tasks = new ArrayList<>();
        tasks.add(getFutureTask(paramMap, type2));
        tasks.add(getFutureTask(paramMap, type3));
        try {
        	List<Future<Integer>> rlist = executorService.invokeAll(tasks);
            for (Future<Integer> t : rlist) {
            	result+=t.get();
            }//等等所有线程结束
        } catch (Throwable e) {
        	log.error("createTask出错",e);
        }

    }

    private Callable<Integer> getFutureTask(ConcurrentMap<String, Object> paramMap, int type) {
    	Callable<Integer> task = new ApproveTaskWorker(paramMap, type);
        return task;
    }


    /**
     * 生产任务
     */
    public class ApproveTaskWorker implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private int type;


        public ApproveTaskWorker(ConcurrentMap<String, Object> paramMap, int type) {
            this.paramMap = paramMap;
            this.type = type;
        }

        @Override
        public Integer call() throws Exception {
            FirstTaskService firstTaskService = (FirstTaskService) paramMap.get("firstTaskService");
            OnsiteTaskService onsiteTaskService = (OnsiteTaskService) paramMap.get("onsiteTaskService");
            HashMap<String,Object> queryMap = new HashMap<>();
            if (type == type2) {//首件
                int maxlevel = firstTaskService.getMaxApproveLevel(null);
                queryMap.put("approval_level", maxlevel);
                List<ApproveEntry> approves = firstTaskService.getApproveTaskNoDeal(queryMap);
                log.debug("FA需要自动升级放行：{}",approves.size());
                for (ApproveEntry ap : approves) {
                    try {
                        firstTaskService.addTaskForNext(ap);
                    } catch (Throwable e) {
                        e.printStackTrace();
                    }
                }
            } else {//防错
                int maxlevel = onsiteTaskService.getMaxApproveLevel(null);
                queryMap.put("approval_level", maxlevel);
                List<ApproveEntry> approves = onsiteTaskService.getApproveTaskNoDeal(queryMap);
                log.debug("OT需要自动升级放行：{}",approves.size());
                for (ApproveEntry ap : approves) {
                    try {
                        onsiteTaskService.addTaskForNext(ap);
                    } catch (Throwable e) {
                        e.printStackTrace();
                    }
                }
            }

            return 0;
        }
    }

}
