package com.factory.init;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

import org.apache.commons.lang3.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import com.factory.base.entity.task.FirstCalendar;
import com.factory.base.entity.task.SendMsgReq;
import com.factory.base.util.time.TimeUtils;
import com.factory.service.FirstTaskService;
import com.factory.service.MsgService;
import com.factory.websocket.WsSessionManager;

import lombok.extern.slf4j.Slf4j;

/**
 * onsite 提醒job
 */
@Slf4j
@DisallowConcurrentExecution
public class FirstRemindJob implements Job {


    private int remind_handle_flag = 1;//设置提醒为已经处理
    private int type = 1;//首件

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        ConcurrentMap<String, Object> paramMap = (ConcurrentMap<String, Object>) jobExecutionContext.getJobDetail().getJobDataMap().get(QuartzManager.JOB_PARAM_NAME);
        try {
            createTask(paramMap);// 执行任务
        } catch (Throwable e) {
        	 log.error("execute生产任务出错",e);
        }
    }

    /**
     * 创建Task主方法
     *
     * @param paramMap
     */
    public void createTask(ConcurrentMap<String, Object> paramMap) {
    	ExecutorService executorService = (ExecutorService) paramMap.get("executorService");
        Integer result = 0;//没什么意义只是为了不出现整体并行
        List<Callable<Integer>> tasks = new ArrayList<>();
        tasks.add(getFirstFutureTask(paramMap, true));
        tasks.add(get200FutureTask(paramMap, true));
        tasks.add(getFirstFutureTask(paramMap, false));
        tasks.add(get200FutureTask(paramMap, false));
        try {
        	List<Future<Integer>> rlist = executorService.invokeAll(tasks);
            for (Future<Integer> t : rlist) {
            	result+=t.get();
            }//等等所有线程结束
        } catch (Throwable e) {
        	 log.error("createTask生产任务出错",e);
        }

    }


    /**
     * first
     *
     * @param paramMap
     * @param isWork
     * @return
     */
    private Callable<Integer> getFirstFutureTask(ConcurrentMap<String, Object> paramMap, boolean isWork) {
        HashMap<String,Object> param = new HashMap<>();
        param.put("type_flag", 0);
        Callable<Integer> cal = null;
        if (isWork) {
            param.put("remind_task_flag", 0);
            cal = new RemindFirstWorker(paramMap, param,0);
        } else {
            param.put("remind_nowork_flag", 0);
            cal = new RemindFirstNoWorker(paramMap, param,0);
        }
        
        return cal;
    }


    /**
     * 200
     *
     * @param paramMap
     * @param isWork
     * @return
     */
    private  Callable<Integer>  get200FutureTask(ConcurrentMap<String, Object> paramMap, boolean isWork) {
    	HashMap<String,Object> param = new HashMap<>();
        param.put("type_flag", 1);
        Callable<Integer> cal = null;
        if (isWork) {
            param.put("remind_task_flag", 0);
            cal = new RemindFirstWorker(paramMap, param,1);
        } else {
            param.put("remind_nowork_flag", 0);
            cal = new RemindFirstNoWorker(paramMap, param,1);
        }
        return cal;
    }

    /**
     * 发送提醒 需要工作的提醒
     */
    public class RemindFirstWorker implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private HashMap<String,Object> paramQuery;
        private int typeFlag=0;

        public RemindFirstWorker(ConcurrentMap<String, Object> paramMap, HashMap<String,Object> paramQuery,int typeFlag) {
            this.paramMap = paramMap;
            this.paramQuery = paramQuery;
            this.typeFlag = typeFlag;
        }


		@Override
		public Integer call() throws Exception {
			FirstTaskService firstTaskService = (FirstTaskService) paramMap.get("firstTaskService");
			// 获取全部微提醒的任务
			List<FirstCalendar> cals = firstTaskService.getRemindWork(paramQuery);
			LocalDateTime now = LocalDateTime.now();
			log.info("等待中1首件防错任务数量{}",cals.size());
			for (FirstCalendar cal : cals) {
				if (typeFlag==0&&StringUtils.isNotBlank(cal.getFirstRemind()) && now
						.isBefore(TimeUtils.paserDayTimeString(cal.getTaskTime() + " " + cal.getFirstRemind() + ":00"))) {//首件
					// 还未到时间
					continue;
				} else if (typeFlag==1&&now.isBefore(TimeUtils.paserDayTimeString(cal.getTaskTime() + " "+cal.getTwoRemind()+":00"))) {//200件
					continue;
				}
				log.info("发送超期提醒到PAD{}",cal);
				String tokenA = String.valueOf(cal.getPrincipalOneId());
				String tokenB = String.valueOf(cal.getPrincipalTwoId());
				Arrays.asList(tokenA, tokenB).forEach(token -> {
					if (WsSessionManager.SESSION_POOL.containsKey(token)) {
						WsSessionManager.sendMessage(token, "有一个首件巡检需要处理，请及时操作！");
						Map<String, Object> qm = new HashMap<>();
						qm.put("flag", "remind_task_flag");
						qm.put("flagVal", remind_handle_flag);
						qm.put("first_task_id", cal.getFirstTaskId());
						firstTaskService.updateFirstRemindFlag(qm);
					}else {
	                	log.warn("FA巡检负责人不在线{}/{}",token,cal.getFirstTaskId());
	                }
				});
			}
			return 0;
		}
    }


    /**
     * 发送提醒  没有工作的提醒
     */
    public class RemindFirstNoWorker implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private HashMap<String,Object> paramQuery = new HashMap<>();
        private  int typeFlag;

        public RemindFirstNoWorker(ConcurrentMap<String, Object> paramMap, HashMap<String,Object> paramQuery,int typeFlag) {
            this.paramMap = paramMap;
            this.typeFlag = typeFlag;
            this.paramQuery = paramQuery;
            paramQuery.put("type_flag", typeFlag);
        }

        @Override
        public Integer call() throws Exception {
            FirstTaskService firstTaskService = (FirstTaskService) paramMap.get("firstTaskService");
            MsgService msgService = (MsgService) paramMap.get("msgService");
            List<FirstCalendar> cals = firstTaskService.getRemindWork(paramQuery);
            LocalDateTime now = LocalDateTime.now();
            log.info("等待中2首件防错任务数量{}",cals.size());
            for (FirstCalendar cal : cals) {
                if (typeFlag==0&&StringUtils.isNotBlank(cal.getFirstRemind()) && now
						.isBefore(TimeUtils.paserDayTimeString(cal.getTaskTime() + " " + cal.getFirstRemind() + ":00").plusMinutes(cal.getUpleverTimeNum()))) {
					// 还未到时间
					continue;
				} else if (typeFlag==1&&now.isBefore(TimeUtils.paserDayTimeString(cal.getTaskTime() + " "+cal.getTwoRemind()+":00").plusMinutes(cal.getUpleverTimeNum()))) {
					continue;
				}

                log.info("发送超期提醒到小程序{}",cal);
                Map<String,Object> map = new HashMap<>();
                map.put("type", type);
                Long roleId = firstTaskService.getRemindNoworkToManager(map);
                if (roleId != null && roleId > 0) {
                    map = new HashMap<>();
                    map.put("task_id", cal.getFirstTaskId());
                    map.put("approval_role_id", roleId);
                    map.put("task_type", type);
                    map.put("flag", "remind_nowork_flag");
                    map.put("flagVal", remind_handle_flag);
                    map.put("first_task_id", cal.getFirstTaskId());
                    firstTaskService.addRemindNowork(map);
                    
                    Map<String,Object> plan = firstTaskService.getApprovalPlanInfo(cal.getFirstTaskId());
                    String planName = plan.get("planName").toString();
            		planName = planName.length()>15?(planName.substring(0,12)+"..."):planName;
                  
                    //需要推动调用这个 todo
                    //发送微信消息
                    SendMsgReq sendMsgReq = new SendMsgReq();
                    sendMsgReq.setToUserId(roleId);
                    sendMsgReq.setTitle("计划提醒("+plan.get("typeFlagStr")+")");
                    sendMsgReq.setContent(planName+"待提醒");
                    sendMsgReq.setPageIndex("/pagesPlan/reminder");
                    msgService.sendWechatMessageByRole(sendMsgReq,roleId);
                }

            }
            return 0;
        }
    }

}
