package com.factory.init;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.factory.base.entity.task.FirstCalendar;
import com.factory.base.entity.task.FirstTask;
import com.factory.common.CommonUtil;
import com.factory.service.FirstTaskService;

/**
 * 首件任务定时器
 */
@DisallowConcurrentExecution
public class FirstTaskJob implements Job {
    private static Logger log = LoggerFactory.getLogger(FirstTaskJob.class);

    @Override
    public void execute(JobExecutionContext context) {
        ConcurrentMap<String, Object> paramMap = (ConcurrentMap<String, Object>) context.getJobDetail().getJobDataMap().get(QuartzManager.JOB_PARAM_NAME);
        try {
            createTask(paramMap);// 执行任务
        } catch (Throwable e) {
        	log.error("execute生产任务出错",e);
        }
    }

    /**
     * 创建Task主方法
     *
     * @param paramMap
     */
    public void createTask(ConcurrentMap<String, Object> paramMap) {
    	ExecutorService executorService = (ExecutorService) paramMap.get("executorService");
        try {
            Future<Integer> future = executorService.submit(new AddTaskWorker(paramMap));
            future.get();
        } catch (Throwable e) {
        	log.error("createTask生产任务出错",e);
        }

    }

    /**
     * 生产任务
     */
    public class AddTaskWorker implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;

        public AddTaskWorker(ConcurrentMap<String, Object> paramMap) {
            this.paramMap = paramMap;
        }

        @Override
        public Integer call() {
            FirstTaskService firstTaskService = (FirstTaskService) paramMap.get("firstTaskService");
            LocalDate now = LocalDate.now();
            int num = 0;
            List<FirstCalendar> cals = null;
            FirstTask task = new FirstTask();
            task.setTask_time(now);
            for(int typeFlag=0;typeFlag<=1;typeFlag++) {//2中断在pad手动建立
            	cals = firstTaskService.getCalenderFirstForType(typeFlag,now);
            	num +=cals.size();
            	log.debug("FA需要生成新任务数量：{}",cals.size());
            	for (FirstCalendar cal : cals) {
            		try {
            			task.setFirst_article_calendar_id(cal.getId());
            			task.setType_flag(typeFlag);
            			firstTaskService.insertFirstTask(task);
            		} catch (Throwable e) {
            			log.error("addTask生产任务出错",e);
            		}
            	}
            }
            return num;
        }
    }

}
