package com.factory.init;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import com.factory.base.entity.task.OnsiteCalendar;
import com.factory.base.entity.task.SendMsgReq;
import com.factory.base.util.time.TimeUtils;
import com.factory.service.MsgService;
import com.factory.service.OnsiteTaskService;
import com.factory.websocket.WsSessionManager;

import lombok.extern.slf4j.Slf4j;

/**
 * onsite 提醒job
 */
@Slf4j
@DisallowConcurrentExecution
public class OnsiteRemindJob implements Job {


    private int remind_handle_flag = 1;//设置提醒为已经处理
    private int type = 2;//接错

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        ConcurrentMap<String, Object> paramMap = (ConcurrentMap<String, Object>) jobExecutionContext.getJobDetail().getJobDataMap().get(QuartzManager.JOB_PARAM_NAME);
        try {
            OnsiteTaskService onsiteTaskService = (OnsiteTaskService) paramMap.get("onsiteTaskService");
            //去掉，已经判断今天有任务不生成了
//            onsiteTaskService.delTaskOld(null);
            //  week_day month_day
            createTask(paramMap);// 执行任务
        } catch (Throwable e) {
        	log.error("execute生产任务出错",e);
        }
    }

    /**
     * 创建Task主方法
     *
     * @param paramMap
     */
    public void createTask(ConcurrentMap<String, Object> paramMap) {
    	ExecutorService executorService = (ExecutorService) paramMap.get("executorService");
        Integer result = 0;//没什么意义只是为了不出现整体并行
        List<Callable<Integer>> tasks = new ArrayList<>();
        tasks.add(getDayFutureTask(paramMap, true));
        tasks.add(getWeekFutureTask(paramMap, true));
        tasks.add(getMonthFutureTask(paramMap, true));
        tasks.add(getDayFutureTask(paramMap, false));
        tasks.add(getWeekFutureTask(paramMap, false));
        tasks.add(getMonthFutureTask(paramMap, false));
        try {
        	List<Future<Integer>> rlist = executorService.invokeAll(tasks);
            for (Future<Integer> t : rlist) {
            	result+=t.get();
            }//等等所有线程结束
        } catch (Throwable e) {
        	log.error("createTask生产任务出错",e);
        }

    }


    /**
     * day
     *
     * @param paramMap
     * @return
     */
    private Callable<Integer> getDayFutureTask(ConcurrentMap<String, Object> paramMap, boolean isWork) {
        HashMap<String,Object> param = new HashMap<>();
        param.put("type_flag", 0);
        LocalTime now = LocalTime.now();
        param.put("time", now.format(TimeUtils.timeMiniteFormatter));
        Callable<Integer> cal = null;
        if (isWork) {
            param.put("remind_task_flag", 0);
            cal = new RemindOnsiteWorker(paramMap, param);
        } else {
            param.put("remind_nowork_flag", 0);
            cal = new RemindOnsiteNoWorker(paramMap, param);
        }
        return cal;
    }

    /**
     * week
     *
     * @param paramMap
     * @return
     */
    private Callable<Integer> getWeekFutureTask(ConcurrentMap<String, Object> paramMap, boolean isWork) {
    	HashMap<String,Object> param = new HashMap<>();
        param.put("type_flag", 1);
        Callable<Integer> cal = null;
        if (isWork) {
            param.put("remind_task_flag", 0);
            cal = new RemindOnsiteWorker(paramMap, param);
        } else {
            param.put("remind_nowork_flag", 0);
            cal = new RemindOnsiteNoWorker(paramMap, param);
        }
        return cal;
    }

    /**
     * month
     *
     * @param paramMap
     * @return
     */
    private Callable<Integer> getMonthFutureTask(ConcurrentMap<String, Object> paramMap, boolean isWork) {
    	HashMap<String,Object> param = new HashMap<>();
        param.put("type_flag", 2);
        Callable<Integer> cal = null;
        if (isWork) {
            param.put("remind_task_flag", 0);
            cal = new RemindOnsiteWorker(paramMap, param);
        } else {
            param.put("remind_nowork_flag", 0);
            cal = new RemindOnsiteNoWorker(paramMap, param);
        }
        return cal;
    }


    /**
     * 发送提醒 需要工作的提醒
     */
    public class RemindOnsiteWorker implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private HashMap<String,Object> paramQuery;


        public RemindOnsiteWorker(ConcurrentMap<String, Object> paramMap, HashMap<String,Object> paramQuery) {
            this.paramMap = paramMap;
            this.paramQuery = paramQuery;
        }


        @Override
        public Integer call() throws Exception {
            OnsiteTaskService onsiteTaskService = (OnsiteTaskService) paramMap.get("onsiteTaskService");
            //获取了全部等待中的任务
            List<OnsiteCalendar> cals = onsiteTaskService.getRemindWork(paramQuery);
            LocalDateTime now = LocalDateTime.now();
            log.info("等待中1的防错任务数量{}",cals.size());
            try {
	            for (OnsiteCalendar cal : cals) {
	                if (cal.getType_flag()==0&&now.isBefore(TimeUtils.paserDayTimeString(cal.getTask_time()+ " " + cal.getStart_day()+":00").plusMinutes(cal.getDay_validate()))) {
	                   //还未到时间
	                   continue;
	                } 
	                log.debug("发送工作提醒到PAD{}", cal); 
	                String token = String.valueOf(cal.getUser_id());
	                if (WsSessionManager.SESSION_POOL.containsKey(token)) {
	                    
	                    WsSessionManager.sendMessage(token,"有一个防错巡检需要处理，请及时操作！");
	                    //update  onsite_task set ${flag}=${flagVal} where id=${onsite_task_id}
	                    Map<String, Object> qm = new HashMap<>();
	                    qm.put("flag", "remind_task_flag");
	                    qm.put("flagVal", remind_handle_flag);
	                    qm.put("onsite_task_id", cal.getOnsite_task_id());
	                    onsiteTaskService.updateOnsiteRemindFlag(qm);
	                }else {
	                	log.warn("OT巡检负责人不在线{}/{}",token,cal.getOnsite_task_id());
	                }
	            }
            } catch (Throwable e) {
            	log.error("remindOnsiteWork生产任务出错",e);
            }
            return 0;
        }
    }


    /**
     * 发送提醒  没有工作的提醒
     */
    public class RemindOnsiteNoWorker implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private HashMap<String,Object> paramQuery;


        public RemindOnsiteNoWorker(ConcurrentMap<String, Object> paramMap, HashMap<String,Object> paramQuery) {
            this.paramMap = paramMap;
            this.paramQuery = paramQuery;
        }

        @Override
        public Integer call() throws Exception {
            OnsiteTaskService onsiteTaskService = (OnsiteTaskService) paramMap.get("onsiteTaskService");
            MsgService msgService = (MsgService) paramMap.get("msgService");
            //获取了全部发送工作提醒但是到期
            List<OnsiteCalendar> cals = onsiteTaskService.getRemindWork(paramQuery);
            LocalDateTime now = LocalDateTime.now();
            log.info("等待中2的防错任务数量{}",cals.size());
            for (OnsiteCalendar cal : cals) {
				if (cal.getType_flag() == 0 && now
						.isBefore(TimeUtils.paserDayTimeString(cal.getTask_time() + " " + cal.getStart_day() + ":00").plusMinutes(cal.getDay_validate()+cal.getDay_outtime()))) {
					// 还未到时间
					continue;
				} else if (cal.getType_flag() == 1 && now.getDayOfWeek().getValue() < cal.getWeek_day()) {// 周
					// 还未到时间
					continue;
				} else if (cal.getType_flag() == 2 && now.getDayOfMonth() < cal.getMonth_day()) {// 月
					// 还未到时间
					continue;
				}
	            log.debug("发送超期提醒到小程序{}", cal); 
                Map<String,Object> map = new HashMap<>();
                map.put("type", type);
                Long roleId = onsiteTaskService.getRemindNoworkToManager(map);
                if (roleId != null && roleId > 0) {
                    map = new HashMap<>();
                    map.put("task_id", cal.getOnsite_task_id());
                    map.put("approval_role_id", roleId);
                    map.put("task_type", type);
                    map.put("flag", "remind_nowork_flag");
                    map.put("flagVal", remind_handle_flag);
                    map.put("onsite_task_id", cal.getOnsite_task_id());
                    onsiteTaskService.addRemindNowork(map);
                    Map<String,Object> plan = onsiteTaskService.getApprovalPlanInfo(cal.getOnsite_task_id());
                    String planName = plan.get("planName").toString();
            		planName = planName.length()>15?(planName.substring(0,12)+"..."):planName;
                
                    //需要推动调用这个 todo
                    //发送微信消息
                    SendMsgReq sendMsgReq = new SendMsgReq();
                    sendMsgReq.setToUserId(roleId);//3
                    sendMsgReq.setTitle("计划提醒("+plan.get("typeFlagStr")+"-防错)");
                    sendMsgReq.setContent(planName+"待提醒");
                    sendMsgReq.setPageIndex("/pagesPlan/reminder");
                    msgService.sendWechatMessageByRole(sendMsgReq,roleId);
                }
                

            }
            return 0;
        }
    }

    public static void main(String[] args) {
    	 LocalDateTime now = LocalDateTime.now();
    	 System.out.println(now.getDayOfWeek().getValue());
    }
}
