package com.factory.init;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;

import org.quartz.DisallowConcurrentExecution;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

import com.factory.base.entity.task.OnsiteCalendar;
import com.factory.base.entity.task.OnsiteTask;
import com.factory.base.util.time.TimeUtils;
import com.factory.service.OnsiteTaskService;

import lombok.extern.slf4j.Slf4j;


/**
 * onsite生产任务job
 */
@Slf4j
@DisallowConcurrentExecution
public class OnsiteTaskJob implements Job {

    @Override
    public void execute(JobExecutionContext context) throws JobExecutionException {
        ConcurrentMap<String, Object> paramMap = (ConcurrentMap<String, Object>) context.getJobDetail().getJobDataMap().get(QuartzManager.JOB_PARAM_NAME);
        try {
            createTask(paramMap);// 执行任务
        } catch (Throwable e) {
        	log.error("execute生产任务出错",e);
        }
    }

    /**
     * 创建Task主方法
     *
     * @param paramMap
     */
    public void createTask(ConcurrentMap<String, Object> paramMap) {
    	ExecutorService executorService = (ExecutorService) paramMap.get("executorService");
        Integer result = 0;//没什么意义只是为了不出现整体并行
        List<Callable<Integer>> tasks = new ArrayList<>();
        tasks.add(getDayFutureTask(paramMap));
        tasks.add(getWeekFutureTask(paramMap));
        tasks.add(getMonthFutureTask(paramMap));
        try {
        	List<Future<Integer>> rlist = executorService.invokeAll(tasks);
            for (Future<Integer> t : rlist) {
            	result+=t.get();
            }//等等所有线程结束
        } catch (Throwable e) {
        	log.error("createTask生产任务出错",e);
        }

    }

    /**
     * day
     *
     * @param paramMap
     * @return
     */
    private Callable<Integer> getDayFutureTask(ConcurrentMap<String, Object> paramMap) {
        HashMap<String,Object> param = new HashMap<>();
        param.put("type_flag", 0);
        param.put("today", LocalDate.now());
        Callable<Integer> dayTask = new AddTaskWorker(paramMap, param);
        
        return dayTask;
    }

    /**
     * week
     *
     * @param paramMap
     * @return
     */
    private Callable<Integer> getWeekFutureTask(ConcurrentMap<String, Object> paramMap) {
    	HashMap<String,Object> param = new HashMap<>();
        param.put("type_flag", 1);
        LocalDate[] se = TimeUtils.getStartAndEndDay(LocalDate.now(), 1, 0);
        param.put("sday", se[0]);
        param.put("eday", se[1]);
        param.put("today", se[1]);
        Callable<Integer> weekTask = new AddTaskWorker(paramMap, param);
        return weekTask;
    }

    /**
     * month
     *
     * @param paramMap
     * @return
     */
    private Callable<Integer> getMonthFutureTask(ConcurrentMap<String, Object> paramMap) {
    	HashMap<String,Object> param = new HashMap<>();
        param.put("type_flag", 2);
        LocalDate[] se = TimeUtils.getStartAndEndDay(LocalDate.now(), 2, 0);
        param.put("sday", se[0]);
        param.put("eday", se[1]);
        param.put("today", se[1]);
        Callable<Integer> monthTask =new AddTaskWorker(paramMap, param);
        return monthTask;
    }


    /**
     * 生产任务
     */
    public class AddTaskWorker implements Callable<Integer> {

        private ConcurrentMap<String, Object> paramMap;
        private HashMap<String,Object> paramQuery;


        public AddTaskWorker(ConcurrentMap<String, Object> paramMap, HashMap<String,Object> paramQuery) {
            this.paramMap = paramMap;
            this.paramQuery = paramQuery;
        }

        @Override
        public Integer call() throws Exception {
        	LocalDate today = (LocalDate)paramQuery.get("today");
            OnsiteTaskService onsiteTaskService = (OnsiteTaskService) paramMap.get("onsiteTaskService");
            //直接查询没有生成任务的
            List<OnsiteCalendar> cals = onsiteTaskService.getCalenderOnsitesForType(paramQuery);
            log.debug("OT需要生成新任务数量：{}",cals.size());
            for (OnsiteCalendar cal : cals) {
                try {
                        OnsiteTask task = new OnsiteTask();
                        task.setTask_time(today);//周末和月末
                        task.setOnsite_calendar_id(cal.getId());
                        onsiteTaskService.insertOnsiteTask(task);

                } catch (Throwable e) {
                    log.error("addTask生产任务出错",e);
                }

            }
            return 0;
        }
    }


}
