package com.factory.init;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ExecutorService;

import javax.annotation.PostConstruct;

import org.quartz.Scheduler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.quartz.SchedulerFactoryBean;
import org.springframework.stereotype.Component;

import com.factory.common.TaskProperties;
import com.factory.service.FirstTaskService;
import com.factory.service.MsgService;
import com.factory.service.OnsiteTaskService;

/**
 * 这个是启动就触发的部分
 */
@Component
public class TimerTaskInit {

    @Value("${task.onsite.cron:'0 */5 * * * ?'}")
    private String taskOnsiteCron;
    @Value("${task.onsite.remind:'0 */5 * * * ?'}")
    private String taskOnsiteRemind;

    @Value("${task.first.cron:'0 */5 * * * ?'}")
    private String taskFirstCron;
    @Value("${task.first.remind:'0 */5 * * * ?'}")
    private String taskFirstRemind;
    
    @Value("${task.pass.cron:'0 */1 * * * ?'}")
    private String taskPassCron;

    @Autowired
    private TaskProperties taskProperties;
    
    @Autowired
    OnsiteTaskService onsiteTaskService;

    @Autowired
    FirstTaskService firstTaskService;

    @Autowired
    SchedulerFactoryBean schedulerFactoryBean;

    @Autowired
    MsgService msgService;
    @Autowired
	private ExecutorService executorService;

    @PostConstruct
    public void run() {
        ConcurrentMap<String, Object> param = new ConcurrentHashMap<>(16);
        param.put("onsiteTaskService", onsiteTaskService);
        param.put("firstTaskService", firstTaskService);
        param.put("msgService", msgService);
        param.put("executorService", executorService);
        Scheduler sche = schedulerFactoryBean.getScheduler();
        addAutoApproveErrorTask(param, sche);
//        addOnsiteTask(param, sche);//放在schudule定时器中执行
        addOnsiteRemindWork(param, sche);
//        addFirstTask(param, sche);//放在schudule定时器中执行
        addFirstRemindWork(param, sche);
    }

    /**
     * 添加异常自动审批
     *
     * @param param
     * @param sche
     */
    private void addAutoApproveErrorTask(ConcurrentMap<String, Object> param, Scheduler sche) {
        JobFactoryParam onsite = new JobFactoryParam();
        onsite.setJobGroupName(QuartzManager.JOB_GROUP_NAME);
        onsite.setJobName("errAutoTask");
        onsite.setParamMap(param);
        onsite.setTriggerRule(taskProperties.getPass().getCron());
        onsite.setClazz(AutoApproveErrorTaskJob.class);
        QuartzManager.addJob(sche, onsite);
    }

    /**
     * 添加onsit任务的定时器
     *
     * @param param
     * @param sche
     */
    private void addOnsiteTask(ConcurrentMap<String, Object> param, Scheduler sche) {
        JobFactoryParam onsite = new JobFactoryParam();
        onsite.setJobGroupName(QuartzManager.JOB_GROUP_NAME);
        onsite.setJobName("onsiteTask");
        onsite.setParamMap(param);
        onsite.setTriggerRule(taskProperties.getOnsite().getCron());//"0 */1 * * * ?" 这个用于测试一分钟一次
        onsite.setClazz(OnsiteTaskJob.class);
        QuartzManager.addJob(sche, onsite);
    }

    private void addFirstTask(ConcurrentMap<String, Object> param, Scheduler sche) {
        JobFactoryParam onsite = new JobFactoryParam();
        onsite.setJobGroupName(QuartzManager.JOB_GROUP_NAME);
        onsite.setJobName("firstTask");
        onsite.setParamMap(param);
        onsite.setTriggerRule(taskProperties.getFirst().getCron());//"0 */1 * * * ?" 这个用于测试一分钟一次
        onsite.setClazz(FirstTaskJob.class);
        QuartzManager.addJob(sche, onsite);
    }


    /**
     * 添加onsit工作提醒
     *
     * @param param
     * @param sche
     */
    private void addOnsiteRemindWork(ConcurrentMap<String, Object> param, Scheduler sche) {
        JobFactoryParam onsite = new JobFactoryParam();
        onsite.setJobGroupName(QuartzManager.JOB_GROUP_NAME);
        onsite.setJobName("onsiteRemindWork");
        onsite.setParamMap(param);
        onsite.setTriggerRule(taskProperties.getOnsite().getRemind());//"0 */1 * * * ?" 这个用于测试一分钟一次
        onsite.setClazz(OnsiteRemindJob.class);
        QuartzManager.addJob(sche, onsite);
    }

    private void addFirstRemindWork(ConcurrentMap<String, Object> param, Scheduler sche) {
        JobFactoryParam onsite = new JobFactoryParam();
        onsite.setJobGroupName(QuartzManager.JOB_GROUP_NAME);
        onsite.setJobName("firstRemindWork");
        onsite.setParamMap(param);
        onsite.setTriggerRule(taskProperties.getFirst().getRemind());//"0 */1 * * * ?" 这个用于测试一分钟一次
        onsite.setClazz(FirstRemindJob.class);
        QuartzManager.addJob(sche, onsite);
    }

}
