package com.factory.service.impl;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.factory.base.entity.task.ApprovalUserRes;
import com.factory.base.entity.task.ApproveEntry;
import com.factory.base.entity.task.FirstCalendar;
import com.factory.base.entity.task.FirstTask;
import com.factory.base.entity.task.SendMsgReq;
import com.factory.base.util.time.TimeUtils;
import com.factory.mapper.FirstTaskMapper;
import com.factory.service.FirstTaskService;
import com.factory.service.MsgService;

@Service
public class FirstTaskServiceImp implements FirstTaskService {

    @Autowired
    FirstTaskMapper firstTaskMapper;
    @Autowired
    MsgService msgService;

    /**
     * 获取对应的规则
     *
     * @return
     */
    @Override
    public List<FirstCalendar> getCalenderFirstForType(Integer typeFlag,LocalDate now) {
        return firstTaskMapper.getCalenderFirstForType(typeFlag,now);
    }

    /**
     * 添加当天任务
     *
     * @param task
     * @return
     */
    @Override
    public int insertFirstTask(FirstTask task) {
    	firstTaskMapper.insertFirstTaskKey(task);
        return firstTaskMapper.insertFirstTask(task);
    }

    /**
     * 需要提醒的工作
     *
     * @param param
     * @return
     */
    @Override
    public List<FirstCalendar> getRemindWork(Map<String, Object> param) {
        return firstTaskMapper.getRemindWork(param);
    }


    /**
     * 更新提醒状态，不太好的写法但是符合现在的需求
     *
     * @param param
     * @return
     */
    @Override
    public int updateFirstRemindFlag(Map<String, Object> param) {
        return firstTaskMapper.updateFirstRemindFlag(param);
    }

    /**
     * 添加不工作提醒信息
     *
     * @param map
     * @return
     */
    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int addRemindNowork(Map<String, Object> map) {
        int result = firstTaskMapper.addRemindNowork(map);
        firstTaskMapper.updateFirstRemindFlag(map);
        return result;
    }

    /**
     * 获取需要提醒人
     *
     * @param map
     * @return
     */
    @Override
    public Long getRemindNoworkToManager(Map<String, Object> map) {
        return firstTaskMapper.getRemindNoworkToManager(map);
    }


    /**
     * 获取最大审批级别
     *
     * @param map
     * @return
     */
    @Override
    public int getMaxApproveLevel(Map<String, Object> map) {
        return firstTaskMapper.getMaxApproveLevel(map);
    }

    /**
     * 或许超时没处理记录
     *
     * @param map
     * @return
     */
    @Override
    public List<ApproveEntry> getApproveTaskNoDeal(Map<String, Object> map) {
        return firstTaskMapper.getApproveTaskNoDeal(map);
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int addTaskForNext(ApproveEntry vo) {
        HashMap<String,Object> map = new HashMap<>();
        map.put("approvalLevel", vo.getApproval_level());
        vo.setStatus(2);
        vo.setApproval_user_id(0);
        ApprovalUserRes nextRole = firstTaskMapper.getApprovalNextUser(map);
        if (nextRole != null) {
            int result = firstTaskMapper.updateApproveTask(vo);
            if (result > 0) {
                ApproveEntry approve = new ApproveEntry();
                approve.setTask_id(vo.getTask_id());
                approve.setApproval_level(nextRole.getApproval_level());
                approve.setApproval_role(nextRole.getRole_id());
                
                int status = firstTaskMapper.addApproveTask(approve);
                if (status > 0) {
                    //发送微信消息
                    Map<String,Object> plan = firstTaskMapper.getApprovalPlanInfo(vo.getTask_id());
            		String planName = plan.get("planName").toString();
            		planName = planName.length()>15?(planName.substring(0,12)+"..."):planName;
                    SendMsgReq sendMsgReq = new SendMsgReq();
                    sendMsgReq.setToUserId(nextRole.getUser_id());
                    sendMsgReq.setTitle("计划放行("+plan.get("typeFlagStr")+")");
                    sendMsgReq.setContent(planName+"待审批");
                    sendMsgReq.setEndTime(TimeUtils.getDayString(LocalDateTime.now()));
                    sendMsgReq.setPageIndex("/pagesPlan/pass");
                    msgService.sendWechatMessageByRole(sendMsgReq,nextRole.getRole_id());
                }
                return status;
            }
        }
        return 0;
    }

	@Override
	public Map<String, Object> getApprovalPlanInfo(Long taskId) {
		return firstTaskMapper.getApprovalPlanInfo(taskId);
	}


}
