package com.factory.service.impl;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.factory.base.entity.task.AppEntry;
import com.factory.base.entity.task.PushWxMessage;
import com.factory.base.entity.task.SendMsgReq;
import com.factory.base.entity.task.TemplateEntry;
import com.factory.base.entity.task.ValEntry;
import com.factory.base.util.time.TimeUtils;
import com.factory.common.HttpCilentUntil;
import com.factory.mapper.UserMapper;
import com.factory.service.MsgService;

import lombok.extern.slf4j.Slf4j;

/**
 * @author sMArT
 * @version 1.0
 * @date 2021-08-04 10:56 上午
 */
@Slf4j
@Service
public class MsgServiceImpl implements MsgService {
    @Value("${wx.minprogram.appsecret}")
    private String appsecret;

    @Value("${wx.minprogram.appid}")
    private String appid;

    @Value("${wx.minprogram.templateId}")
    private String templateId;

    @Value("${wx.minprogram.page}")
    private String page;

    @Value("${wx.minprogram.miniprogramState}")
    private String miniprogramState;

    @Autowired
    private UserMapper userMapper;

    @Override
    public String sendWechatMessage(SendMsgReq sendMsgReq) {
        try {
            Map params = new HashMap();
            params.put("secret", appsecret);
            params.put("appid", appid);
            params.put("grant_type", "client_credential");

            HashMap<String, ValEntry> dataMap = new HashMap();
            dataMap.put("thing1", new ValEntry(sendMsgReq.getContent()));
            dataMap.put("thing2", new ValEntry(sendMsgReq.getTitle()));
            if (null == sendMsgReq.getEndTime()) {
                Date date = new Date();
                SimpleDateFormat ft = new SimpleDateFormat("yyyy-MM-dd");
                dataMap.put("time3", new ValEntry(ft.format(date)));
            } else {
                dataMap.put("time3", new ValEntry(sendMsgReq.getEndTime()));
            }
            dataMap.put("thing4", new ValEntry("智云工厂系统"));

            String openId = userMapper.getOpenIdByUserId(sendMsgReq.getToUserId());
            TemplateEntry template = new TemplateEntry();
            template.setTemplate_id(templateId);
            template.setTouser(openId);
            template.setPage(sendMsgReq.getPageIndex()==null?page:sendMsgReq.getPageIndex());
            template.setMiniprogram_state(miniprogramState);
            template.setData(dataMap);

            String result = HttpCilentUntil.httpRequestToString(
                    "https://api.weixin.qq.com/cgi-bin/token", params);
            JSONObject jsonObject = JSON.parseObject(result);
            String access_token = jsonObject.get("access_token").toString();
            String pushUrl = new StringBuffer("https://api.weixin.qq.com/cgi-bin/message/subscribe/send?access_token=").append(access_token).toString();
            result = HttpCilentUntil.httpJsonToString(pushUrl, JSON.toJSONString(template));
            jsonObject = JSON.parseObject(result);
            String errCode = jsonObject.get("errcode").toString();
            String errMsg = jsonObject.get("errmsg").toString();
            System.out.println(errCode);
            return errCode;
        } catch (Throwable e) {
            e.printStackTrace();
            return "-1";
        }
    }

	@Override
	public String sendWechatMessageByRole(SendMsgReq sendMsgReq, Long roleId) {
		List<Long> list = userMapper.getUserIdsByRoleId(roleId);
		if(list.isEmpty()) {
	        log.error("下发微信消息，角色:{}接收人为空", roleId);
			return "-1";
		}
		log.info("发消息内容{}",sendMsgReq);
		HashMap<String, ValEntry> dataMap = new HashMap<>();
		 //提醒内容
        dataMap.put("thing1", ValEntry.builder().value(sendMsgReq.getContent()).build());
        //作业名称
        dataMap.put("thing2", ValEntry.builder().value(sendMsgReq.getTitle()).build());
        //截止日期
        if (null != sendMsgReq.getEndTime()) {
            dataMap.put("time3", ValEntry.builder().value(sendMsgReq.getEndTime()).build());
        } else {
            dataMap.put("time3", ValEntry.builder().value(TimeUtils.getDayString(LocalDateTime.now())).build());
        }
        dataMap.put("thing4", new ValEntry("智云工厂系统"));
        Set<String> openIdSet = new HashSet<>();
		for(Long userId:list) {
			try {
				String openId = userMapper.getOpenIdByUserId(userId);
				if(!openIdSet.add(openId)) {
					log.warn("下发微信消息，接收人userId:{}的openId重复",sendMsgReq.getToUserId());
					continue;
				}
				if (null != openId && !openId.isEmpty()) {
					AppEntry appEntry = AppEntry.builder().appId(appid).secret(appsecret).build();
					TemplateEntry tpl = TemplateEntry.builder().template_id(templateId).touser(openId)
							.page(sendMsgReq.getPageIndex()==null?page:sendMsgReq.getPageIndex())
							.miniprogram_state(miniprogramState).data(dataMap).build();
					PushWxMessage.sendMessage(appEntry, tpl);
				} else {
					log.error("下发微信消息，接收人userId:{}的openId为空", sendMsgReq.getToUserId());
				}
			} catch (Exception e) {
				 log.error("下发微信消息异常",e);
				 return "-1";
			}
		}
		return "1";
	}
    
    
}
