package com.factory.service.impl;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.factory.base.entity.task.ApprovalUserRes;
import com.factory.base.entity.task.ApproveEntry;
import com.factory.base.entity.task.OnsiteCalendar;
import com.factory.base.entity.task.OnsiteTask;
import com.factory.base.entity.task.SendMsgReq;
import com.factory.base.util.time.TimeUtils;
import com.factory.mapper.OnsiteTaskMapper;
import com.factory.service.MsgService;
import com.factory.service.OnsiteTaskService;

@Service("onsiteTaskServiceImp")
public class OnsiteTaskServiceImp implements OnsiteTaskService {


    @Autowired
    OnsiteTaskMapper onsiteTaskMapper;
    @Autowired
    private MsgService msgService;

    public int getTaskExistCount(Map<String,Object> map) {
        return onsiteTaskMapper.getTaskExistCount(map);
    }

    public int delTaskOld(Map<String,Object> map) {
        return onsiteTaskMapper.delTaskOld(map);
    }

    @Override
    @Transactional(rollbackFor = Throwable.class)
    public int insertOnsiteTask(OnsiteTask task) {
        return onsiteTaskMapper.insertOnsiteTask(task);
    }


    /**
     * 获取对应的规则
     *
     * @param paarm
     * @return
     */
    @Override
    public List<OnsiteCalendar> getCalenderOnsitesForType(Map<String, Object> param) {
        return onsiteTaskMapper.getCalenderOnsitesForType(param);
    }

    /**
     * 需要提醒的工作
     *
     * @param param
     * @return
     */
    public List<OnsiteCalendar> getRemindWork(Map<String, Object> param) {
        return onsiteTaskMapper.getRemindWork(param);
    }

    /**
     * 更新提醒状态，不太好的写法但是符合现在的需求
     *
     * @param param
     * @return
     */
    public int updateOnsiteRemindFlag(Map<String, Object> param) {
        return onsiteTaskMapper.updateOnsiteRemindFlag(param);
    }


    /**
     * 获取需要提醒人
     *
     * @param map
     * @return
     */
    public Long getRemindNoworkToManager(Map<String,Object> map) {
        return onsiteTaskMapper.getRemindNoworkToManager(map);
    }

    /**
     * 添加不工作提醒信息
     *
     * @param map
     * @return
     */
    @Transactional(rollbackFor = Throwable.class)
    public int addRemindNowork(Map<String,Object> map) {
        int result = onsiteTaskMapper.addRemindNowork(map);
        onsiteTaskMapper.updateOnsiteRemindFlag(map);
        return result;

    }


    /**
     * 获取最大审批级别
     *
     * @param map
     * @return
     */
    public int getMaxApproveLevel(Map<String,Object> map) {
        return onsiteTaskMapper.getMaxApproveLevel(map);
    }

    /**
     * 或许超时没处理记录
     *
     * @param map
     * @return
     */
    public List<ApproveEntry> getApproveTaskNoDeal(Map<String,Object> map) {
        return onsiteTaskMapper.getApproveTaskNoDeal(map);
    }

    @Transactional(rollbackFor = Throwable.class)
    public int addTaskForNext(ApproveEntry vo) {
        vo.setStatus(2);
        vo.setApproval_user_id(0);
        HashMap<String,Object> map = new HashMap<>();
        map.put("approvalLevel", vo.getApproval_level());
        ApprovalUserRes nextRole = onsiteTaskMapper.getApprovalNextUser(map);
        if (nextRole != null) {
            int result = onsiteTaskMapper.updateApproveTask(vo);
            if (result > 0) {
                ApproveEntry approve = new ApproveEntry();
                approve.setTask_id(vo.getTask_id());
                approve.setApproval_level(nextRole.getApproval_level());
                approve.setApproval_role(nextRole.getRole_id());
                int status = onsiteTaskMapper.addApproveTask(approve);
                if (status > 0) {
                    Map<String,Object> plan = this.getApprovalPlanInfo(vo.getTask_id());
                    //发送微信消息
                    String planName = plan.get("planName").toString();
            		planName = planName.length()>15?(planName.substring(0,12)+"..."):planName;
                    SendMsgReq sendMsgReq = new SendMsgReq();
                    sendMsgReq.setToUserId(nextRole.getUser_id());
                    sendMsgReq.setTitle("计划放行("+plan.get("typeFlagStr")+"-防错)");
                    sendMsgReq.setContent(planName+"待审批");
                    sendMsgReq.setEndTime(TimeUtils.getDayString(LocalDateTime.now()));
                    sendMsgReq.setPageIndex("/pagesPlan/pass");
                    msgService.sendWechatMessageByRole(sendMsgReq,nextRole.getRole_id());
                }
                return status;
            }
        }
        return 0;
    }

    public Map<String,Object> getApprovalPlanInfo(Long taskId){
    	return onsiteTaskMapper.getApprovalPlanInfo(taskId);
    }
}
