package com.factory.websocket;

import org.springframework.stereotype.Component;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.TextWebSocketHandler;

//ws://127.0.0.1:8088/task/ws/pad?token=123617832 token 对应的是userid
@Component
public class WebSocketHandler extends TextWebSocketHandler {

    /**
     * 建立成功事件
     *
     * @param session
     * @throws Exception
     */
    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        Object token = session.getAttributes().get("token");
        if (token != null) {
            WsSessionManager.add(token.toString(), session);   // 用户连接成功，放入在线用户缓存  存储信息可以使用redis代替
        } else {
            throw new RuntimeException("用户登录已经失效!");
        }
    }


    /**
     * 接收消息事件
     *
     * @param session
     * @param message
     * @throws Exception
     */
    @Override
    protected void handleTextMessage(WebSocketSession session, TextMessage message) throws Exception {
        String payload = message.getPayload();
        Object token = session.getAttributes().get("token");
        session.sendMessage(new TextMessage("active"));
    }

    /**
     * 关闭
     *
     * @param session
     * @param status
     * @throws Exception
     */
    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus status) throws Exception {
        Object token = session.getAttributes().get("token");
        if (token != null) {
            WsSessionManager.remove(token.toString());
        }
    }

    /**
     * 异常处理
     *
     * @param session
     * @param exception
     * @throws Exception
     */
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
        WsSessionManager.removeAndClose(String.valueOf(session.getAttributes().get("token")));
    }

}
