package com.factory.websocket;
import java.io.IOException;
import java.util.concurrent.ConcurrentHashMap;

import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class WsSessionManager {

    public final static ConcurrentHashMap<String, WebSocketSession> SESSION_POOL = new ConcurrentHashMap<>();

    /**
     * 添加 session
     *
     * @param key
     */
    public static void add(String key, WebSocketSession session) {
        SESSION_POOL.put(key, session);
    }

    /**
     * 删除 session,会返回删除的 session
     *
     * @param key
     * @return
     */
    public static WebSocketSession remove(String key) {
        return SESSION_POOL.remove(key);
    }

    /**
     * 删除并同步关闭连接
     *
     * @param key
     */
    public static void removeAndClose(String key) {
        WebSocketSession session = remove(key);
        if (session != null) {
            try {
                session.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 获得 session
     *
     * @param key
     * @return
     */
    public static WebSocketSession get(String key) {
        return SESSION_POOL.get(key);
    }
    
    public static void sendMessage(String key,String message) {
    	try {
    		log.debug("send message to:{},content:{}",key,message);
			WebSocketSession session = WsSessionManager.get(key);
			if (session != null) {
				session.sendMessage(new TextMessage(message));
				log.debug("send message to:{} ok",key,message);
			}else {
				log.debug("send message to:{},fail,no session",key,message);
			}
    	}catch (Throwable e) {
    		log.error("send message error",e);
			WsSessionManager.removeAndClose(key);// 异常移除以免内存泄漏
		}
    }
}
