package com.factory.base.config.handler;


import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.util.res.ResponseBeanBuilder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpStatus;
import org.springframework.validation.BindException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import java.util.ArrayList;
import java.util.List;


@RestControllerAdvice
public class ExceptionController {

    /**
     * 异常
     */
    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler({Exception.class})
    public ResponseBean<String> exceptionHandler(Exception e) {
        e.printStackTrace();
        return ResponseBeanBuilder.fail(e.getMessage());
    }


    /**
     * 参数校验
     *
     * @param e
     * @return
     */
    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler({MethodArgumentNotValidException.class})
    public ResponseBean<String> handleMethodArgumentNotValidException(MethodArgumentNotValidException e) {
        e.printStackTrace();
        List<String> errorList = new ArrayList<>();
        e.getBindingResult().getAllErrors().forEach(
                objectError -> {
                    errorList.add(objectError.getDefaultMessage());
                }
        );
        String error = "";
        for (int i = 0; i < errorList.size(); i++) {
            error = StringUtils.join(error, errorList.get(i));
            if (i != errorList.size() - 1) {
                error = StringUtils.join(error, ",");
            }
        }
        return ResponseBeanBuilder.fail(error);
    }

    /**
     * 参数校验
     *
     * @param e
     * @return
     */
    @ResponseStatus(HttpStatus.OK)
    @ExceptionHandler({BindException.class})
    public ResponseBean<String> handleBindException(BindException e) {
        e.printStackTrace();
        List<String> errorList = new ArrayList<>();
        e.getBindingResult().getAllErrors().forEach(
                objectError -> {
                    errorList.add(objectError.getDefaultMessage());
                }
        );
        String error = "";
        for (int i = 0; i < errorList.size(); i++) {
            error = StringUtils.join(error, errorList.get(i));
            if (i != errorList.size() - 1) {
                error = StringUtils.join(error, ",");
            }
        }
        return ResponseBeanBuilder.fail(error);
    }


}
