package com.factory.base.config.mybatis;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.factory.base.entity.constant.CacheConstant;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;

@Slf4j
@Component
public class AutoFillObjectHandler implements MetaObjectHandler {

    //适配其他认证方案时候也可以采用本方法
    private Long getLoginUserId() {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (servletRequestAttributes == null) {
            return null;
        }
        HttpServletRequest request = servletRequestAttributes.getRequest();
        return (Long) request.getAttribute(CacheConstant.LOGIN_USER_ID);
    }

    private String getUsername() {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (servletRequestAttributes == null) {
            return null;
        }
        HttpServletRequest request = servletRequestAttributes.getRequest();
        return (String) request.getAttribute(CacheConstant.LOGIN_USER_NAME);
    }

    @Override
    public void insertFill(MetaObject metaObject) {
        this.strictInsertFill(metaObject, "createdTime", LocalDateTime.class, LocalDateTime.now());
        this.strictInsertFill(metaObject, "updatedTime", LocalDateTime.class, LocalDateTime.now());
        this.strictInsertFill(metaObject, "deleted", Integer.class, 0);
        if (getLoginUserId() != null) {
            this.strictInsertFill(metaObject, "createdUserId", Long.class, getLoginUserId());
            this.strictInsertFill(metaObject, "updatedUserId", Long.class, getLoginUserId());
        }
        if (getUsername() != null) {
            this.strictInsertFill(metaObject, "createdUsername", String.class, getUsername());
            this.strictInsertFill(metaObject, "updatedUsername", String.class, getUsername());
        }
    }

    @Override
    public void updateFill(MetaObject metaObject) {
        this.strictUpdateFill(metaObject, "updatedTime", LocalDateTime.class, LocalDateTime.now());
        if (getLoginUserId() != null) {
            this.strictUpdateFill(metaObject, "updatedUserId", Long.class, getLoginUserId());
        }
        if (getUsername() != null) {
            this.strictUpdateFill(metaObject, "updatedUsername", String.class, getUsername());
        }
    }
}
