package com.factory.base.entity.enumPo;

import java.time.*;
import java.time.temporal.TemporalAdjusters;
import java.util.function.Function;

public enum FrequencyType {
    //频次  日  周  月
    日(1,
            "每日",
            Function.identity(),
            Function.identity()),

    周(2,
            "每周",
            item -> item.with(TemporalAdjusters.previous(DayOfWeek.MONDAY)),
            item -> {
                LocalDate with = item.with(TemporalAdjusters.next(DayOfWeek.SUNDAY));
                return with;
            }),
    月(3,
            "每月",
            item -> item.with(TemporalAdjusters.firstDayOfMonth()),
            item -> item.with(TemporalAdjusters.lastDayOfMonth())),
    季度(4,
            "每季度",
            item -> {
                Month firstMonthOfQuarter = java.time.Month.of(item.getMonth().firstMonthOfQuarter().getValue());
                Month firstMonthOfNextQuarter = firstMonthOfQuarter.plus(3L);
                int yearOfNextQuarter = firstMonthOfQuarter.getValue() > 9 ? item.getYear() + 1 : item.getYear();
                return LocalDateTime.of(LocalDate.of(yearOfNextQuarter, firstMonthOfNextQuarter, 1), LocalTime.MIN).toLocalDate();
            },
            item -> {
                Month firstMonthOfQuarter = java.time.Month.of(item.getMonth().firstMonthOfQuarter().getValue());
                Month firstMonthOfNextQuarter = firstMonthOfQuarter.plus(5L);
                int yearOfNextQuarter = firstMonthOfQuarter.getValue() > 9 ? item.getYear() + 1 : item.getYear();
                return LocalDateTime.of(LocalDate.of(yearOfNextQuarter, firstMonthOfNextQuarter, firstMonthOfNextQuarter.maxLength()), LocalTime.MAX).toLocalDate();
            }),
    半年(5,
            "每半年",
            item -> {
                int lastHalfYear = (item.getMonthValue() > 6) ? item.getYear() : item.getYear() - 1;
                java.time.Month firstMonthOfLastHalfYear = (item.getMonthValue() > 6) ? java.time.Month.JANUARY : java.time.Month.JULY;
                LocalDateTime of = LocalDateTime.of(LocalDate.of(lastHalfYear, firstMonthOfLastHalfYear, 1), LocalTime.MIN);
                return of.toLocalDate();
            },
            item -> {
                int lastHalfYear = (item.getMonthValue() > 6) ? item.getYear() : item.getYear() - 1;
                Month firstMonthOfLastHalfYear = (item.getMonthValue() > 6) ? java.time.Month.JANUARY : java.time.Month.JULY;
                return LocalDateTime.of(LocalDate.of(lastHalfYear, firstMonthOfLastHalfYear, 1), LocalTime.MIN).toLocalDate();
            }),
    班(6,
            "班/次",
            Function.identity(),
            Function.identity());

    public final Integer value;
    public final String desc;
    public final Function getFirstDate;
    public final Function getEndDate;

    FrequencyType(Integer value, String desc, Function<LocalDate, LocalDate> getFirstDate, Function<LocalDate, LocalDate> getEndDate) {
        this.value = value;
        this.desc = desc;
        this.getFirstDate = getFirstDate;
        this.getEndDate = getEndDate;
    }


}
