package com.factory.base.util;


import com.itextpdf.text.Document;
import com.itextpdf.text.Image;
import com.itextpdf.text.Rectangle;
import com.itextpdf.text.pdf.*;
import com.lowagie.text.DocumentException;
import freemarker.template.Configuration;
import freemarker.template.Template;
import org.springframework.util.ResourceUtils;
import org.xhtmlrenderer.pdf.ITextRenderer;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.UUID;

public class PdfHandler {

    private static Configuration cfg = new Configuration(Configuration.VERSION_2_3_28);
    /**
     * 线上位置
     */
    private static final String SIM_SUN_TTC = "systemLib/font/simsun.ttc";
    private static final String ARIAL_UNI = "systemLib/font/arialuni.ttf";
    private static final String SIM_SUN = "systemLib/font/simsun.ttf";
    private static final String FTL_LOCATION = "systemLib/template/";

    private static final String PDF = ".pdf";
    private static final String HTML = ".html";
    private static final String FTL = ".ftl";

    static {
        cfg.setDefaultEncoding("UTF-8");
        try {
            File ftlDir = ResourceUtils.getFile(FTL_LOCATION);
            cfg.setDirectoryForTemplateLoading(ftlDir);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 生成pdf格式合同
     */
    public static File pdfContractHandler(
            String templateName,
            Map<String, Object> paramMap
    ) {
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        File pdfFilePath = null;
        // 判断本地路径是否存在如果不存在则创建
        try {
            // 组装html和pdf合同的全路径URL
            pdfFilePath = File.createTempFile(
                    templateName + uuid, PDF
            );
            File htmlFilePath = File.createTempFile(
                    templateName + uuid, HTML
            );
            templateName = templateName + FTL;
            htmHandler(templateName, htmlFilePath, paramMap);
            // 生成html合同
            pdfHandler(htmlFilePath, pdfFilePath);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return pdfFilePath;
    }


    public static File createTempPdf(
    ) {
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        File pdfFilePath = null;
        // 组装html和pdf合同的全路径URL
        try {
            pdfFilePath = File.createTempFile(
                    uuid, PDF
            );
        } catch (IOException e) {
            e.printStackTrace();
        }
        return pdfFilePath;
    }


    /**
     * 生成html格式
     */
    private static void htmHandler(String templateName, File outHtmFile, Map<String, Object> paramMap) {
        try {
            Template template = cfg.getTemplate(templateName);
            Writer out = new BufferedWriter(new OutputStreamWriter(
                    new FileOutputStream(outHtmFile), StandardCharsets.UTF_8));
            template.process(paramMap, out);
            out.close();
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    /**
     * 生成pdf格式
     */
    private static void pdfHandler(File htmFile, File pdfFile)
            throws DocumentException, IOException {
        String url = htmFile.toURI().toURL().toString();
        OutputStream os = new FileOutputStream(pdfFile);
        org.xhtmlrenderer.pdf.ITextRenderer renderer = new ITextRenderer();
        renderer.setDocument(url);
        org.xhtmlrenderer.pdf.ITextFontResolver fontResolver = renderer
                .getFontResolver();
        //加入字体支持
        //打jar包后 无法直接读取jar包中的文件 放项目同目录即可
        File simSunTtc = ResourceUtils.getFile(SIM_SUN_TTC);
        File arialUni = ResourceUtils.getFile(ARIAL_UNI);
        File simSun = ResourceUtils.getFile(SIM_SUN);
        //设置字体
        fontResolver.addFont(simSunTtc.getCanonicalPath(), BaseFont.IDENTITY_H, BaseFont.NOT_EMBEDDED);
        fontResolver.addFont(arialUni.getCanonicalPath(), BaseFont.IDENTITY_H, BaseFont.NOT_EMBEDDED);
        fontResolver.addFont(simSun.getCanonicalPath(), BaseFont.IDENTITY_H, BaseFont.NOT_EMBEDDED);
        renderer.layout();
        renderer.createPDF(os);
        os.close();
    }


    public static void convert(String source, String target) {
        Document document = new Document();
        // 设置文档页边距
        document.setMargins(0, 0, 0, 0);
        FileOutputStream fos = null;
        try {
            fos = new FileOutputStream(target);
            PdfWriter.getInstance(document, fos);
            // 打开文档
            document.open();
            // 获取图片的宽高
            Image image = Image.getInstance(source);
            float imageHeight = image.getScaledHeight();
            float imageWidth = image.getScaledWidth();
            // 设置页面宽高与图片一致
            Rectangle rectangle = new Rectangle(imageWidth, imageHeight);
            document.setPageSize(rectangle);
            // 图片居中
            image.setAlignment(Image.ALIGN_CENTER);
            // 新建一页添加图片
            document.newPage();
            document.add(image);
        } catch (Exception ioe) {
            System.out.println(ioe.getMessage());
        } finally {
            // 关闭文档
            document.close();
            try {
                fos.flush();
                fos.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public static boolean mergePdf(String[] files, String targetFile) {
        boolean retValue = false;
        Document document = null;
        try {
            document = new Document(new PdfReader(files[0]).getPageSize(1));
            PdfCopy copy = new PdfCopy(document, new FileOutputStream(targetFile));
            document.open();
            for (int i = 0; i < files.length; i++) {
                PdfReader reader = new PdfReader(files[i]);
                int n = reader.getNumberOfPages();
                for (int j = 1; j <= n; j++) {
                    document.newPage();
                    PdfImportedPage page = copy.getImportedPage(reader, j);
                    copy.addPage(page);
                }
            }
            retValue = true;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (document != null) {
                document.close();
            }
        }
        return retValue;
    }


}
