package com.factory.base.util.file;

import org.apache.commons.lang3.StringUtils;

import java.io.*;
import java.util.Arrays;
import java.util.Objects;
import java.util.UUID;


public class FileUtils {

    /**
     * @param inputStream
     * @param outputFilePath
     */
    public static void storeFileByInputStream(InputStream inputStream, String outputFilePath) {
        try {
            File targetFile = new File(outputFilePath);
            OutputStream outStream = new FileOutputStream(targetFile);
            byte[] buffer = new byte[1024 * 8];
            int bytesRead;
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outStream.write(buffer, 0, bytesRead);
            }
            outStream.flush();
            outStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * 防止文件重复
     *
     * @return
     */

    public static byte[] getFileBytes(File tradeFile) {
        byte[] buffer = null;
        try {
            FileInputStream fis = new FileInputStream(tradeFile);
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            byte[] b = new byte[1024];
            int n;
            while ((n = fis.read(b)) != -1) {
                bos.write(b, 0, n);
            }
            fis.close();
            bos.close();
            buffer = bos.toByteArray();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return buffer;
    }

    public static String getFileNo() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }

    /**
     * 获取后缀
     *
     * @return
     */
    public static String getSuffix(String fileFullName, String spliter) {
        if (StringUtils.isNotBlank(fileFullName)) {
            String[] split = fileFullName.split(spliter);
            if (split.length > 1) {
                return split[1];
            }
        }
        return "";
    }

    /**
     * 获取前缀
     *
     * @return
     */
    public static String getPrefix(String fileFullName, String spliter) {
        if (StringUtils.isNotBlank(fileFullName)) {
            String[] split = fileFullName.split(spliter);
            if (split.length > 1) {
                return split[0];
            }
        }
        return "";
    }

    public static void deleteDirFiles(String fileDirPath) {
        File file = new File(fileDirPath);
        if (file.isDirectory()) {
            Arrays.asList(Objects.requireNonNull(file.listFiles())).forEach(
                    File::delete
            );
        } else {
            file.delete();
        }
    }

    public static void mkdirs(String fileDirPath) {
        File file = new File(fileDirPath);
        if (!file.isDirectory()) {
            file.mkdirs();
        }
    }

    public static File createTempPdf(
            String suffix
    ) {
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        File pdfFilePath = null;
        // 组装html和pdf合同的全路径URL
        try {
            pdfFilePath = File.createTempFile(
                    uuid, suffix
            );
        } catch (IOException e) {
            e.printStackTrace();
        }
        return pdfFilePath;
    }


}
