package com.factory.base.util.time;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

public class TimeUtils {

    static Map<Integer, String> weekInfo = new HashMap<>();

    static {
        weekInfo.put(1, "周一");
        weekInfo.put(2, "周二");
        weekInfo.put(3, "周三");
        weekInfo.put(4, "周四");
        weekInfo.put(5, "周五");
        weekInfo.put(6, "周六");
        weekInfo.put(7, "周日");

    }

    public static String getTimeTypeNameByLocalTime(LocalTime time) {
        String timeType = "";
        if (time == null) {
            return timeType;
        }
        if (time.getHour() <= 8) {
            timeType = "早";
        }
        if (time.getHour() >= 8 && time.getHour() <= 11) {
            timeType = "上午";
        }
        if (time.getHour() >= 14 && time.getHour() <= 18) {
            timeType = "下午";
        }
        if (time.getHour() >= 19) {
            timeType = "晚";
        }
        return timeType;
    }


    public static String getWeekInfo(int week) {
        return weekInfo.get(week);
    }
    /**
     * 0为当前的时间
     * @param now
     * @param type
     * @param num
     * @return
     */
    public static LocalDate[] getStartAndEndDay(LocalDate now, int type, int num) {
        LocalDate[] se = new LocalDate[2];
        if (type == 1) {//周一到周日
            int week = now.getDayOfWeek().getValue();
            se[0] = now.minusDays(week - 1);
            se[1] = now.plusDays(7-week).plusWeeks(num);
        } else if (type == 2) {//月初到月末
            se[0] = now.with(TemporalAdjusters.firstDayOfMonth());
            se[1] = now.with(TemporalAdjusters.lastDayOfMonth()).plusMonths(num);
        }
        return se;
    }
    /**
     * 0为当前的时间
     * @param now
     * @param type
     * @param num
     * @return
     */
    public static LocalDateTime[] getStartAndEndDayTime(LocalDateTime now, int type, int num) {
        LocalDateTime[] se = new LocalDateTime[2];
        if (type == 1) {//日
            se[0] = LocalDateTime.of(now.toLocalDate(),LocalTime.MIN);
            se[1] =  LocalDateTime.of(now.toLocalDate(),LocalTime.MAX);
        } else if (type == 2) {//周
        	int week = now.getDayOfWeek().getValue();
            se[0] = LocalDateTime.of(now.minusDays(week - 1).toLocalDate(),LocalTime.MIN);
            se[1] = LocalDateTime.of(now.plusDays(7-week+num*7).toLocalDate(),LocalTime.MAX);
        } else if (type == 3) {//月
            se[0] = LocalDateTime.of(now.with(TemporalAdjusters.firstDayOfMonth()).toLocalDate(),LocalTime.MIN);
            se[1] = LocalDateTime.of(now.with(TemporalAdjusters.lastDayOfMonth()).plusMonths(num).toLocalDate(),LocalTime.MAX);
        }
        return se;
    }
    
    public static LocalDateTime[] getMonthStartAndEnd(String monthStr) {
    	LocalDateTime n = null;
    	try {
    		n = paserMonthString(monthStr);
    	}catch (Exception e) {
			n = LocalDateTime.now();
		}
        return getStartAndEndDayTime(n, 3, 0);
  }

    public static String formatLocalTime(LocalTime time) {
        if (time == null) {
            return "";
        }
        return time.format(timeFormatter);
    }
    
    public static String formatLocalDate(LocalDate date) {
        if (date == null) {
            return "";
        }
        return date.format(dayFormatter);
    }

    public static String getDayString(LocalDateTime now) {
        return now.format(dayFormatter);
    }

    public static String getMonthString(LocalDateTime now) {
        return now.format(monthFormatter);
    }

    public static String getDayTimeString(LocalDateTime now) {
        return now.format(dayTimeFormatter);
    }

    public static LocalTime paserLocalTime(String time) {
        if (time == null) {
            return null;
        }
        return LocalTime.parse(time, timeFormatter);
    }

    public static LocalDateTime paserDayString(String now) {
        return LocalDate.parse(now, dayFormatter).atStartOfDay();
    }

    public static LocalDateTime paserMonthString(String now) {
        return LocalDate.parse(now + "-01", dayFormatter).atStartOfDay();
    }

    public static LocalDateTime paserDayTimeString(String now) {
        return LocalDateTime.parse(now, dayTimeFormatter);
    }

    public static DateTimeFormatter dayTimeFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
    public static DateTimeFormatter dayFormatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
    public static DateTimeFormatter timeFormatter = DateTimeFormatter.ofPattern("HH:mm:ss");
    public static DateTimeFormatter monthFormatter = DateTimeFormatter.ofPattern("yyyy-MM");
    public static DateTimeFormatter timeMiniteFormatter = DateTimeFormatter.ofPattern("HH:mm");

    public static void main(String[] args) {
        LocalDateTime now = LocalDateTime.now();
        LocalDate now2 = LocalDate.now();
        System.out.println(Arrays.toString(getStartAndEndDay(now2, 1, 0)));
        System.out.println(Arrays.toString(getStartAndEndDay(now2, 2, 0)));
        System.out.println(Arrays.toString(getStartAndEndDayTime(now, 1, 0)));
        System.out.println(Arrays.toString(getStartAndEndDayTime(now, 2, 0)));
        System.out.println(Arrays.toString(getStartAndEndDayTime(now, 3, 0)));
        System.out.println(now.getDayOfMonth());
        System.out.println(now.getDayOfWeek().getValue());
        System.out.println(getMonthString(now));
        System.out.println(TimeUtils.paserMonthString("2021-05"));
    }
}
