package com.factory.user.config;

import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.aggregates.ResponseEnum;
import com.factory.base.entity.constant.RemoteCommonConstants;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.service.AuthService;
import com.factory.user.util.AuthUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author 认证拦截器
 * description
 * createDate     2019/4/15 14:54
 * updateRemark
 * version        1.0
 */

@Slf4j
@Component
public class AuthenticationInterceptor implements HandlerInterceptor {

    @Resource
    private AuthenticationProperties authenticationProperties;
    @Resource
    private AuthService authService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        if (request.getMethod().equals(HttpMethod.OPTIONS.name())) {
            response.setStatus(HttpServletResponse.SC_OK);
            return true;
        }
        String token = request.getHeader(authenticationProperties.getHeaderName());

        //判断token是否存在
        if (StringUtils.isBlank(token)) {
            token = request.getParameter(authenticationProperties.getHeaderName());
        }

        if (StringUtils.isBlank(token)) {
            AuthUtils.writeJson(response, ResponseBeanBuilder.fail(ResponseEnum.UN_AUTHENTICATION, authenticationProperties.getLoginUrl()));
            return false;
        }

        //认证token是否合法
        ResponseBean<Long> checkResult = authService.checkToken(token);
        String checkToken = checkResult.getStatus();
        if (ResponseEnum.UN_AUTHENTICATION.getStatus().equals(checkToken)) {
            AuthUtils.writeJson(response, ResponseBeanBuilder.fail(ResponseEnum.UN_AUTHENTICATION, authenticationProperties.getLoginUrl()));
            return false;
        }
        if (ResponseEnum.AUTHENTICATION_EXPIRE.getStatus().equals(checkToken)) {
            AuthUtils.writeJson(response, ResponseBeanBuilder.fail(ResponseEnum.AUTHENTICATION_EXPIRE));
            return false;
        }
        request.setAttribute("UserId", checkResult.getData());
        //管理员不校验权限
        if (RemoteCommonConstants.ADMIN.equals(checkResult.getData())) {
            return true;
        }


        //动态鉴权
        boolean checkPermission = authService.checkPermission(request.getRequestURI(), authenticationProperties.getClient(), token, checkResult.getData());
        if (!checkPermission) {
            AuthUtils.writeJson(response, ResponseBeanBuilder.fail(ResponseEnum.UNAUTHORIZED));
            return false;
        }
        return true;
    }


}
