package com.factory.user.config;

import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.aggregates.ResponseEnum;
import com.factory.base.entity.constant.CacheConstant;
import com.factory.user.service.AuthService;
import com.factory.user.service.AuthUserInfoService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * token转userid并存放于请求域
 */
@Slf4j
@Component
public class TokenUserInfoInterceptor implements HandlerInterceptor {

    @Resource
    private AuthenticationProperties authenticationProperties;
    @Resource
    private AuthService authService;
    @Resource
    private AuthUserInfoService authUserInfoService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) {
        if (request.getMethod().equals(HttpMethod.OPTIONS.name())) {
            response.setStatus(HttpServletResponse.SC_OK);
            return true;
        }
        String token = request.getHeader(authenticationProperties.getHeaderName());
        //判断token是否存在
        if (StringUtils.isBlank(token)) {
            token = request.getParameter(authenticationProperties.getHeaderName());
        }
        //认证token是否合法
        if (StringUtils.isNotBlank(token)) {
            ResponseBean<Long> checkResult = authService.checkToken(token);
            if (ResponseEnum.OK.getStatus().equals(checkResult.getStatus())) {
                //后续修改为缓存
                Long loginUserId = checkResult.getData();
                String username = authUserInfoService.getNameById(loginUserId);
                request.setAttribute(CacheConstant.LOGIN_USER_ID, loginUserId);
                request.setAttribute(CacheConstant.LOGIN_USER_NAME, username);
            }
        }
        return true;
    }


}
