package com.factory.user.config.handler;


import com.factory.base.config.exception.CustomException;
import com.factory.base.entity.constant.CacheConstant;
import com.factory.user.config.AuthenticationProperties;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

@Component
public class LoginContext {

    @Resource
    private AuthenticationProperties authenticationProperties;

    public Long getLoginUserId() {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (servletRequestAttributes == null) {
            throw new CustomException("必须为请求的方式");
        }
        HttpServletRequest request = servletRequestAttributes.getRequest();
        Long userId = (Long) request.getAttribute(CacheConstant.LOGIN_USER_ID);
        if (userId == null) {
            throw new CustomException("未获得认证信息,请重新登陆");
        }
        return userId;
    }

    public String getUsername() {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (servletRequestAttributes == null) {
            throw new CustomException("必须为请求的方式");
        }
        HttpServletRequest request = servletRequestAttributes.getRequest();
        return (String) request.getAttribute(CacheConstant.LOGIN_USER_NAME);
    }

    public String getLoginToken() {
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (servletRequestAttributes == null) {
            throw new CustomException("必须为请求的方式");
        }
        HttpServletRequest request = servletRequestAttributes.getRequest();
        String token = request.getHeader(authenticationProperties.getHeaderName());
        if (StringUtils.isBlank(token)) {
            token = request.getParameter(authenticationProperties.getHeaderName());
            if (StringUtils.isBlank(token)) {
                throw new CustomException("未获得认证信息,请重新登陆");
            }
        }
        return token;
    }

}
