package com.factory.user.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.HttpStatus;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.constant.RemoteCommonConstants;
import com.factory.base.entity.po.remote.*;
import com.factory.base.entity.vo.BaseSelectVo;
import com.factory.base.entity.vo.UserInfoVo;
import com.factory.base.util.BeanCopyUtil;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.config.AuthenticationProperties;
import com.factory.user.config.handler.LoginContext;
import com.factory.user.entity.vo.res.FileRes;
import com.factory.user.entity.vo.res.RoleListRes;
import com.factory.user.entity.vo.res.UserMenuRes;
import com.factory.user.mapper.DeptMapper;
import com.factory.user.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Slf4j
@Service
public class CommonServiceImpl implements CommonService {
    @Resource
    private DeptMapper deptMapper;
    @Resource
    private LoginContext loginContext;
    @Resource
    private UserInfoService userInfoService;
    @Resource
    private DeptService deptService;
    @Resource
    private DictService dictService;
    @Resource
    private MenuService menuService;
    @Resource
    private UserRoleService userRoleService;
    @Resource
    private RoleService roleService;
    @Resource
    private RolePermissionService rolePermissionService;
    @Resource
    private PermissionService permissionService;
    @Autowired
    private AuthenticationProperties authenticationProperties;


    @Override
    public ResponseBean<List<BaseSelectVo>> getDept() {
        LambdaQueryWrapper<Dept> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Dept::getPId, 0);
        queryWrapper.eq(Dept::getDeleted, 0);
        List<Dept> all = deptMapper.selectList(queryWrapper);
        return ResponseBeanBuilder.ok(all, item -> {
            BaseSelectVo baseSelectVo = new BaseSelectVo();
            baseSelectVo.setId(item.getId());
            baseSelectVo.setName(item.getName());
            return baseSelectVo;
        });
    }

    @Override
    public ResponseBean<List<BaseSelectVo>> getDeptCode() {
        LambdaQueryWrapper<Dept> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(Dept::getPId, 0);
        queryWrapper.eq(Dept::getDeleted, 0);
        List<Dept> all = deptMapper.selectList(queryWrapper);
        List<Dept> collect = all.stream().filter(item -> {
            if (item.getDeptCode() != null) {
                return true;
            }
            return false;
        }).collect(Collectors.toList());
        return ResponseBeanBuilder.ok(collect, item -> {
            BaseSelectVo baseSelectVo = new BaseSelectVo();
            baseSelectVo.setId(item.getDeptCode());
            baseSelectVo.setName(item.getName());
            return baseSelectVo;
        });
    }


    @Override
    public UserInfoVo getLoginUserInfo() {
        Long loginUserId = loginContext.getLoginUserId();
        UserInfo userInfo = userInfoService.getById(loginUserId);
        if (userInfo == null) {
            return null;
        }
        UserInfoVo userInfoVo = new UserInfoVo();
        userInfoVo.setId(userInfo.getId());
        userInfoVo.setDepartmentId(userInfo.getDeptId());
        userInfoVo.setDepartmentName(deptService.getDeptNameById(userInfo.getDeptId()));
        userInfoVo.setShiftId(userInfo.getShiftId());
        userInfoVo.setShift(dictService.getNameById(userInfoVo.getShiftId()));
        userInfoVo.setName(userInfo.getName());
        userInfoVo.setNo(userInfo.getNo());
        userInfoVo.setRemark(userInfo.getRemark());
        userInfoVo.getMenuList().addAll(menuService.list());
        List<UserRole> list = userRoleService
                .list(Wrappers.<UserRole>lambdaQuery().eq(UserRole::getUserId, loginUserId));
        if (list.size() > 0) {
            userInfoVo.getRoleList().addAll(roleService.list(Wrappers.<Role>lambdaQuery().in(Role::getId,
                    list.stream().map(UserRole::getRoleId).distinct().collect(Collectors.toList()))));
        }
        return userInfoVo;
    }

    @Override
    public ResponseBean<List<UserMenuRes>> userMenu() {
        Long loginUserId = loginContext.getLoginUserId();
        if (loginUserId != null) {
            // 判断是否admin
            if (RemoteCommonConstants.ADMIN.equals(loginUserId)) {
                // 如果是admin直接跳过验证返回按钮
                List<Permission> permissions = permissionService
                        .list(Wrappers.<Permission>lambdaQuery().eq(Permission::getModuleId, 511));
                List<UserMenuRes> userMenuRes = BeanCopyUtil.copyListProperties(permissions, UserMenuRes::new);
                return ResponseBeanBuilder.ok(userMenuRes);
            } else {
                // 查询当前账号所有角色
                List<UserRole> list = userRoleService
                        .list(Wrappers.<UserRole>lambdaQuery().eq(UserRole::getUserId, loginUserId));
                // 取出所有角色id
                List<Long> longList = list.stream().map(UserRole::getRoleId).collect(Collectors.toList());
                if (CollUtil.isNotEmpty(longList)) {
                    // 查询角色下所有权限
                    List<RolePermission> rolePermissions = rolePermissionService
                            .list(Wrappers.<RolePermission>lambdaQuery().in(RolePermission::getRoleId, longList));
                    // 遍历出所有符合权限的id
                    List<Long> permissionIds = rolePermissions.stream().map(x -> {
                        if (x.getPermissionId() == 1504) {
                            return x.getPermissionId();
                        } else if (x.getPermissionId() == 1505) {
                            return x.getPermissionId();
                        } else if (x.getPermissionId() == 1506) {
                            return x.getPermissionId();
                        } else if (x.getPermissionId() == 1507) {
                            return x.getPermissionId();
                        } else if (x.getPermissionId() == 1508) {
                            return x.getPermissionId();
                        }
                        return null;
                    }).collect(Collectors.toList());
                    // 查询权限id下的名称及路径
                    if (CollUtil.isNotEmpty(permissionIds)) {
                        List<Permission> permissions = permissionService.listByIds(permissionIds);
                        List<UserMenuRes> userMenuRes = BeanCopyUtil.copyListProperties(permissions, UserMenuRes::new);
                        return ResponseBeanBuilder.ok(userMenuRes);
                    }
                }
            }
        }
        return ResponseBeanBuilder.fail("用户id为空,重新登陆");
    }


    @Override
    public ResponseBean<List<RoleListRes>> roleList() {
        List<Role> list = roleService.list(Wrappers.<Role>lambdaQuery().eq(Role::getType, DictKey.JH));
        List<RoleListRes> roleListRes = BeanCopyUtil.copyListProperties(list, RoleListRes::new);
        return ResponseBeanBuilder.ok(roleListRes);
    }

    @Override
    public String selectUserName(Long id) {
        if (id != null) {
            UserInfo byId = userInfoService.getOne(Wrappers.<UserInfo>lambdaQuery().eq(UserInfo::getId, id).select(UserInfo::getUsername));
            return Optional.ofNullable(byId).map(item -> item.getUsername()).orElse("");
        }
        return "";
    }

    @Override
    public List<UserInfo> selectUserByRole(List<Long> roleList) {
        return userInfoService.getUserByRole(roleList);
    }

    @Override
    public ResponseBean<FileRes> exportExcel(File tempFile) {
        FileRes fileInfo = null;
        HashMap<String, Object> fileUploadMap = new HashMap<>();
        fileUploadMap.put("file", tempFile);
        try {
            log.debug(">>>>>>>>>>{}<<<<<", "http://127.0.0.1:40045/file/file/uploadOne");
            HttpResponse response = HttpRequest.post("http://127.0.0.1:40045/file/file/uploadOne")
                    .form(fileUploadMap)
                    .header("content-type", "multipart/form-data")
                    .header(authenticationProperties.getHeaderName(), "")
                    .execute();
            if (response.getStatus() == HttpStatus.HTTP_OK) {
                fileInfo = JSONObject.toJavaObject(JSON.parseObject(response.body()).getJSONObject("data"), FileRes.class);
            }
        } catch (Exception e) {
            log.error("下载文件保错", e);
            return ResponseBeanBuilder.fail("获取文件服务失败,请重试");

        }
        return ResponseBeanBuilder.ok(fileInfo);
    }


}
