package com.factory.user.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.factory.base.entity.aggregates.ResponseEnum;
import com.factory.base.entity.constant.RemoteCommonConstants;
import com.factory.base.entity.constant.RemoteRedisConstants;
import com.factory.base.entity.po.remote.Permission;
import com.factory.base.entity.po.remote.RolePermission;
import com.factory.base.entity.po.remote.UserRole;
import com.factory.user.entity.po.token.RefreshTokenVal;
import com.factory.user.entity.po.token.TokenType;
import com.factory.user.entity.po.token.TokenVal;
import com.factory.user.entity.po.token.TokenVo;
import com.factory.user.mapper.PermissionsMapper;
import com.factory.user.mapper.RolePermissionsMapper;
import com.factory.user.mapper.UserRolesMapper;
import com.factory.user.service.TokenService;
import com.factory.user.util.AESUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.ObjectUtils;

import java.util.List;
import java.util.stream.Collectors;

@Service
@Transactional
public class TokenServiceImpl implements TokenService {

    @Value("${token.expire.long.time}")
    private String longExpTime;
    //private String longExpTime = "172800";
    //
    @Value("${token.expire.short.time}")
    private String shortExpTime;
    //private String shortExpTime ="345600";

    @Value("${token.expire.long.refreshTime}")
    private String longRefreshTime;
    //private String longRefreshTime = "7200";

    @Autowired
    private StringRedisTemplate stringRedisTemplate;
    @Autowired
    private UserRolesMapper userRolesMapper;
    @Autowired
    private PermissionsMapper permissionMapper;
    @Autowired
    private RolePermissionsMapper rolePermissionsMapper;
    @Autowired
    private RedisTemplate<Object, Object> redisTemplate;


    private String createRefreshToken(String client, Long userId, long refreshTokenExp, String token) {
        String refreshTokenKey = AESUtil.encrypt(client + userId);
        RefreshTokenVal refreshTokenVal = new RefreshTokenVal(client, userId, refreshTokenExp, token);
        this.redisTemplate.opsForHash().put(RemoteRedisConstants.REFRESH_TOKEN_HASH_NAME, refreshTokenKey, refreshTokenVal);
        return refreshTokenKey;
    }

    @Override
    public TokenVo getToken(String client, Long userId) {
        String key = client + userId;
        Boolean hasToken = stringRedisTemplate.opsForHash().hasKey(RemoteRedisConstants.USER_ID_HASH_NAME, key);
        if (!ObjectUtils.isEmpty(hasToken) && hasToken) {
            String tokenKey = String.valueOf(stringRedisTemplate.opsForHash().get(RemoteRedisConstants.USER_ID_HASH_NAME, key));
            TokenVal tokenVal = (TokenVal) redisTemplate.opsForHash().get(RemoteRedisConstants.TOKEN_HASH_NAME, tokenKey);
            if (TokenType.LONG_EXPIRED_TOKEN.toString().equals(client)) {
                //TODO 删除旧的refreshToken
                redisTemplate.opsForHash().delete(RemoteRedisConstants.REFRESH_TOKEN_HASH_NAME, tokenVal.getRefreshToken());
                //TODO 删除旧的token
                redisTemplate.opsForHash().delete(RemoteRedisConstants.TOKEN_HASH_NAME, tokenKey);
                //TODO 删除旧的 USER_ID_token
                stringRedisTemplate.opsForHash().delete(RemoteRedisConstants.USER_ID_HASH_NAME, key);
            } else {
                if (System.currentTimeMillis() - tokenVal.getExpireTime() >= Long.parseLong(shortExpTime) * 1000) {
                    redisTemplate.opsForHash().delete(RemoteRedisConstants.TOKEN_HASH_NAME, tokenKey);
                    return createToken(client, userId);
                } else {
                    tokenVal.setExpireTime(System.currentTimeMillis());
                    redisTemplate.opsForHash().put(RemoteRedisConstants.TOKEN_HASH_NAME, tokenKey, tokenVal);
                    return new TokenVo(tokenKey, null);
                }
            }
        }
        return createToken(client, userId);
    }

    @Override
    public TokenVo createToken(String client, Long userId) {
        TokenVo vo = new TokenVo();
        client = client.toUpperCase();
        long tokenExp = System.currentTimeMillis();
        ;
        String tokenKey = AESUtil.encrypt(client + userId + tokenExp);
        vo.setToken(tokenKey);
        TokenVal tokenVal = new TokenVal();
        tokenVal.setClient(client);
        tokenVal.setUserId(userId);
        tokenVal.setExpireTime(tokenExp);
        if (TokenType.LONG_EXPIRED_TOKEN.toString().equals(client)) {
            String refreshToken = createRefreshToken(client, userId, tokenExp, tokenKey);
            vo.setRefreshToken(refreshToken);
            tokenVal.setRefreshToken(refreshToken);
        }
        redisTemplate.opsForHash().put(RemoteRedisConstants.TOKEN_HASH_NAME, tokenKey, tokenVal);
        stringRedisTemplate.opsForHash().put(RemoteRedisConstants.USER_ID_HASH_NAME, client + userId, tokenKey);
        return vo;
    }

    @Override
    public int checkPermission(Long userId, String url, String client) {
        List<UserRole> userRoles =
                userRolesMapper.selectList(Wrappers.<UserRole>lambdaQuery().eq(UserRole::getUserId, userId));
        List<Long> roleIdList = userRoles.stream().map(UserRole::getRoleId).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(roleIdList)) {
            if (roleIdList.contains(RemoteCommonConstants.ROLE_ADMIN)) {
                return RemoteCommonConstants.NOT_DEL;
            }
            List<RolePermission> rolePermissions = rolePermissionsMapper.selectList(Wrappers.<RolePermission>lambdaQuery().in(RolePermission::getRoleId, roleIdList));
            List<Long> permissionIds = rolePermissions.stream().map(RolePermission::getPermissionId).collect(Collectors.toList());
            if (CollUtil.isNotEmpty(permissionIds)) {
                List<Permission> permissions = permissionMapper.selectBatchIds(permissionIds);
                List<String> urlList = permissions.stream().map(Permission::getUrl).collect(Collectors.toList());
                if (!CollectionUtils.isEmpty(urlList) && urlList.contains(url)) {
                    return RemoteCommonConstants.NOT_DEL;
                }
            }
        }
        return Integer.valueOf(ResponseEnum.USER_NOT_PERMISSION.getStatus());
    }

}
