package com.factory.user.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.factory.base.entity.aggregates.ResponseBean;
import com.factory.base.entity.aggregates.ResponseEnum;
import com.factory.base.entity.constant.RemoteCommonConstants;
import com.factory.base.entity.po.remote.Role;
import com.factory.base.entity.po.remote.UserInfo;
import com.factory.base.util.res.ResponseBeanBuilder;
import com.factory.user.entity.po.token.TokenType;
import com.factory.user.entity.po.token.TokenVo;
import com.factory.user.entity.vo.LoginVo;
import com.factory.user.entity.vo.UserVo;
import com.factory.user.mapper.UsersMapper;
import com.factory.user.service.TokenService;
import com.factory.user.service.UsersService;
import com.factory.user.util.Base64;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
@Transactional
public class UsersServiceImpl extends ServiceImpl<UsersMapper, UserInfo> implements UsersService {

    @Autowired
    private TokenService tokenService;


    @Override
    public ResponseBean<LoginVo> login(String client, String username, String password) {
        client = client.toUpperCase();
        String psd = Base64.encode(password.getBytes());
        UserVo userVo = this.baseMapper.getUserByUsername(username);

        if (null == userVo) {
            return ResponseBeanBuilder.fail(ResponseEnum.USERNAME_NOT_FOND.getMessage());
        }
        //TODO 暂时注掉
//        if(BeanUtil.isEmpty(userVo)) {
//           return ResponseBeanBuilder.fail(ResponseEnum.USERNAME_NOT_FOND.getMessage());
//        }
        if (userVo.isLock()) {
            return ResponseBeanBuilder.fail(ResponseEnum.USER_STOP.getMessage());
        }
        if (!psd.equals(userVo.getPassword())) {
            return ResponseBeanBuilder.fail(ResponseEnum.PASSWORD_FILE.getMessage());
        }
        if (!RemoteCommonConstants.ADMIN.equals(userVo.getId())) {
            if (TokenType.SHORT_EXPIRED_TOKEN.toString().equals(client)) {
                int code = tokenService.checkPermission(userVo.getId(), "login", client);
                if (code != RemoteCommonConstants.NOT_DEL) {
                    return ResponseBeanBuilder.fail(ResponseEnum.USER_NOT_PERMISSION.getMessage());
                }
            }
        }
        TokenVo tokenVo = this.tokenService.getToken(client, userVo.getId());
        LoginVo loginVo = new LoginVo(userVo.getNo(), userVo.getUsername(), userVo.getDeptName(),
                tokenVo.getToken(), tokenVo.getRefreshToken(), userVo.getId(), "");

        List<Role> list = this.baseMapper.getUserRole(userVo.getId());
        String roleName = list.stream().map(r -> r.getName()).collect(Collectors.joining());
        loginVo.setRoleName(roleName);
        return ResponseBeanBuilder.ok(loginVo);
    }

    @Override
    public ResponseBean<LoginVo> login(String client, Long userId) {
        client = client.toUpperCase();
        UserVo userVo = this.baseMapper.getUserById(userId);

        if (BeanUtil.isEmpty(userVo)) {
            return ResponseBeanBuilder.fail(ResponseEnum.USERNAME_NOT_FOND.getMessage());
        }
        if (userVo.isLock()) {
            return ResponseBeanBuilder.fail(ResponseEnum.USER_STOP.getMessage());
        }
        if (!RemoteCommonConstants.ADMIN.equals(userVo.getId())) {
            if (TokenType.SHORT_EXPIRED_TOKEN.toString().equals(client)) {
                int code = tokenService.checkPermission(userVo.getId(), "login", client);
                if (code != RemoteCommonConstants.NOT_DEL) {
                    return ResponseBeanBuilder.fail(ResponseEnum.USER_NOT_PERMISSION.getMessage());
                }
            }
        }
        TokenVo tokenVo = this.tokenService.getToken(client, userVo.getId());
        LoginVo loginVo = new LoginVo(userVo.getNo(), userVo.getUsername(), userVo.getDeptName(), tokenVo.getToken(),
                tokenVo.getRefreshToken(), userVo.getId(), "");

        List<Role> list = this.baseMapper.getUserRole(userVo.getId());
        String roleName = list.stream().map(r -> r.getName()).collect(Collectors.joining());
        loginVo.setRoleName(roleName);
        return ResponseBeanBuilder.ok(loginVo);
    }
}
